var plugin = {},
  request = require('request'),
//  testUtils = require("../../../test/testUtils"),
  plugins = require('../../pluginManager.js'),
  crypto = require('crypto'),
  countlyStats = require('../../../api/parts/data/stats.js'),
  countlyConfig = require('../../../frontend/express/config'),
  versionInfo = require('../../../frontend/express/version.info'),
  COUNTLY_VERSION = versionInfo.version,
  COUNTLY_TYPE = versionInfo.type;

var countlyDb = plugins.dbConnection(countlyConfig);

function sha1Hash(str, addSalt) {
    var salt = (addSalt) ? new Date().getTime() : "";
    return crypto.createHmac('sha1', salt + "").update(str + "").digest('hex');
}

function md5Hash(str) {
    return crypto.createHash('md5').update(str + "").digest('hex');
}

plugins.setConfigs("security", {
    login_tries: 10,
    login_wait: 60,
    password_min: 1,
    password_char: false,
    password_number: false,
    password_symbol: false,
    password_expiration: 0,
    dashboard_additional_headers: "X-Frame-Options:deny\nX-XSS-Protection:1; mode=block\nStrict-Transport-Security:max-age=31536000 ; includeSubDomains",
    api_additional_headers: "X-Frame-Options:deny\nX-XSS-Protection:1; mode=block"
});

(function (plugin) {
	  plugin.renderDashboard = function(params){
		  params.data.countlyGlobal.aomAppId = params.req.session.aomApp;
		  delete params.req.session.aomApp;
	  };
	
	
	  plugin.init = function(app, countlyDb, express){
		  
		 app.get(countlyConfig.path+'/aomUrl', function(req, res, next) {
			  res.json({url : countlyConfig.aomDashboardUrl, countlyPath: countlyConfig.path});
		 });
		  
		 
	    //add your middleware or process requests here
	    app.get(countlyConfig.path+'/aomlogin', function(req, res, next) {
	    	/*var username = decodeURIComponent(req.param('username',''));
	    	var password = decodeURIComponent(req.param('key',''));
	    	var lang = decodeURIComponent(req.param('lang','en'));
	    	
	    	req.session.aomApp = req.param('appId','');*/
            
            if(!req.query.appId)req.query.appId = "";
            if(!req.query.username)req.query.username = "";
            if(!req.query.key)req.query.key = "";
            if(!req.query.lang)req.query.lang = "en";
            
            req.session.aomApp = req.query.appId;
            var username = decodeURIComponent(req.query.username);
	    	var password = decodeURIComponent(req.query.key);
	    	var lang = decodeURIComponent(req.query.lang);
            
	    	if (req.session) {
	            req.session.uid = null;
	            req.session.gadm = null;
	            req.session.email = null;
	            res.clearCookie('uid');
	            res.clearCookie('gadm');
	        }
	    	
	        if (username !== '' && password !== '') {
	            countlyDb.collection('members').findOne({$or: [ {"username":username}, {"email":username} ], "password":password}, function (err, member) {
	                if (member) {
	                    plugins.callMethod("loginSuccessful", {req:req, res:res, next:next, data:member});
	                    if (countlyConfig.web.use_intercom && member['global_admin']) {
	                        countlyStats.getOverall(countlyDb, function(statsObj){
	                            request({
	                                uri:"https://cloud.count.ly/s",
	                                method:"POST",
	                                timeout:4E3,
	                                json:{
	                                    email:member.email,
	                                    full_name:member.full_name,
	                                    v:COUNTLY_VERSION,
	                                    t:COUNTLY_TYPE,
	                                    u:statsObj["total-users"],
	                                    e:statsObj["total-events"],
	                                    a:statsObj["total-apps"],
	                                    m:statsObj["total-msg-users"],
	                                    mc:statsObj["total-msg-created"],
	                                    ms:statsObj["total-msg-sent"]
	                                }
	                            }, function(a, c, b) {
	                                a = {};
	                                b && (b.in_user_id && !member.in_user_id && (a.in_user_id = b.in_user_id), b.in_user_hash && !member.in_user_hash && (a.in_user_hash = b.in_user_hash));
	                                Object.keys(a).length && countlyDb.collection("members").update({_id:member._id}, {$set:a}, function() {})
	                            });
	                        });
	                    }

	                    req.session.uid = member["_id"];
	                    req.session.gadm = (member["global_admin"] == true);
	    				req.session.email = member["email"];
	                    if(lang !== '' && lang != member["lang"]){
	                        countlyDb.collection('members').update({_id:member["_id"]}, {$set:{lang: lang}}, function(){});
	                    }
	    				if(plugins.getConfig("frontend").session_timeout)
	    					req.session.expires = Date.now()+plugins.getConfig("frontend").session_timeout;
	                    res.redirect(countlyConfig.path+'/dashboard');
	                } else {
	                    plugins.callMethod("loginFailed", {req:req, res:res, next:next, data:req.body});
	                    res.redirect(countlyConfig.path+'/login'); 
	                }
	            });
	        } else {
                res.redirect(countlyConfig.path+'/login');
	        }

	    });
	    
	    
	    app.get(countlyConfig.path+'/aomsetup', function (req, res, next) {
	        countlyDb.collection('members').count({}, function (err, memberCount) {
	            if (memberCount) {
	            	//member already exists
	            	res.json({error : true, errorCode: 1});
	            } else {
                    if(!req.query.full_name)req.query.full_name = "";
                    if(!req.query.username)req.query.username = "";
                    if(!req.query.password)req.query.password = "";
                    if(!req.query.email)req.query.email = "";
                    if(!req.query.lang)req.query.lang = "en";
                    
                    var full_name = decodeURIComponent(req.query.full_name);
                    var username = decodeURIComponent(req.query.username);
                    var password = decodeURIComponent(req.query.password);
                    var email = decodeURIComponent(req.query.email);
                    var lang = decodeURIComponent(req.query.lang);
	    	    	
	    	        if (full_name !== '' && username !== '' && password !== '' && email !== '') {
	                    password = sha1Hash(password);
	                    
	                    var doc = {"full_name": full_name, "username": username, "password": password, "email": email, "global_admin":true, "global_admin":true, created_at: Math.floor(((new Date()).getTime()) / 1000), password_changed: Math.floor(((new Date()).getTime()) / 1000)};
	                    if(lang !== '')
	                        doc.lang = lang;
	                    countlyDb.collection('members').insert(doc, {safe:true}, function (err, member) {
	                        member = member.ops;
	                        if (countlyConfig.web.use_intercom) {
	                            var options = {uri:"https://cloud.count.ly/s", method:"POST", timeout:4E3, json:{email:email, full_name: full_name, v:COUNTLY_VERSION, t:COUNTLY_TYPE}};
	                            request(options, function(a, c, b) {
	                                a = {};
	                                a.api_key = md5Hash(member[0]._id + (new Date).getTime());
	                                b && (b.in_user_id && (a.in_user_id = b.in_user_id), b.in_user_hash && (a.in_user_hash = b.in_user_hash));

	                                countlyDb.collection("members").update({_id:member[0]._id}, {$set:a}, function(err, mem) {
	                                    plugins.callMethod("setup", {req:req, res:res, next:next, data:member[0]});
	                                    req.session.uid = member[0]._id;
	                                    req.session.gadm = !0;
	                                    req.session.email = member[0].email;
	                                    req.session.install = true;
	                                    //registration successful
	                                    res.json({error : false, username: member[0].username, email: member[0].email, api_key: a.api_key, _id: member[0]._id});
	                                })
	                            });
	                        } else {
	                            a = {};
	                            a.api_key = md5Hash(member[0]._id + (new Date).getTime());

	                            countlyDb.collection("members").update({_id:member[0]._id}, {$set:a}, function() {
	                                req.session.uid = member[0]._id;
	                                req.session.gadm = !0;
	                                req.session.email = member[0].email;
	                                req.session.install = true;
                                    //registration successful
                                    res.json({error : false, username: member[0].username, email: member[0].email, api_key: a.api_key, _id: member[0]._id});
	                            })
	                        }
	                    });
	                } else {
	                	//formular invalid
	                	res.json({error : true, errorCode: 2});
	                }
	            }
	        });
	    });
	    
	    
	  };
	}(plugin));

module.exports = plugin;