var plugin = {},
	common = require('../../../api/utils/common.js'),
    fetch = require("../../../api/parts/data/fetch.js"),
	async = require('async'),
	geoip = require('geoip-lite'),
	useragent = require('express-useragent'),
    plugins = require('../../pluginManager.js'),
	countlyCommon = require('../../../api/lib/countly.common.js'),
    Promise = require("bluebird"),
	request = require('request'),
	http = require('http'),
	urlParse = require('url');

plugins.setConfigs("attribution", {
    custom_links:false,
    pass_campaign: true
});

(function (plugin) {
	//write api call
	var organic_id = "_organic";
	var map = {
		lang:"l",
		browser:"b",
		browserVersion:"bv",
		platform:"pl",
		os:"os",
		mobile:"m",
		ip:"ip",
		referer:"r",
		campaign:"c",
		country:"cnty",
		city:"cty"
	};
    
    function replaceUrlsPlaceholders(url, segments){
        var re = /\{([^}]+)\}/g;
        var match;
        do {
            match = re.exec(url)
            if(match && match[0] && match[1] && segments[match[1]])
                url = url.replace(match[0], encodeURIComponent(segments[match[1]]));
        } while (match)
		return url;
	};
	
	function postBack(url, user){
		url = url.replace("{deviceID}", user.did);
		
		if(user.p)
			url = url.replace("{platform}", user.p);
		else
			url = url.replace("{platform}", "unknown");
        
        url = replaceUrlsPlaceholders(url, user);
		//console.log("url", url);
		var parts = urlParse.parse(url);
		if(parts.hostname){
			var options = {
				host: parts.hostname,
				port: parts.port,
				path: parts.path
			};
	
			http.get(options, function(resp){
				resp.on('data', function(chunk){
					//do something with chunk
				});
			}).on("error", function(e){
				console.log("Got error: " + e.message);
			});
		}
	};
	
	var fetchCampaignData = function (collection, params) {
        fetch.getTimeObj(collection, params, {unique: "clk", id:params.qstring.campaign_id}, function(output) {
            common.returnOutput(params, output);
        });
    };
	
	var recordMetric = function(params, user, campaignId, appId, metrics, value, custom){
		value = value || 1;
		common.db.collection('apps').findOne({'_id':common.db.ObjectID(appId)}, function (err, app) {
			var segments = ["l", "b", "pl", "cnty"];
            if(custom && custom.length){
                for(var i = 0; i < custom.length; i++){
                    segments.push(custom[i]);
                }
            }
			params.app_id = app['_id'];
			params.app_cc = app['country'];
			params.app_name = app['name'];
			params.appTimezone = app['timezone'];
			params.time = common.initTimeObj(params.appTimezone, params.qstring.timestamp);
            if(metrics.indexOf("aclk") !== -1 && metrics.indexOf("clk") === -1)
                metrics.push("clk");
            
            //fill segments
            var segs = {};
            for(var i = 0; i < segments.length; i++){
                if(typeof user[segments[i]] != "undefined"){
                    segs[segments[i]] = ("["+segments[i]+"]_"+user[segments[i]]+"").toLowerCase();
                }
            }
            
            common.recordCustomMetric(params, "campaigndata", campaignId, metrics, value, segs, ["clk"], user["last_click"]);
		});
	};
	
    var firstInstall = function(ob, dbAppUser, app, user, callback){
        var ip = ob.params.ip_address;
		var userID = ob.params.app_user_id+"";
		common.db.collection("attribution").findOne({app:app+"", ip:ip, p:user.p || "Unknown"}, function(err, campaign){
			if(!err && campaign){					
				common.db.collection("attribution").remove({_id:campaign._id}, function(){});
				common.db.collection('campaign_users'+app).findOne({'_id': campaign.u}, function(err, dbUser){
					if(dbUser){
						recordMetric(ob.params, dbUser, campaign.c, app, ["ins", "ses"]);
						dbUser.n = campaign.n;
                        common.updateAppUser(ob.params, {'$set': {cmp:dbUser}}, function(){
                            callback();
                        });
						common.db.collection("campaigns").findOne({_id:campaign.c}, function(err, data){
							if(!err && data && data.postbacks){
                                dbUser.did = ob.params.qstring.device_id;
                                dbUser.p = dbUser.pl || ob.params.app_user.p;
                                for(var i = 0; i < data.postbacks.length; i++){
                                    postBack(data.postbacks[i], dbUser);
                                }
							}
						});
					}
                    else{
                        callback();
                    }
				});
			}
            //checking if there is match by ip without fingerprint
            else{
                common.db.collection("attribution").findOne({app:app+"", ip:ip}, function(err, campaign){
                    if(!err && campaign){
                        common.db.collection("campaigns").findOne({_id:campaign.c}, function(err, data){
                            //if campaign has fingerprint disabled
                            if(!data.fingerprint){
                                common.db.collection("attribution").remove({_id:campaign._id}, function(){});
                                common.db.collection('campaign_users'+app).findOne({'_id': campaign.u}, function(err, dbUser){
                                    if(dbUser){
                                        recordMetric(ob.params, dbUser, campaign.c, app, ["ins", "ses"]);
                                        dbUser.n = campaign.n;
                                        common.updateAppUser(ob.params, {'$set': {cmp:dbUser}}, function(){callback();});
                                        if(!err && data && data.postbacks){
                                            dbUser.did = ob.params.qstring.device_id;
                                            dbUser.p = dbUser.pl || ob.params.app_user.p;
                                            for(var i = 0; i < data.postbacks.length; i++){
                                                postBack(data.postbacks[i], dbUser);
                                            }
                                        }
                                    }
                                    else{
                                        callback();
                                    }
                                });
                            }
                            else{
                                callback();
                            }
                        });
                    }
                    //counting as organic user
                    else{
                        recordMetric(ob.params, {}, organic_id+app, app, ["ins", "ses"]);
                        callback();
                    }
                });
            }
		});
    };
    
	plugins.register("/session/retention", function(ob){
        return new Promise(function(resolve, reject){
            var dbAppUser = ob.user;
            var app = ob.params.app_id;
            //console.log("retention");
            if((!ob.params.app.type || ob.params.app.type == "mobile") && (!dbAppUser || (!dbAppUser[common.dbUserMap['first_seen']] && !dbAppUser.cmp))){
                //console.log("new user");
                var userID = ob.params.app_user_id+"";
                user = ob.params.app_user;
                //console.log("user info", user);
                common.db.collection("campaigns").find({app_id:app+"", type:"adwords"}).toArray(function(err, campaigns){
                    //console.log("adwords campaigns", err, campaigns);
                    var attr_cmp = {};
                    if(!err && campaigns){
                        var cmp_names = {};
                        for(var i = 0; i < campaigns.length; i++){
                            cmp_names[campaigns[i]._id] = campaigns[i].name;
                        }
                        var attributed = false;
                        async.map(campaigns, function(campaign,done){
                            //console.log("checking campaign", campaign);
                            var p = "";
                            if(user && user.p)
                                p = user.p.toLowerCase();
                            if(campaign && campaign.typedata && campaign.typedata.adwords && campaign.typedata.adwords && campaign.typedata.adwords[p] &&
                            campaign.typedata.adwords[p].conversion && campaign.typedata.adwords[p].conversion.length &&
                            campaign.typedata.adwords[p].label && campaign.typedata.adwords[p].label.length &&
                            campaign.typedata.adwords[p].bundle && campaign.typedata.adwords[p].bundle.length){
                                var url = "https://www.googleadservices.com/pagead/conversion/"+campaign.typedata.adwords[p].conversion+"/?";
                                url += "label="+campaign.typedata.adwords[p].label;
                                url += "&bundleid="+campaign.typedata.adwords[p].bundle;
                                url +="&lat=0&rdid="+ob.params.qstring.device_id;
                                if(p == "ios"){
                                    url += "&idtype=idfa";
                                }
                                else if(p == "android"){
                                    url += "&idtype=advertisingid";
                                    url += "&appversion="+(user.av || "").replace(/:/g, ".");
                                    url += "&osversion="+(user.pv || "").replace(/:/g, ".").replace(/[^\d.-]/g, '');
                                    url += "&sdkversion=countly-sdk-"+p+"-v"+(ob.params.qstring.sdk_version || "");
                                    url += "&referrer="+(user.dpl || "").replace(/&#46;/g, '.');
                                }
                                //console.log("pinging", url);
                                var options = {
                                    uri: url,
                                    method: 'GET',
                                    followRedirect: false
                                };
                    
                                request(options, function (error, response, body) {
                                    //console.log("ping response", error, response.statusCode, body);
                                    if (!error && response.statusCode == 302) {
                                        var code = campaign._id;
                                        var cmp = {};
                                        var segments = [];
                                        if(response.headers['location']){
                                            //console.log("parsing postback", response.headers['location']);
                                            var parts = urlParse.parse(response.headers['location'], true);
                                            //console.log("postback url", parts.query);
                                            if(parts.query.trv_campaign_id && parts.query.trv_campaign_id.length){
                                                segments.push("TrueView");
                                                cmp["TrueView"] = parts.query.trv_campaign_id;
                                                if(parts.query.video_id && parts.query.video_id.length){
                                                    segments.push("VideoID");
                                                    cmp["VideoID"] = parts.query.video_id;
                                                }
                                            }
                                            if(parts.query.click_url){
                                                var click = urlParse.parse(parts.query.click_url, true);
                                                if(click.pathname && click.pathname.indexOf("/at/") == 0){
                                                    var click_parts = click.pathname.split("/");
                                                    var part = click_parts[click_parts.length-1];
                                                    if(cmp_names[part])
                                                        code = part;
                                                    for(var i in click.query){
                                                        if(click.query[i].length){
                                                            segments.push(i);
                                                            cmp[i] = click.query[i];
                                                        }
                                                    }
                                                    //console.log("clicked through countly tracking link");
                                                }
                                                //console.log("click url", click.query);
                                            }
    
                                        }
                                        if(!attr_cmp[code]){
                                            attributed = true;
                                            //console.log("got install from adwords for ", code, cmp_names[code]);
                                            attr_cmp[code] = true;
                                            cmp.c = code;
                                            cmp.n = cmp_names[code];
                                            recordMetric(ob.params, cmp, code, app, ["ins", "ses"], 1, segments);
                                            common.updateAppUser(ob.params, {'$set': {cmp:cmp}});
                                            if(campaign && campaign.postbacks){
                                                user.did = ob.params.qstring.device_id;
                                                user.c = cmp.c;
                                                user.n = cmp.n;
                                                for(var i = 0; i < campaign.postbacks.length; i++){
                                                    postBack(campaign.postbacks[i], user);
                                                }
                                            }
                                        }
                                    }
                                    done(null, null);
                                });
                            }
                            else{
                                done(null, null);
                            }
                        }, function (err, results) {
                            //console.log("finished checking adwords", err, results, attributed);
                            if(!attributed)
                                firstInstall(ob, dbAppUser, app, user, resolve);
                            else
                                resolve();
                        });
                    }
                    else{
                        firstInstall(ob, dbAppUser, app, user, resolve);
                    }
                });
            }
            else if(dbAppUser && dbAppUser.cmp){
                recordMetric(ob.params, dbAppUser.cmp, dbAppUser.cmp.c, app, ["ses"]);
                resolve();
            }
            //organic user session/retention
            else{
                recordMetric(ob.params, {}, organic_id+app, app, ["ses"]);
                resolve();
            }
        });
	});
	
	plugins.register("/i/events", function(ob){
		var params = ob.params;
		var currEvent = ob.currEvent;
		var app = params.app_id;
		if (params.iap_event && params.iap_event == currEvent.key) {
			var dbAppUser = params.app_user;
			if(dbAppUser && dbAppUser.cmp){
				recordMetric(params, dbAppUser.cmp, dbAppUser.cmp.c, app, ["rev"], currEvent.count*currEvent.sum);
			}
            else{
                recordMetric(params, {}, organic_id+app, app, ["rev"], currEvent.count*currEvent.sum);
            }
			return true;
        }
	});
	
	plugins.register("/i/campaign", function(ob){
		var params = ob.params, 
			paths = ob.paths,
			validateUserForWriteAPI = ob.validateUserForWriteAPI,
            validate = ob.validateUserForDataWriteAPI;
		if (params.qstring.args) {
            try {
                params.qstring.args = JSON.parse(params.qstring.args);
            } catch (SyntaxError) {
                console.log('Parse /i/campaign JSON failed');
            }
        }

        switch (paths[3]) {
            case 'create':
                validateUserForWriteAPI(function (params) {	
					if (!((params.member.user_of && params.member.admin_of.indexOf(params.qstring.args.app_id) != -1) || params.member.global_admin)) {
						common.returnMessage(params, 200, 'User does not have right to create campaign');
						return false;
					}
					var argProps = {
							'_id':     { 'required': false, 'type': 'String' },
							'name':     { 'required': true, 'type': 'String', 'min-length': 1 },
							'link':     { 'required': false, 'type': 'String' },
							'fingerprint':{ 'required': false, 'type': 'Boolean' },
							'cost':     { 'required': false, 'type': 'String' },
							'costtype': { 'required': false, 'type': 'String' },
							'links':	{ 'required': false, 'type': 'Object' },
							'type':	    { 'required': false, 'type': 'String' },
							'typedata':	{ 'required': false, 'type': 'Object' },
							'postbacks':{ 'required': false, 'type': 'Array' },
							'app_id': 	{ 'required': true, 'type': 'String', 'min-length': 1 }
						},
						newAtt = {};

					if (!(newAtt = common.validateArgs(params.qstring.args, argProps))) {
						common.returnMessage(params, 200, 'Not enough args');
						return false;
					}
                    if(!newAtt._id)
                        newAtt._id = common.crypto.createHash('sha1').update(newAtt.name+newAtt.app_id+new Date().getTime()).digest('hex');
					newAtt.cost = (newAtt.cost) ? parseFloat(newAtt.cost) : 0;
					newAtt.costtype = (newAtt.costtype) ? newAtt.costtype : "click";
					newAtt.links = (newAtt.links) ? newAtt.links : {};
					newAtt.type = (newAtt.type) ? newAtt.type : "default";
					newAtt.typedata = (newAtt.typedata) ? newAtt.typedata : {};
					newAtt.postbacks = (newAtt.postbacks) ? newAtt.postbacks : [];
			
					common.db.collection('campaigns').insert(newAtt, function(err, app) {
						if(err){
                            if(err.code == 11000)
                                common.returnMessage(params, 200, "Duplicate campaign link");
                            else
                                common.returnMessage(params, 200, err);
						}
						else{
                            plugins.dispatch("/systemlogs", {params:params, action:"attribution_campaign_created", data:newAtt});
							common.returnMessage(params, 200, "Success");
                        }
					});
				}, params);
                break;
            case 'delete':
                validateUserForWriteAPI(function (params) {
					if (!((params.member.user_of && params.member.admin_of.indexOf(params.qstring.args.app_id) != -1) || params.member.global_admin)) {
						common.returnMessage(params, 200, 'User does not have right to delete campaign');
						return false;
					}
			
					var argProps = {
							'_id': { 'required': true, 'type': 'String'}
						},
						id = '';
			
					if (!(id = common.validateArgs(params.qstring.args, argProps)._id)) {
						common.returnMessage(params, 200, 'Not enough args');
						return false;
					}
			
                    common.db.collection('campaigns').findOne({'_id': id}, function(err, campaign) {
                        common.db.collection('campaigns').remove({'_id': id}, {safe: true}, function(err, result) {
                            if (err) {
                                common.returnMessage(params, 200, 'Error deleting campaign');
                                return false;
                            }
                            common.db.collection('attribution').remove({'c': id},function(){});
                            common.db.collection('apps').find({}).toArray(function (err, apps) {
                                if (!apps || err) {
                                    console.log("No apps");
                                    return;
                                }
                                for(var i = 0; i < apps.length; i++){
                                    common.db.collection('campaign_users'+apps[i]._id).remove({'c': id},function(){});
                                }
                            });
                            plugins.dispatch("/systemlogs", {params:params, action:"attribution_campaign_deleted", data:campaign});
                            common.returnMessage(params, 200, 'Success');
                        });
                    });
			
					return true;
				}, params);
                break;
			case 'update':
                validateUserForWriteAPI(function (params) {
					if (!((params.member.user_of && params.member.admin_of.indexOf(params.qstring.args.app_id) != -1) || params.member.global_admin)) {
						common.returnMessage(params, 200, 'User does not have right to update campaign');
						return false;
					}
					var argProps = {
							'_id':   	{ 'required': true, 'type': 'String', 'exclude-from-ret-obj': true },
							'name':     { 'required': false, 'type': 'String' },
							'link':     { 'required': false, 'type': 'String' },
                            'fingerprint':{ 'required': false, 'type': 'Boolean' },
							'cost':     { 'required': false, 'type': 'String' },
							'costtype': { 'required': false, 'type': 'String' },
                            'type':	    { 'required': false, 'type': 'String' },
							'typedata':	{ 'required': false, 'type': 'Object' },
							'links': 	{ 'required': false, 'type': 'Object' },
							'postbacks':{ 'required': false, 'type': 'Array' }
						},
						updatedAtt = {};

					if (!(updatedAtt = common.validateArgs(params.qstring.args, argProps))) {
						common.returnMessage(params, 200, 'Not enough args');
						return false;
					}

					if(updatedAtt.cost)
						updatedAtt.cost = parseFloat(updatedAtt.cost);
					
					if(updatedAtt.name && updatedAtt.name.length == 0)
						delete updatedAtt.name;
                    
                    if(updatedAtt.type && updatedAtt.type.length == 0)
						delete updatedAtt.type;
					
					if(updatedAtt.link && updatedAtt.link.length == 0)
						delete updatedAtt.link;
					
					if (Object.keys(updatedAtt).length === 0) {
						common.returnMessage(params, 200, 'Success');
						return true;
					}

					common.db.collection('campaigns').findOne({'_id': params.qstring.args._id}, function(err, att){
						if (err || !att) common.returnMessage(params, 200, 'Campaign not found');
						else {
							common.db.collection('campaigns').update({'_id': params.qstring.args._id}, {$set: updatedAtt}, function(err, app) {
                                var data = {};
                                data.campaign_id = params.qstring.args._id;
                                data.app_id = params.qstring.args.app_id;
                                data.before = att;
                                data.update = updatedAtt;
                                plugins.dispatch("/systemlogs", {params:params, action:"attribution_campaign_updated", data:data});
								common.returnMessage(params, 200, 'Success');
							});
						}
					});
			
					return true;
				}, params);
                break;
			case 'click':
				var code = paths[4];
				common.db.collection('campaigns').findOne({_id:code}, function(err,result){
					if (err || !result){
						params.res.writeHead(404, {'Content-Type': 'text/plain; charset=utf-8'});
						params.res.write("Campaign not Found");
						params.res.end();
					}
					else{
						var data = {};
						if (typeof params.req.headers['accept-language'] === "undefined") {
							params.req.headers['accept-language'] = 'unknown';
						}
						data[map.lang] = params.req.headers['accept-language'].split(',')[0];
						if(data[map.lang])
							data[map.lang] = data[map.lang].replace(/_/g, '-');
						if (params.req.headers['user-agent'] !== undefined) {
							var ua = useragent.parse(params.req.headers['user-agent']);
							data[map.browser] = ua.Browser || "unknown";
							data[map.browserVersion] = ua.Version || "unknown";
							data[map.os] = ua.OS || "unknown";
							var platform = ua.Platform;
							data[map.platform] = "unknown";
							if(platform == "iPad" || platform == "iPod" || platform == "iPhone"){
								data[map.platform] = "iOS";
							}
							else{
								data[map.platform] = platform.replace("Microsoft ", "").replace("Apple ", "");
								if(data[map.platform].toLowerCase() == "android"){
									var uas = params.req.headers['user-agent'];
									if((/Kindle/i.test(uas) || /Silk/i.test(uas) || /KFTT/i.test(uas) || /KFOT/i.test(uas) || /KFJWA/i.test(uas) || /KFJWI/i.test(uas) || /KFSOWI/i.test(uas) || /KFTHWA/i.test(uas) || /KFTHWI/i.test(uas) || /KFAPWA/i.test(uas) || /KFAPWI/i.test(uas) || /KFARWI/i.test(uas) || /KFASWI/i.test(uas) || /KFSAWI/i.test(uas) || /KFSAWA/i.test(uas) || /SD4930UR/i.test(uas)) && !/Playstation/.test(uas)){
										data[map.platform] = "Kindle";
									}
								}
							}
							data[map.mobile] = ua.isMobile || false;
						}
						var ip = params.qstring.ip_address || common.getIpAddress(params.req);
						data[map.country] == "unknown";
						data[map.city] == "unknown";
						if(ip){
							var locationData = geoip.lookup(ip);
							if (locationData) {
								if (locationData.country) {
									data[map.country] = locationData.country;
								}
					
								if (locationData.city) {
									data[map.city] = locationData.city;
								}
							}
						}
						data[map.campaign] = result._id;
						if(params.req.headers.cookie){
							var parts = params.req.headers.cookie.split(";");
							for(var i = 0; i < parts.length; i++){
								var cookies = parts[i].split("=");
								if(cookies[0].trim() == code){
									data._id = cookies[1];
									break;
								}
							}
						}
						if(params.qstring.test || !data._id)
							data._id = common.crypto.createHash('sha1').update(JSON.stringify(data)).digest('hex');
						if (params.req.headers.referer === undefined) {
							params.req.headers.referer = 'unknown';
						}
						data[map.referer] = params.req.headers.referer;
						var link = result.link;
						var scheme;
						var platform = "unknown";
						if(data[map.platform])
							platform = data[map.platform].toLowerCase().replace(/\s/g, '-');
						if(result.links[platform]){
							link = result.links[platform];
						}
						
						if(result.links[platform+"-custom-scheme"]){
							scheme = result.links[platform+"-custom-scheme"];
						}
                        
                        if(params.qstring.url){
                            scheme = null;
                            link = params.qstring.url;
                        }
						
						var parts = urlParse.parse(link, true);
						if(parts.hostname){
							delete parts.search;
                            if(parts.hostname.indexOf("itunes.apple.com") > -1){
								if(!parts.query)
									parts.query = {};
								if(!parts.query.cid)
									parts.query.cid = result.name;		
							}
							else if(parts.hostname.indexOf("play.google.com") > -1 || parts.protocol == "market:"){
								if(!parts.query)
									parts.query = {};
								if(!parts.query.referrer)
									parts.query.referrer = "countly_cid="+result._id+"&countly_cuid="+data._id;
								else
									parts.query.referrer += "&countly_cid="+result._id+"&countly_cuid="+data._id;	
							}
                            else if(plugins.getConfig("attribution").pass_campaign){
                                if(!parts.query)
									parts.query = {};
                                parts.query.cly_id = result._id;
                                parts.query.cly_uid = data._id;
                            }
							link = urlParse.format(parts);
						}
						
						if(scheme){
							link = "/campaign/"+platform+"/?url="+encodeURIComponent(link)+"&scheme="+encodeURIComponent(scheme);							
						}
                        
                        var segments = [];
                        if(typeof params.qstring.segments != "undefined"){
                            var custom = {};
                            try{
                                custom = JSON.parse(params.qstring.segments);
                            }
                            catch(ex){
                                custom = {};
                            }

                            for(var i in custom){
                                segments.push(i);
                                data[i] = custom[i];
                            }
                        }
						
                        link = replaceUrlsPlaceholders(link.replace(/%7B/g, "{").replace(/%7D/g, "}"), data);
                        
						var exp = new Date();
						exp.setFullYear(9999);
						params.res.writeHead(302, {
							'Set-Cookie': code+'='+data._id+'; expires='+ exp.toUTCString()+';',
							'Campaign-User': data._id,
							'Location': link
						});
						params.res.end();
                        data["last_click"] = params.qstring.timestamp || Math.floor(new Date().getTime() / 1000);
						var ip = params.qstring.ip_address || common.getIpAddress(params.req);
						common.db.collection("attribution").update({"app":result.app_id, "ip": ip, "p":data[map.platform]}, {$set: {"cd": new Date(), "ip": ip, "app":result.app_id, "c":code, "p":data[map.platform], n:result.name, u:data._id}}, {'upsert': true}, function(err, res){} );
						common.db.collection('campaign_users'+result.app_id).findAndModify({'_id': data._id},{},{$set: data}, {'upsert': true},function(err, dbUser){
							var metrics = ["aclk"];
                            data["last_click"] = null
							if(dbUser && dbUser.value){
								data["last_click"] = dbUser.value["last_click"];
							}
							recordMetric(params, data, code, result.app_id, metrics, 1, segments);
						});
					}
				});
			
				return true;
                break;
            case 'hide':
                validate(params, function (params) {
                    common.db.collection('campaigns').update({'_id': params.qstring.args.campaign_id }, {"$set":{is_hidden:true}}, function (err, res){
                        plugins.dispatch("/systemlogs", {params:params, action:"attribution_campaign_hidden", data:{campaign_id: params.qstring.args.campaign_id, app_id:params.qstring.app_id}});
                        common.returnMessage(params, 200, 'Success');
						return true;
                    });
				});
                break;
            case 'show':
                validate(params, function (params) {
                    common.db.collection('campaigns').update({'_id': params.qstring.args.campaign_id }, {"$set":{is_hidden:false}}, function (err, res){
                        plugins.dispatch("/systemlogs", {params:params, action:"attribution_campaign_shown", data:{campaign_id: params.qstring.args.campaign_id, app_id:params.qstring.app_id}});
                        common.returnMessage(params, 200, 'Success');
						return true;
                    });
				});
                break;
            default:
                common.returnMessage(params, 404, 'Invalid endpoint');
                break;
        }
		return true;
	});
	
	plugins.register("/o/campaign", function(ob){
		var params = ob.params, 
			paths = ob.paths,
			validateUserForDataReadAPI = ob.validateUserForDataReadAPI;

		validateUserForDataReadAPI(params, function(){
			if(params.qstring.camp_id)
				common.db.collection('campaigns').findOne({_id:params.qstring.camp_id},function (err, atts) {
					if (!atts || err) {
						common.returnOutput(params, {});
						return false;
					}
					
					params.qstring.campaign_id = params.qstring.camp_id;
					fetch.getTimeObj('campaigndata', params, {unique: "clk", id:params.qstring.campaign_id}, function(output) {
						atts.data = output;
						common.returnOutput(params, atts);
					});
					return true;
				});
            else if(params.qstring.list)
                common.db.collection('campaigns').find({app_id:params.qstring.app_id},{name:1}).toArray(function (err, atts) {
					if (!atts || err) {
						common.returnOutput(params, []);
						return false;
					}
					common.returnOutput(params, atts);
					return true;
				});
            else if(params.qstring.data){
                var atts = [];
                try{
                    atts = JSON.parse(params.qstring.data);
                }
                catch(e){
                    atts = [];
                }
                async.map(atts, function(id, callback){
                    params.qstring.campaign_id = id;
                    if(id == organic_id)
                        params.qstring.campaign_id += params.qstring.app_id;
					fetch.getTimeObj('campaigndata', params, {unique: "clk", id:params.qstring.campaign_id}, function(output) {
						var obj = {_id:id, data:output};
						callback(null, obj);
					});
				}, function (err, results) {
					common.returnOutput(params, results);
				});
            }
			else{
                var filter = {app_id:params.qstring.app_id};
                if(params.qstring.filter && params.qstring.filter != ""){
                    switch (params.qstring.filter) {
                        case "campaign-hidden":
                            filter["is_hidden"] = true;
                            break;
                    }
                }
                if(params.qstring.filter !== "campaign-hidden"){
                    filter["is_hidden"] = {$ne: true};
                }
				common.db.collection('campaigns').find(filter).toArray(function (err, atts) {
					if (!atts || err) {
						atts = [];
					}
                    if(params.qstring.filter !== "campaign-hidden"){
                        atts.push({
                            "_id": organic_id,
                            "name": "Organic",
                            "link": "",
                            "fingerprint": true,
                            "cost": 0,
                            "costtype": "campaign",
                            "links": {},
                            "postbacks": [],
                            "app_id": params.qstring.app_id
                        });
                    }
					async.map(atts, function(obj, callback){
						params.qstring.campaign_id = obj._id;
                        if(params.qstring.campaign_id == organic_id)
                            params.qstring.campaign_id += obj.app_id;
						fetch.getTimeObj('campaigndata', params, {unique: "clk", id:params.qstring.campaign_id}, function(output) {
                            var data = aggregateData(output);
                            obj.totalCost = 0;
                            obj.userCost = 0;
                            obj.aclk = data.usage.click.total;
                            obj.ins = data.usage.install.total;
                            obj.clk = data.usage.unique.total;
                            obj.rev = data.usage.payment.total;
                            obj.ses = data.usage.session.total;
                            if(obj.cost > 0){
                                if(obj.costtype == "click"){
                                    obj.totalCost = obj.aclk*obj.cost;
                                }
                                else if(obj.costtype == "install"){
                                    obj.totalCost = obj.ins*obj.cost;
                                }
                                else if(obj.costtype == "campaign"){
                                    obj.totalCost = obj.cost;
                                }
                                obj.userCost = (obj.ins > 0) ? obj.totalCost/obj.ins : 0;
                            }
							callback(null, obj);
						});
					}, function (err, results) {
						common.returnOutput(params, results);
					});
					return true;
				});
            }
	   });
	   return true;
	});
	
	plugins.register("/i", function(ob){
		var params = ob.params;
		
		if(params.qstring.campaign_id && (typeof params.app_user.sc === "undefined" || (params.app_user.sc === 1 && typeof params.app_user.fs === "number" && Math.floor(Date.now() / 1000) - params.app_user.fs <= 300))){
            if(params.qstring.campaign_id == "cly_organic"){
                recordMetric(ob.params, {}, organic_id+params.app_id, params.app_id, ["ins", "ses"]);
            }
            else{
                common.db.collection('campaigns').findOne({_id:params.qstring.campaign_id}, function(err, campaign){
                    if (!err && campaign){
                        if(params.qstring.campaign_user){
                            common.db.collection('campaign_users'+campaign.app_id).findOne({'_id': params.qstring.campaign_user}, function(err, dbUser){
                                if(!err && dbUser){
                                    var user = params.app_user;
                                    if((!user) || (user && !user.cmp)){
                                        recordMetric(params, dbUser, params.qstring.campaign_id, campaign.app_id, ["ins"]);
                                        dbUser.n = campaign.name;
                                        dbUser.c = campaign._id;
                                        common.db.collection('attribution').remove({'u': params.qstring.campaign_user},function(){});
                                        common.updateAppUser(params, {'$set': {_id:params.app_user_id, cmp:dbUser}}, function(){
                                            if(campaign.postbacks){
                                                dbUser.did = params.qstring.device_id;
                                                dbUser.p = dbUser.pl || params.app_user.p;
                                                for(var i = 0; i < campaign.postbacks.length; i++){
                                                    postBack(campaign.postbacks[i], dbUser);
                                                }
                                            }
                                        });
                                    }
                                }
                            });
                        }
                        else{
                            var user = params.app_user;
                            if((!user) || (user && !user.cmp)){
                                var dbUser = {};
                                var metrics = {
                                    platform:"pl",
                                    browser:"b",
                                    locale:"l",
                                };
                                var prop;
                                var segments = [];
                                for(var i in params.qstring){
                                    if(i != "campaign_id" && i.indexOf("campaign_") == 0){
                                        prop = i.replace("campaign_", "");
                                        if(metrics[prop])
                                            dbUser[metrics[prop]] = params.qstring[i];
                                        else{
                                            dbUser[prop] = params.qstring[i];
                                            segments.push(prop);
                                        }
                                    }
                                }
        
                                recordMetric(params, dbUser, params.qstring.campaign_id, campaign.app_id, ["ins"], 1, segments);
                                dbUser.n = campaign.name;
                                dbUser.c = campaign._id;
                                common.updateAppUser(params, {'$set': {_id:params.app_user_id, cmp:dbUser}}, function(){
                                    if(campaign.postbacks){
                                        dbUser.did = params.qstring.device_id;
                                        dbUser.p = dbUser.pl || params.app_user.p;
                                        for(var i = 0; i < campaign.postbacks.length; i++){
                                            postBack(campaign.postbacks[i], dbUser);
                                        }
                                    }
                                });
                            }
                        }
                    }
                });
            }
			return true;
		}
		return false;
	});
	
	 //waiting for read request
	plugins.register("/o", function(ob){
		var params = ob.params;
		var validateUserForDataReadAPI = ob.validateUserForDataReadAPI;
		
		//if user requested to read our metric
		if (params.qstring.method == "campaigndata") {
		
			//validate user and output data using fetchTimeObj method
			validateUserForDataReadAPI(params, fetchCampaignData, 'campaigndata');
		
			//return true, we responded to this request
			return true;
		}
		
		//else we are not interested in this request
		return false;
	});
	
	plugins.register("/i/apps/reset", function(ob){
		var appId = ob.appId;
		common.db.collection('campaigns').remove({'app_id': appId},function(){});
		common.db.collection('attribution').remove({'app': appId},function(){});
		common.db.collection('campaign_users'+appId).drop(function(){});
        common.db.collection('campaigndata').remove({'_id': {$regex: appId + ".*"}},function(){});
	});
    
    plugins.register("/i/apps/clear_all", function(ob){
		var appId = ob.appId;
		common.db.collection('attribution').remove({'app': appId},function(){});
		common.db.collection('campaign_users'+appId).drop(function(){});
        common.db.collection('campaigndata').remove({'_id': {$regex: appId + ".*"}},function(){});
	});
    
    plugins.register("/i/apps/clear", function(ob){
		var appId = ob.appId;
        var dates = ob.dates;
        var docs = [];
        common.db.collection('campaigns').find({app_id:appId}, {_id:1}).toArray(function(err, res){
            if(res && res.length){
                for(var i = 0; i < res.length; i++){
                    for(var j = 0; j < dates.length; j++){
                        docs.push(res[i]._id+"_"+dates[j]);
                    }
                }
            }
            common.db.collection('campaigndata').remove({$and:[{'a': appId}, {'_id': {$nin:docs}}]},function(){});
        });
	});
	
	plugins.register("/i/apps/delete", function(ob){
		var appId = ob.appId;
		common.db.collection('campaigns').remove({'app_id': appId},function(){});
		common.db.collection('attribution').remove({'app': appId},function(){});
		common.db.collection('campaign_users'+appId).drop(function(){});
        common.db.collection('campaigndata').remove({'_id': {$regex: appId + ".*"}},function(){});
	});
    
    //returns the union of two arrays
    function union(x, y) {
        var obj = {};
        for (var i = x.length-1; i >= 0; -- i) {
            obj[x[i]] = x[i];
        }

        for (var i = y.length-1; i >= 0; -- i) {
            obj[y[i]] = y[i];
        }

        var res = [];

        for (var k in obj) {
            res.push(obj[k]);
        }

        return res;
    }
    
    //aggregate data
    function aggregateData(_campaignData) {

        //Update the current period object in case selected date is changed
        _periodObj = countlyCommon.periodObj;

        var dataArr = {},
            tmp_x,
            tmp_y,
            currentTotal = 0,
            previousTotal = 0,
            currentUnique = 0,
            previousUnique = 0,
            currentInstall = 0,
            previousInstall = 0,
            currentPayments = 0,
            previousPayments = 0,
			currentSessions = 0,
            previousSessions = 0,
            isEstimate = false;

        if (_periodObj.isSpecialPeriod) {

            for (var i = 0; i < (_periodObj.currentPeriodArr.length); i++) {
                tmp_x = countlyCommon.getDescendantProp(_campaignData, _periodObj.currentPeriodArr[i]);
                tmp_x = clearObject(tmp_x);
                currentUnique += tmp_x["clk"];
                currentTotal += tmp_x["aclk"];
                currentInstall += tmp_x["ins"];
                currentPayments += tmp_x["rev"];
                currentSessions += tmp_x["ses"];
            }

            for (var i = 0; i < (_periodObj.previousPeriodArr.length); i++) {
                tmp_y = countlyCommon.getDescendantProp(_campaignData, _periodObj.previousPeriodArr[i]);
                tmp_y = clearObject(tmp_y);
                previousUnique += tmp_y["clk"];
                previousTotal += tmp_y["aclk"];
                previousInstall += tmp_y["ins"];
                previousPayments += tmp_y["rev"];
                previousSessions += tmp_y["ses"];
            }
        } else {
            tmp_x = countlyCommon.getDescendantProp(_campaignData, _periodObj.activePeriod);
            tmp_y = countlyCommon.getDescendantProp(_campaignData, _periodObj.previousPeriod);
            tmp_x = clearObject(tmp_x);
            tmp_y = clearObject(tmp_y);

            currentTotal = tmp_x["aclk"];
            previousTotal = tmp_y["aclk"];
            currentInstall = tmp_x["ins"];
            previousInstall = tmp_y["ins"];
            currentUnique = tmp_x["clk"];
            previousUnique = tmp_y["clk"];
            currentPayments = tmp_x["rev"];
            previousPayments = tmp_y["rev"];
            currentSessions = tmp_x["ses"];
            previousSessions = tmp_y["ses"];
        }

        var changeTotal = countlyCommon.getPercentChange(previousTotal, currentTotal),
            changeInstall = countlyCommon.getPercentChange(previousInstall, currentInstall),
            changeUnique = countlyCommon.getPercentChange(previousUnique, currentUnique),
            changePayments = countlyCommon.getPercentChange(previousPayments, currentPayments),
            changeSessions = countlyCommon.getPercentChange(previousSessions, currentSessions);

        dataArr =
        {
            usage:{
                "click":{
                    "total":currentTotal,
                    "change":changeTotal.percent,
                    "trend":changeTotal.trend,
                    "isEstimate":isEstimate
                },
                "unique":{
                    "total":currentUnique,
                    "prev-total":previousUnique,
                    "change":changeUnique.percent,
                    "trend":changeUnique.trend,
                    "isEstimate":isEstimate
                },
                "install":{
                    "total":currentInstall,
                    "prev-total":previousInstall,
                    "change":changeInstall.percent,
                    "trend":changeInstall.trend,
                    "isEstimate":isEstimate
                },
                "payment":{
                    "total":parseFloat(parseFloat(currentPayments).toFixed(2)),
                    "prev-total":previousPayments,
                    "change":changePayments.percent,
                    "trend":changePayments.trend,
                    "isEstimate":isEstimate
                },
                "session":{
                    "total":currentSessions,
                    "prev-total":previousSessions,
                    "change":changeSessions.percent,
                    "trend":changeSessions.trend,
                    "isEstimate":isEstimate
                }
            }
        };

        return dataArr;
    };
    
    function clearObject(obj) {
        if (obj) {
            if (!obj["aclk"]) obj["aclk"] = 0;
            if (!obj["clk"]) obj["clk"] = 0;
            if (!obj["ins"]) obj["ins"] = 0;
            if (!obj["ses"]) obj["ses"] = 0;
            if (!obj["rev"]) obj["rev"] = 0;
        }
        else {
            obj = {"aclk":0, "clk":0, "ins":0, "ses":0, "rev":0};
        }

        return obj;
    };
}(plugin));

module.exports = plugin;