var request = require('supertest');
var should = require('should');
var testUtils = require("../../test/testUtils");
request = request(testUtils.url);

var APP_KEY = "";
var API_KEY_ADMIN = "";
var APP_ID = "";
var CAMPAIGN_ID = "";
var CAMPAIGNF_ID = "";
var CAMPAIGN_USER = "";
var CAMPAIGNF_USER = "";
var DEVICE_ID = "1234567890";
var RE = /^-{0,1}\d*\.{0,1}\d+$/;
var userAgent1 = "Mozilla/5.0 (Linux; Android 4.4; Nexus 5 Build/_BuildID_) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/30.0.0.0 Mobile Safari/537.36";
var userAgent2 = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3";
var userAgent3 = "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2272.118 Safari/537.36";

function verifyMetrics(err, ob, done, correct){
	ob.should.not.be.empty;
	if(ob["meta"]){
		for(var i in ob["meta"])
			ob["meta"][i].sort()
	}
	if(correct["meta"]){
		for(var i in correct["meta"])
			correct["meta"][i].sort()
	}
	ob.should.have.property("meta").eql(correct.meta);
	for(i in ob)
	{
		ob.should.have.property(i).and.not.eql({});
		if(RE.test(i))
		{
			for(var c in correct){
				if(c != "meta")
					ob[i].should.have.property(c, correct[c]);
			}
			for(j in ob[i])
			{
				if(RE.test(j))
				{
					for(var c in correct){
						if(c != "meta")
							ob[i][j].should.have.property(c, correct[c]);
					}
					for(k in ob[i][j])
					{
						if(RE.test(k))
						{
							for(var c in correct){
								if(c != "meta")
									ob[i][j][k].should.have.property(c, correct[c]);
							}
						}
					}
				}
			}
		}
	}
	setTimeout(done, 10000);
};

describe('Testing Attribution', function(){
	//{"data":{},"meta":[],"lu":"2015-01-20T12:00:06.176Z"}
	describe('Empty campaigns', function(){
		it('should success', function(done){
			API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
	describe('Create campaign without fingerprint', function(){
		describe('No name', function(){
			it('should Not enough args', function(done){
				var args = {
					link: "http://count.ly",
					app_id: APP_ID
				};
				request
				.get('/i/campaign/create?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Not enough args");
					setTimeout(done, 100)
				});
			});
		});
		describe('No app id', function(){
			it('should Not enough args', function(done){
				var args = {
					name: "Test",
					link: "http://count.ly"
				};
				request
				.get('/i/campaign/create?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Not enough args");
					setTimeout(done, 100)
				});
			});
		});
		describe('All args', function(){
			it('should success', function(done){
				var args = {
					name: "Test",
					link: "http://count.ly",
					app_id: APP_ID
				};
				request
				.get('/i/campaign/create?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Success");
					setTimeout(done, 100)
				});
			});
		});
	//[{"_id":"2c54a3ac541153f4b3077f2a3975e43547b67426","name":"test","link":"test","cost":0,"costtype":"click","links":{},"postbacks":[],"app_id":"552b79d251f2712c3ae7bce1","data":{}}]
		describe('Verify campaign', function(){
			it('should success', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[0].should.have.property("_id");
					CAMPAIGN_ID = ob[0]._id;
					ob[0].should.have.property("name", "Test");
					ob[0].should.have.property("link", "http://count.ly");
					ob[0].should.have.property("cost", 0);
					ob[0].should.have.property("costtype", "click");
					ob[0].should.have.property("links", {});
					ob[0].should.have.property("postbacks", []);
					ob[0].should.have.property("app_id", APP_ID);
					setTimeout(done, 100)
				});
			});
		});
		
		describe('Verify campaign details', function(){
			it('should success', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&camp_id="+CAMPAIGN_ID)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.property("_id");
					ob.should.have.property("name", "Test");
					ob.should.have.property("link", "http://count.ly");
					ob.should.have.property("cost", 0);
					ob.should.have.property("costtype", "click");
					ob.should.have.property("data", {});
					ob.should.have.property("links", {});
					ob.should.have.property("postbacks", []);
					ob.should.have.property("app_id", APP_ID);
					setTimeout(done, 100)
				});
			});
		});
	});
    
    describe('Create campaign with fingerprint', function(){
		describe('All args', function(){
			it('should success', function(done){
				var args = {
					name: "Test",
					link: "http://count.ly",
                    fingerprint: true,
					app_id: APP_ID
				};
				request
				.get('/i/campaign/create?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Success");
					setTimeout(done, 100)
				});
			});
		});
	//[{"_id":"2c54a3ac541153f4b3077f2a3975e43547b67426","name":"test","link":"test","cost":0,"costtype":"click","links":{},"postbacks":[],"app_id":"552b79d251f2712c3ae7bce1","data":{}}]
		describe('Verify campaign', function(){
			it('should success', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(3);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("name", "Test");
					ob[0].should.have.property("link", "http://count.ly");
					ob[0].should.have.property("cost", 0);
					ob[0].should.have.property("costtype", "click");
					ob[0].should.have.property("links", {});
					ob[0].should.have.property("postbacks", []);
					ob[0].should.have.property("app_id", APP_ID);
                    
                    ob[1].should.have.property("_id");
					CAMPAIGNF_ID = ob[1]._id;
					ob[1].should.have.property("name", "Test");
					ob[1].should.have.property("link", "http://count.ly");
					ob[1].should.have.property("cost", 0);
					ob[1].should.have.property("costtype", "click");
					ob[1].should.have.property("fingerprint", true);
					ob[1].should.have.property("links", {});
					ob[1].should.have.property("postbacks", []);
					ob[1].should.have.property("app_id", APP_ID);
                    
					setTimeout(done, 100)
				});
			});
		});
		
		describe('Verify campaign details', function(){
			it('should success', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&camp_id="+CAMPAIGNF_ID)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.property("_id");
					ob.should.have.property("name", "Test");
					ob.should.have.property("link", "http://count.ly");
					ob.should.have.property("cost", 0);
					ob.should.have.property("costtype", "click");
                    ob.should.have.property("fingerprint", true);
					ob.should.have.property("data", {});
					ob.should.have.property("links", {});
					ob.should.have.property("postbacks", []);
					ob.should.have.property("app_id", APP_ID);
					setTimeout(done, 100)
				});
			});
		});
	});
	
	describe('Update Campaign', function(){
		describe('No ID', function(){
			it('should Not enough args', function(done){
				var args = {
					name: "test2",
					link: "http://resources.count.ly"
				};
				request
				.get('/i/campaign/update?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Not enough args");
					setTimeout(done, 100)
				});
			});
		});
		describe('Update', function(){
			it('should success', function(done){
				var args = {
					_id: CAMPAIGN_ID,
					name: "test2",
					link: "http://resources.count.ly"
				};
				request
				.get('/i/campaign/update?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Success");
					setTimeout(done, 100)
				});
			});
		});
	//[{"_id":"2c54a3ac541153f4b3077f2a3975e43547b67426","name":"test","link":"test","cost":0,"costtype":"click","links":{},"postbacks":[],"app_id":"552b79d251f2712c3ae7bce1","data":{}}]
		describe('Verify update campaign', function(){
			it('should success', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(3);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("name", "test2");
					ob[0].should.have.property("link", "http://resources.count.ly");
					ob[0].should.have.property("cost", 0);
					ob[0].should.have.property("costtype", "click");
					ob[0].should.have.property("links", {});
					ob[0].should.have.property("postbacks", []);
					ob[0].should.have.property("app_id", APP_ID);
					setTimeout(done, 100)
				});
			});
		});
	});
	
	describe('Click Campaign', function(){
		describe('Incorrect code', function(){
			it('should not find campaign', function(done){
				request
				.get('/i/campaign/click/1234567890')
				.expect(404, done);
			});
		});
		
		describe('First click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGN_ID)
				.set('User-Agent',userAgent1)
				.expect(302, done);
			});
		});
		
        describe('Verify update campaign data', function(){
			it('should have first click', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(1);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("data");
					verifyMetrics(err, ob[0].data, done, {meta:{b: [ '[b]_chrome' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android' ]}, "[l]_unknown":{aclk: 1, clk: 1}, "[b]_chrome":{aclk: 1, clk: 1}, "[pl]_android":{aclk: 1, clk: 1}});
				});
			});
		});
		
		describe('Asist click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGN_ID)
				.set('User-Agent',userAgent1)
				.expect(302, done);
			});
		});
        
        describe('Verify update campaign data', function(){
			it('should 2 asist clicks', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(1);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("data");
					verifyMetrics(err, ob[0].data, done, {meta:{b: [ '[b]_chrome' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android' ]}, "[l]_unknown":{aclk: 2, clk: 1}, "[b]_chrome":{aclk: 2, clk: 1}, "[pl]_android":{aclk: 2, clk: 1}});
				});
			});
		});
		
		describe('Another click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGN_ID)
				.set('User-Agent',userAgent2)
				.expect(302)
				.end(function(err, res){
					res.headers.should.have.property("campaign-user");
					CAMPAIGN_USER = res.headers["campaign-user"];
					done();
				});
			});
		});
		
		describe('Verify update campaign', function(){
			it('should have new click', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(1);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("data");
					verifyMetrics(err, ob[0].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2}, "[b]_chrome":{aclk: 2, clk: 1}, "[pl]_android":{aclk: 2, clk: 1}, "[b]_chromium":{aclk: 1, clk: 1}, "[pl]_ios":{aclk: 1, clk: 1}});
				});
			});
		});
	});
    
    describe('Attribute Campaign without fingerprint', function(){
		describe('by matching ip', function(){
			it('should success', function(done){
				var params = {"_os": "Windows"};
				request
				.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&begin_session=1&metrics="+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000)
				});
			});
		});
		
		describe('Verify ip attribution', function(){
			it('should have 1 install', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(1);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("data");
					verifyMetrics(err, ob[0].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2, ins:1, ses:1}, "[b]_chrome":{aclk: 2, clk: 1, ins:1, ses:1}, "[pl]_android":{aclk: 2, clk: 1, ins:1, ses:1}, "[b]_chromium":{aclk: 1, clk: 1}, "[pl]_ios":{aclk: 1, clk: 1}});
				});
			});
		});
		
		describe('by direct attribution', function(){
			it('should success', function(done){
				request
				.get('/i?device_id='+DEVICE_ID+'1&app_key='+APP_KEY+"&begin_session=1&campaign_id="+CAMPAIGN_ID+"&campaign_user="+CAMPAIGN_USER)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000)
				});
			});
		});
		
		describe('Verify direct attribution', function(){
			it('should have 2 installs', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(1);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("data");
					verifyMetrics(err, ob[0].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2, ins:2, ses:1}, "[b]_chrome":{aclk: 2, clk: 1, ins:1, ses:1}, "[pl]_android":{aclk: 2, clk: 1, ins:1, ses:1}, "[b]_chromium":{aclk: 1, clk: 1, ins:1}, "[pl]_ios":{aclk: 1, clk: 1, ins:1}});
				});
			});
		});
		
		describe('by postback attribution', function(){
			it('should success', function(done){
				request
				.get('/i?device_id='+DEVICE_ID+'2&app_key='+APP_KEY+"&begin_session=1&campaign_id="+CAMPAIGN_ID+"&campaign_platform=Android&campaign_browser=Chrome&campaign_locale=unknown")
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000)
				});
			});
		});
		
		describe('Verify postback attribution', function(){
			it('should have 3 installs', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(1);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("data");
					verifyMetrics(err, ob[0].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2, ins:3, ses:1}, "[b]_chrome":{aclk: 2, clk: 1, ins:2, ses:1}, "[pl]_android":{aclk: 2, clk: 1, ins:2, ses:1}, "[b]_chromium":{aclk: 1, clk: 1, ins:1}, "[pl]_ios":{aclk: 1, clk: 1, ins:1}});
				});
			});
		});
	});
    
    describe('Click Other Campaign', function(){
		describe('First click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGNF_ID)
				.set('User-Agent',userAgent1)
				.expect(302, done);
			});
		});
		
		describe('Verify update campaign', function(){
			it('should have first click', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID, CAMPAIGNF_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[1].should.have.property("_id", CAMPAIGNF_ID);
					ob[1].should.have.property("data");
					verifyMetrics(err, ob[1].data, done, {meta:{b: [ '[b]_chrome' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android' ]}, "[l]_unknown":{aclk: 1, clk: 1}, "[b]_chrome":{aclk: 1, clk: 1}, "[pl]_android":{aclk: 1, clk: 1}});
				});
			});
		});
		
		describe('Asist click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGNF_ID)
				.set('User-Agent',userAgent1)
				.expect(302, done);
			});
		});
		
		describe('Verify update campaign', function(){
			it('should 2 asist clicks', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID, CAMPAIGNF_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[1].should.have.property("_id", CAMPAIGNF_ID);
					ob[1].should.have.property("data");
					verifyMetrics(err, ob[1].data, done, {meta:{b: [ '[b]_chrome' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android' ]}, "[l]_unknown":{aclk: 2, clk: 1}, "[b]_chrome":{aclk: 2, clk: 1}, "[pl]_android":{aclk: 2, clk: 1}});
				});
			});
		});
		
		describe('Another click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGNF_ID)
				.set('User-Agent',userAgent2)
				.expect(302)
				.end(function(err, res){
					res.headers.should.have.property("campaign-user");
					CAMPAIGNF_USER = res.headers["campaign-user"];
					done();
				});
			});
		});
		
		describe('Verify update campaign', function(){
			it('should have new click', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID, CAMPAIGNF_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[1].should.have.property("_id", CAMPAIGNF_ID);
					ob[1].should.have.property("data");
					verifyMetrics(err, ob[1].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2}, "[b]_chrome":{aclk: 2, clk: 1}, "[pl]_android":{aclk: 2, clk: 1}, "[b]_chromium":{aclk: 1, clk: 1}, "[pl]_ios":{aclk: 1, clk: 1}});
				});
			});
		});
	});
	
	describe('Attribute Campaign with fingerprint', function(){
        describe('by matching incorrect fingerprint', function(){
			it('should success', function(done){
				var params = {"_os": "Windows"};
				request
				.get('/i?device_id='+DEVICE_ID+'a&app_key='+APP_KEY+"&begin_session=1&metrics="+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000)
				});
			});
		});
		
		describe('Verify no attribution', function(){
			it('should have no install', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID, CAMPAIGNF_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[1].should.have.property("_id", CAMPAIGNF_ID);
					ob[1].should.have.property("data");
					verifyMetrics(err, ob[1].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2}, "[b]_chrome":{aclk: 2, clk: 1}, "[pl]_android":{aclk: 2, clk: 1}, "[b]_chromium":{aclk: 1, clk: 1}, "[pl]_ios":{aclk: 1, clk: 1}});
				});
			});
		});
        
		describe('by matching fingerprint', function(){
			it('should success', function(done){
				var params = {"_os": "Android"};
				request
				.get('/i?device_id='+DEVICE_ID+'b&app_key='+APP_KEY+"&begin_session=1&metrics="+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000)
				});
			});
		});
		
		describe('Verify fingerprint attribution', function(){
			it('should have 1 install', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID, CAMPAIGNF_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[1].should.have.property("_id", CAMPAIGNF_ID);
					ob[1].should.have.property("data");
					verifyMetrics(err, ob[1].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2, ins:1, ses:1}, "[b]_chrome":{aclk: 2, clk: 1, ins:1, ses:1}, "[pl]_android":{aclk: 2, clk: 1, ins:1, ses:1}, "[b]_chromium":{aclk: 1, clk: 1}, "[pl]_ios":{aclk: 1, clk: 1}});
				});
			});
		});
		
		describe('by direct attribution', function(){
			it('should success', function(done){
				request
				.get('/i?device_id='+DEVICE_ID+'c&app_key='+APP_KEY+"&begin_session=1&campaign_id="+CAMPAIGNF_ID+"&campaign_user="+CAMPAIGNF_USER)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000)
				});
			});
		});
		
		describe('Verify direct attribution', function(){
			it('should have 2 installs', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID, CAMPAIGNF_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[1].should.have.property("_id", CAMPAIGNF_ID);
					ob[1].should.have.property("data");
					verifyMetrics(err, ob[1].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2, ins:2, ses:1}, "[b]_chrome":{aclk: 2, clk: 1, ins:1, ses:1}, "[pl]_android":{aclk: 2, clk: 1, ins:1, ses:1}, "[b]_chromium":{aclk: 1, clk: 1, ins:1}, "[pl]_ios":{aclk: 1, clk: 1, ins:1}});
				});
			});
		});
		
		describe('by postback attribution', function(){
			it('should success', function(done){
				request
				.get('/i?device_id='+DEVICE_ID+'d&app_key='+APP_KEY+"&begin_session=1&campaign_id="+CAMPAIGNF_ID+"&campaign_platform=Android&campaign_browser=Chrome&campaign_locale=unknown")
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000)
				});
			});
		});
		
		describe('Verify postback attribution', function(){
			it('should have 3 installs', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID, CAMPAIGNF_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[1].should.have.property("_id", CAMPAIGNF_ID);
					ob[1].should.have.property("data");
					verifyMetrics(err, ob[1].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2, ins:3, ses:1}, "[b]_chrome":{aclk: 2, clk: 1, ins:2, ses:1}, "[pl]_android":{aclk: 2, clk: 1, ins:2, ses:1}, "[b]_chromium":{aclk: 1, clk: 1, ins:1}, "[pl]_ios":{aclk: 1, clk: 1, ins:1}});
				});
			});
		});
	});
	
	describe('Track Sessions', function(){
		describe('Start android session', function(){
			it('should success', function(done){
				request
				.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&begin_session=1")
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000)
				});
			});
		});
		
		describe('Verify android session', function(){
			it('should have +1 install', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID, CAMPAIGNF_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("data");
					verifyMetrics(err, ob[0].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2, ins:3, ses:2}, "[b]_chrome":{aclk: 2, clk: 1, ins:2, ses:2}, "[pl]_android":{aclk: 2, clk: 1, ins:2, ses:2}, "[b]_chromium":{aclk: 1, clk: 1, ins:1}, "[pl]_ios":{aclk: 1, clk: 1, ins:1}});
				});
			});
		});
		
		describe('Start ios session', function(){
			it('should success', function(done){
				request
				.get('/i?device_id='+DEVICE_ID+'1&app_key='+APP_KEY+"&begin_session=1")
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000)
				});
			});
		});
		
		describe('Verify ios session', function(){
			it('should have +1 install', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&data="+JSON.stringify([CAMPAIGN_ID, CAMPAIGNF_ID]))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.not.be.empty;
					ob.should.have.lengthOf(2);
					ob[0].should.have.property("_id", CAMPAIGN_ID);
					ob[0].should.have.property("data");
					verifyMetrics(err, ob[0].data, done, {meta:{b: [ '[b]_chrome','[b]_chromium' ], l: [ '[l]_unknown' ], pl: [ '[pl]_android','[pl]_ios' ]}, "[l]_unknown":{aclk: 3, clk: 2, ins:3, ses:3}, "[b]_chrome":{aclk: 2, clk: 1, ins:2, ses:2}, "[pl]_android":{aclk: 2, clk: 1, ins:2, ses:2}, "[b]_chromium":{aclk: 1, clk: 1, ins:1, ses:1}, "[pl]_ios":{aclk: 1, clk: 1, ins:1, ses:1}});
				});
			});
		});
	});
	
	describe('Redirect Campaign', function(){
		describe('Update redirect links', function(){
			it('should success', function(done){
				var args = {
					_id: CAMPAIGN_ID,
					links: {android:"http://android/", ios:"http://ios/"}
				};
				request
				.get('/i/campaign/update?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Success");
					setTimeout(done, 100)
				});
			});
		});
		
		describe('Android click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGN_ID)
				.set('User-Agent',userAgent1)
				.expect(function(res){
                    res.header.location.should.startWith("http://android/");
                })
				.expect(302, done);
			});
		});
		
		describe('IOS click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGN_ID)
				.set('User-Agent',userAgent2)
                .expect(function(res){
                    res.header.location.should.startWith("http://ios/");
                })
				.expect(302, done);
			});
		});
	});
	
	describe('Redirect Custom URL scheme', function(){
		describe('Update redirect links', function(){
			it('should success', function(done){
				var args = {
					_id: CAMPAIGN_ID,
					links: {android:"http://android", "android-custom-scheme": "android://test", ios:"http://ios", "ios-custom-scheme": "ios://test"}
				};
				request
				.get('/i/campaign/update?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Success");
					setTimeout(done, 100)
				});
			});
		});
		
		describe('Android click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGN_ID)
				.set('User-Agent',userAgent1)
                .expect(function(res){
                    res.header.location.should.startWith("/campaign/android/?url=http%3A%2F%2Fandroid%2F");
                })
				.expect(302, done);
			});
		});
		
		describe('IOS click', function(){
			it('should redirect', function(done){
				request
				.get('/i/campaign/click/'+CAMPAIGN_ID)
				.set('User-Agent',userAgent2)
                .expect(function(res){
                    res.header.location.should.startWith("/campaign/ios/?url=http%3A%2F%2Fios%2F");
                })
				.expect(302, done);
			});
		});
	});
	
	describe('Delete Campaign', function(){
		describe('No ID', function(){
			it('should Not enough args', function(done){
				var args = {
				};
				request
				.get('/i/campaign/delete?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Not enough args");
					setTimeout(done, 100)
				});
			});
		});
		describe('Delete', function(){
			it('should success', function(done){
				var args = {
					_id: CAMPAIGN_ID
				};
				request
				.get('/i/campaign/delete?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(args))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property("result", "Success");
					setTimeout(done, 100)
				});
			});
		});
	//[{"_id":"2c54a3ac541153f4b3077f2a3975e43547b67426","name":"test","link":"test","cost":0,"costtype":"click","links":{},"postbacks":[],"app_id":"552b79d251f2712c3ae7bce1","data":{}}]
		describe('Verify delete', function(){
			it('should success', function(done){
				request
				.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.lengthOf(2);
					setTimeout(done, 100)
				});
			});
		});
	});
	describe('Reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID, period:"reset"};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
    describe('Verify reset', function(){
		it('should success', function(done){
			request
			.get('/o/campaign?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
});