var plugin = {},
	common = require('../../../api/utils/common.js'),
    log = common.log('block:api'),
    plugins = require('../../pluginManager.js');

(function (plugin) {
    function updateUserMetric(params, userProps){
        common.updateAppUser(params, {'$set': userProps});
    }
	plugins.register("/sdk", function(ob, callback){
        var params = ob.params;
            if(params.app && params.app.blocks){
                var blocks = params.app.blocks;
                
                //process metrics before comparing
                var map = {
                    "_os": "p",
                    "_os_version": "pv",
                    "_device": "d",
                    "_resolution": "r",
                    "_carrier": "c",
                    "_app_version": "av",
                    "_density": "dnst",
                    "_locale": "lo",
                    "_lang": "la",
                    "_store": "src",
                    "_browser": "brw"
                };
                var update = {};
                var user = JSON.parse(JSON.stringify(params.app_user));
                if(params.qstring.metrics){
                    var regexp = /[\-_]+/;
                    for(var i in map){
                        if(params.qstring.metrics[i]){
                            var metric;
                            switch(i){
                                case "_app_version":
                                    metric = params.qstring.metrics["_app_version"]+"";
                                    if(metric.indexOf('.') === -1)
                                        mmetric += ".0";
                                    metric = (metric+"").replace(/^\$/, "").replace(/\./g, ":");
                                    break;
                                case "_locale":
                                    metric = (params.qstring.metrics["_locale"]+"").toLowerCase().split(regexp)[0];
                                    break;
                                case "_store":
                                    metric = (params.qstring.metrics._store+"").replace(/\./g, '&#46;');
                                    break;
                                default:
                                    metric = (params.qstring.metrics[i]+"").replace(/^\$/, "").replace(/\./g, ":");
                            }
                            //if metrics don't match store them for possible update
                            //and use to compare with block rules
                            if(metric != user[map[i]]){
                                update[map[i]] = metric;
                                user[map[i]] = metric;
                            }
                        }
                    }
                }
                user.did = params.qstring.device_id;
                user.ip = params.ip_address;
                log.d("current user", user, update);
                for(var i = 0; i <  blocks.length; i++){
                    try{
                        blocks[i].rule = JSON.parse(blocks[i].rule);
                    }
                    catch(e){
                        log.w("skipping rule", blocks[i]);
                        continue;
                    }
                    if(blocks[i].type == "all"){
                        var shouldBlock = true;
                        for(var prop in blocks[i].rule){
                            var parts = prop.split(".");
                            if(parts[0] && parts[1]){
                                if(parts[0] == "up"){
                                    if(!user[parts[1]] || blocks[i].rule[prop].$in.indexOf(user[parts[1]]) === -1){
                                        shouldBlock = false;
                                    }
                                }
                                else if(!user[parts[0]] || !user[parts[0]][parts[1]] || blocks[i].rule[prop].$in.indexOf(user[parts[0]][parts[1]]) === -1){
                                    shouldBlock = false;
                                }
                            }
                        }
                        if(shouldBlock){
                            log.i("blocked", params.qstring, "by", blocks[i], "from", user);
                            //blocking request
                            params.cancelRequest = true;
                    
                            //let's update the metric ourselves
                            if(Object.keys(update).length > 0){
                                updateUserMetric(params, update);
                                update = {};
                            }
                            
                            return true;
                        }
                    }
                    else if(blocks[i].type == "session" && (params.qstring.begin_session || params.qstring.session_duration || params.qstring.end_session)){
                        var shouldBlock = true;
                        for(var prop in blocks[i].rule){
                            var parts = prop.split(".");
                            if(parts[0] && parts[1]){
                                if(parts[0] == "up"){
                                    if(!user[parts[1]] || blocks[i].rule[prop].$in.indexOf(user[parts[1]]) === -1){
                                        shouldBlock = false;
                                    }
                                }
                                else if(!user[parts[0]] || !user[parts[0]][parts[1]] || blocks[i].rule[prop].$in.indexOf(user[parts[0]][parts[1]]) === -1){
                                    shouldBlock = false;
                                }
                            }
                        }
                        if(shouldBlock){
                            log.i("removed sessions", params.qstring, "by", blocks[i], "from", user);
                            //remove session data
                            delete params.qstring.begin_session;
                            delete params.qstring.session_duration;
                            delete params.qstring.end_session;
                    
                            //let's update the metric ourselves
                            if(Object.keys(update).length > 0){
                                updateUserMetric(params, update);
                                update = {};
                            }
                        }
                    }
                    else if(blocks[i].type == "event" && params.qstring.events && params.qstring.events.length && Array.isArray(params.qstring.events)){
                        var shouldBlock = true;
                        var eventRules = {};
                        for(var prop in blocks[i].rule){
                            var parts = prop.split(".");
                            if(parts[0] && parts[1]){
                                if(parts[0] == "up"){
                                    if(!user[parts[1]] || blocks[i].rule[prop].$in.indexOf(user[parts[1]]) === -1){
                                        shouldBlock = false;
                                    }
                                }
                                else if(parts[0] == "sg"){
                                    eventRules[prop] = blocks[i].rule[prop];
                                }
                                else if(!user[parts[0]] || !user[parts[0]][parts[1]] || blocks[i].rule[prop].$in.indexOf(user[parts[0]][parts[1]]) === -1){
                                    shouldBlock = false;
                                }
                            }
                            else{
                                eventRules[prop] = blocks[i].rule[prop];
                            }
                        }
                        if(shouldBlock){
                            //at this point we know that all user rules have been matched
                            //now we only need to remove events which match event rules
                            //filter out events
                            params.qstring.events = params.qstring.events.filter(function(event){
                                if(blocks[i].key == event.key || blocks[i].key == "*"){
                                    var notRemove = false;
                                    for(var prop in eventRules){
                                        var parts = prop.split(".");
                                        if(parts[0] == "sg"){
                                            if(!event.segmentation || eventRules[prop].$in.indexOf(event.segmentation[parts[1]]) === -1 ){
                                                notRemove = true;
                                            }
                                        }
                                        else if(eventRules[prop].$in.indexOf(event[parts[0]]) === -1 ){
                                            notRemove = true;
                                        }
                                    }
                                    if(!notRemove)
                                        log.i("removing event", event, "by", blocks[i], "from", user);
                                    return notRemove;
                                }
                                return true;
                            });
                        }
                    }
                }
            }
        return true;
	});
    
    plugins.register("/i/blocks", function(ob){
		var params = ob.params;
        var paths = ob.paths;
        var validateUserForDataWriteAPI = ob.validateUserForDataWriteAPI;
		validateUserForDataWriteAPI(params, function(){
			if(!params.qstring.app_id){
                common.returnMessage(params, 400, 'Provide app_id');
				return false;
            }
            switch (paths[3]) {
                case 'create':
                    var data = {}
                    if(params.qstring.blocks){
                        try{
                            data = JSON.parse(params.qstring.blocks);
                        }
                        catch(err){
                            console.log("Error parsing blocks", params.qstring.blocks);
                        }
                    } 
                    if(!data._id){
                        data._id = new common.db.ObjectID().toHexString();
                    }
                    data.rule = JSON.stringify(data.rule);
                    if(Object.keys(data).length > 0){
                        common.db.collection("apps").update({_id:common.db.ObjectID(params.qstring.app_id+"")}, {$addToSet:{blocks:data}}, {upsert:true}, function(err, res){
                            if(err)
                                common.returnMessage(params, 400, 'Error adding rule');
                            else{
                                data.app_id = params.qstring.app_id;
                                plugins.dispatch("/systemlogs", {params:params, action:"block_added", data:data});
                                common.returnMessage(params, 200, 'Success');
                            }
                        });
                    }
                    else{
                        common.returnMessage(params, 400, 'No rule to add');
                    }
                    break;
                case 'delete':
                    if(params.qstring.block_id){
                        common.db.collection("apps").findOne({_id:common.db.ObjectID(params.qstring.app_id+"")}, function(err, app){
                            var block = {};
                            if(app && app.blocks && app.blocks.length){
                                for(var i = 0; i < app.blocks.length; i++){
                                    if(app.blocks[i]._id == params.qstring.block_id){
                                        block = app.blocks[i];
                                        break;
                                    }
                                }
                                block._id = params.qstring.block_id;
                                block.app_id = params.qstring.app_id;
                                common.db.collection("apps").update({_id:common.db.ObjectID(params.qstring.app_id+"")}, {$pull:{blocks:{_id:params.qstring.block_id}}}, {upsert:true}, function(err, res){
                                    if(err)
                                        common.returnMessage(params, 400, 'Error deleting rule');
                                    else{
                                        plugins.dispatch("/systemlogs", {params:params, action:"block_deleted", data:block});
                                        common.returnMessage(params, 200, 'Success');
                                    }
                                });
                            }
                        });
                    }
                    else
                        common.returnMessage(params, 400, 'No rule id');
                    break;
                default:
                    common.returnMessage(params, 400, 'Incorrect path');
            }
        });
        return true;
    });
    
    plugins.register("/o/blocks", function(ob){
		var params = ob.params;
        var validateUserForDataWriteAPI = ob.validateUserForDataWriteAPI;
		validateUserForDataWriteAPI(params, function(){
			if(!params.qstring.app_id){
                common.returnMessage(params, 400, 'Provide app_id');
				return false;
            }
            common.db.collection('apps').findOne({_id:common.db.ObjectID(params.qstring.app_id+"")}, {_id:0, blocks:1}, function(err, app){
                common.returnOutput(params, (app && app.blocks) ? app.blocks : []);
            });
        });
        return true;
    });
    
    plugins.register("/i/apps/reset", function(ob){
        var appId = ob.appId;
        common.db.collection("apps").update({_id:common.db.ObjectID(appId+"")}, {$unset:{blocks:""}}, {upsert:true}, function(err, res){});
    });

}(plugin));

module.exports = plugin;