var request = require('supertest');
var should = require('should');
var testUtils = require("../../test/testUtils");
request = request(testUtils.url);
var escape_html = require('escape-html');

var APP_KEY = "";
var API_KEY_ADMIN = "";
var APP_ID = "";
var APP_VERSION = "2.1";
var APP_VERSION2 = "2.0";
var APP_VERSION22 = "2.2";
var DEVICE_ID = "1234567890";
var BLOCKED_DEVICE_ID = "1234567890";
var BLOCKS = [];
var IP = "192.168.1.1";

describe('Testing Blocking requests', function(){
	
	describe('Empty blocks', function(){
		it('should success', function(done){
			API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 1 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:1,e:1,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 1 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:1});
				});
			});
		});
    });
    
    //Whole request blocking
    
    describe('Add blocking all by device_id', function(){
		it('should success', function(done){
            var block = {type:"all", rule:{"up.did":{"$in":[BLOCKED_DEVICE_ID]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
            var rule = {"up.did":{"$in":[BLOCKED_DEVICE_ID]}};
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				var block = ob[0];
                block.should.have.property("_id");
                block.should.have.property("type", "all");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we cannot make requests with device_id', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 1 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:1,e:1,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 1 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:1});
				});
			});
		});
    });
    
    describe('Remove blocking all by device_id', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Add blocking all by ip', function(){
		it('should success', function(done){
            var block = {type:"all", rule:{"up.ip":{"$in":[IP]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
                var block = ob[0];
                var rule = {"up.ip":{"$in":[IP]}};
                block.should.have.property("_id");
                block.should.have.property("type", "all");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked device id', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 2 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:2,e:2,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 2 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:2});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with blocked ip', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params)+"&ip_address="+IP)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 2 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:2,e:2,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 2 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:2});
				});
			});
		});
    });
    
    describe('Remove blocking all by ip', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Add blocking all by app_version', function(){
		it('should success', function(done){
            var block = {type:"all", rule:{"up.av":{"$in":[APP_VERSION.replace(".", ":")]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
                ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.av":{"$in":[APP_VERSION.replace(".", ":")]}};
                block.should.have.property("_id");
                block.should.have.property("type", "all");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked ip', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params)+"&ip_address="+IP)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 3 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:3,e:3,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 3 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:3});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with app version in metrics', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var metrics = {
                    "_app_version": APP_VERSION
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 3 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:3,e:3,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 3 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:3});
				});
			});
		});
    });
    
    describe('verify we cannot make requests from stored app_version', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 3 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:3,e:3,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 3 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:3});
				});
			});
		});
    });
    
    describe('Remove blocking all by app version', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});

	describe('Add blocking all by platform', function(){
		it('should success', function(done){
            var block = {type:"all", rule:{"up.p":{"$in":["Android"]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.p":{"$in":["Android"]}};
                block.should.have.property("_id");
                block.should.have.property("type", "all");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked app_version', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params)+"&ip_address="+IP)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:4,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with platform in metrics', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var metrics = {
                    "_os": "Android"
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:4,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('verify we cannot make requests from stored platform', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:4,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('Remove blocking all by platform', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Add blocking all by custom property', function(){
		it('should success', function(done){
            var block = {type:"all", rule:{"custom.key":{"$in":["val"]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"custom.key":{"$in":["val"]}};
                block.should.have.property("_id");
                block.should.have.property("type", "all");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked platform', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 5 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:5,e:5,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 5 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:5});
				});
			});
		});
    });
    
    describe('Apply custom property to user', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var user = {custom:{key:"val"}};
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&user_details='+JSON.stringify(user))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with custom property stored', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 5 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:5,e:6,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 5 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:5});
				});
			});
		});
    });
    
    //Reset app
    describe('Reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID, period:"reset"};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
    
    //Blocking sessions only
    
    describe('Verify we can make requests', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 1 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:1,e:1,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 1 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:1});
				});
			});
		});
    });
    
    describe('Add blocking session by device_id', function(){
		it('should success', function(done){
            var block = {type:"session", rule:{"up.did":{"$in":[BLOCKED_DEVICE_ID]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.did":{"$in":[BLOCKED_DEVICE_ID]}};
                block.should.have.property("_id");
                block.should.have.property("type", "session");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we cannot make requests with device_id', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 1 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:1,e:2,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 2 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:2});
				});
			});
		});
    });
    
    describe('Remove blocking session by device_id', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
     
    describe('Add blocking session by ip', function(){
		it('should success', function(done){
            var block = {type:"session", rule:{"up.ip":{"$in":[IP]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.ip":{"$in":[IP]}};
                block.should.have.property("_id");
                block.should.have.property("type", "session");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked device id', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 2 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:2,e:3,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 3 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:3});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with blocked ip', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params)+"&ip_address="+IP)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 2 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:2,e:4,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('Remove blocking session by ip', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
     
    describe('Add blocking session by app_version', function(){
		it('should success', function(done){
            var block = {type:"session", rule:{"up.av":{"$in":[APP_VERSION.replace(".", ":")]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.av":{"$in":[APP_VERSION.replace(".", ":")]}};
                block.should.have.property("_id");
                block.should.have.property("type", "session");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked ip', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params)+"&ip_address="+IP)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 3 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:3,e:5,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 5 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:5});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with app version in metrics', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var metrics = {
                    "_app_version": APP_VERSION
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 3 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:3,e:6,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 6 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:6});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests from stored app_version', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 3 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:3,e:7,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 7 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:7});
				});
			});
		});
    });
    
    describe('Remove blocking session by app version', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});

	describe('Add blocking session by platform', function(){
		it('should success', function(done){
            var block = {type:"session", rule:{"up.p":{"$in":["Android"]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
            var block = {"session":{"ip":[], "device_id":[], "app_version":[], "platform":["android"]}};
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.p":{"$in":["Android"]}};
                block.should.have.property("_id");
                block.should.have.property("type", "session");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked app_version', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params)+"&ip_address="+IP)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:8,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 8 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:8});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with platform in metrics', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var metrics = {
                    "_os": "Android"
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:9,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 9 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:9});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests from stored platform', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:10,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 10 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:10});
				});
			});
		});
    });
    
    describe('Remove blocking session by platform', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Add blocking session by custom property', function(){
		it('should success', function(done){
            var block = {type:"session", rule:{"custom.key":{"$in":["val"]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"custom.key":{"$in":["val"]}};
                block.should.have.property("_id");
                block.should.have.property("type", "session");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked platform', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 5 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:5,e:11,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 11 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:11});
				});
			});
		});
    });
    
    describe('Apply custom property to user', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var user = {custom:{key:"val"}};
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&user_details='+JSON.stringify(user))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with custom property stored', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 5 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:5,e:13,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 12 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:12});
				});
			});
		});
    });
    
    //Reset app
    describe('Reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID, period:"reset"};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
    
    //Blocking events only
    
    describe('Verify we can make requests', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 1 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:1,e:1,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 1 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:1});
				});
			});
		});
    });
    
    describe('Add blocking event by device_id', function(){
		it('should success', function(done){
            var block = {type:"event", key:"*", rule:{"up.did":{"$in":[BLOCKED_DEVICE_ID]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.did":{"$in":[BLOCKED_DEVICE_ID]}};
                block.should.have.property("_id");
                block.should.have.property("type", "event");
                block.should.have.property("key", "*");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we cannot make requests with device_id', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 2 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:2,e:2,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 1 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:1});
				});
			});
		});
    });
    
    describe('Remove blocking event by device_id', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});

    describe('Add blocking event by ip', function(){
		it('should success', function(done){
            var block = {type:"event", key:"*", rule:{"up.ip":{"$in":[IP]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.ip":{"$in":[IP]}};
                block.should.have.property("_id");
                block.should.have.property("type", "event");
                block.should.have.property("key", "*");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked device id', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 3 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:3,e:3,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 2 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:2});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with blocked ip', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params)+"&ip_address="+IP)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:4,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 2 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:2});
				});
			});
		});
    });
    
    describe('Remove blocking event by ip', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});

    describe('Add blocking event by app_version', function(){
		it('should success', function(done){
            var block = {type:"event", key:"*", rule:{"up.av":{"$in":[APP_VERSION.replace(".",":")]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.av":{"$in":[APP_VERSION.replace(".",":")]}};
                block.should.have.property("_id");
                block.should.have.property("type", "event");
                block.should.have.property("key", "*");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked ip', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params)+"&ip_address="+IP)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 5 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:5,e:5,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 3 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:3});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with app version in metrics', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var metrics = {
                    "_app_version": APP_VERSION
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 6 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:6,e:6,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 3 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:3});
				});
			});
		});
    });
    
    describe('verify we cannot make requests from stored app_version', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 7 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:7,e:7,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 3 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:3});
				});
			});
		});
    });

    describe('Remove blocking event by app version', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});

	describe('Add blocking event by platform', function(){
		it('should success', function(done){
            var block = {type:"event", key:"*", rule:{"up.p":{"$in":["Android"]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
            var block = {"event":{"ip":[], "device_id":[], "app_version":[], "platform":["android"]}};
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"up.p":{"$in":["Android"]}};
                block.should.have.property("_id");
                block.should.have.property("type", "event");
                block.should.have.property("key", "*");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with previously blocked app_version', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params)+"&ip_address="+IP)
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 8 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:8,e:8,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with platform in metrics', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var metrics = {
                    "_os": "Android"
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 9 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:9,e:9,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('verify we cannot make requests from stored platform', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 10 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:10,e:10,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('Remove blocking event by platform', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});

    describe('Add blocking event by key', function(){
		it('should success', function(done){
            var block = {type:"event", key:"test", rule:{}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {};
                block.should.have.property("_id");
                block.should.have.property("type", "event");
                block.should.have.property("key", "test");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('verify we cannot make requests with blocked key', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 11 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:11,e:11,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('Remove blocking event by key', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});

    describe('Add blocking event by segment and value', function(){
		it('should success', function(done){
            var block = {type:"event", key:"test", rule:{"sg.test2":{"$in":["myval"]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"sg.test2":{"$in":["myval"]}};
                block.should.have.property("_id");
                block.should.have.property("type", "event");
                block.should.have.property("key", "test");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with same segment, but different value', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1,
                    "segmentation":{
                        "test2":"value"
                    }
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 12 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:12,e:12,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 5 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:5});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with blocked segment and value', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1,
                    "segmentation":{
                        "test2":"myval"
                    }
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 13 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:13,e:13,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 5 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:5});
				});
			});
		});
    });
    
    describe('Remove blocking event by key', function(){
		it('should success', function(done){
			request
			.get('/i/blocks/delete?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&block_id="+BLOCKS[0])
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Add blocking event by segment and value and platform', function(){
		it('should success', function(done){
            var block = {type:"event", key:"test", rule:{"sg.test2":{"$in":["myval"]}, "up.p":{"$in":["Android"]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(1);
				var block = ob[0];
                var rule = {"sg.test2":{"$in":["myval"]}, "up.p":{"$in":["Android"]}};
                block.should.have.property("_id");
                block.should.have.property("type", "event");
                block.should.have.property("key", "test");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with same segment and value, but different platform', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1,
                    "segmentation":{
                        "test2":"myval"
                    }
				}];
                var metrics = {
                    "_os": "iOS"
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 14 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:14,e:14,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 6 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:6});
				});
			});
		});
    });
    
    describe('Verify we can make requests with blocked platform but other segment and value', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1,
                    "segmentation":{
                        "test":"myval"
                    }
				}];
                var metrics = {
                    "_os": "Android"
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 15 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:15,e:15,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 7 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:7});
				});
			});
		});
    });
    
    describe('Verify we can make requests with blocked platform but without segmentation', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 16 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:16,e:16,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 8 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:8});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with blocked platform but other segment and value', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1,
                    "segmentation":{
                        "test2":"myval"
                    }
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 17 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:17,e:17,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 8 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:8});
				});
			});
		});
    });
    
    describe('Add blocking event by segment and value and platform and sum', function(){
		it('should success', function(done){
            var block = {type:"event", key:"test", rule:{"sg.test3":{"$in":["myval"]}, "up.p":{"$in":["Android"]}, "s":{"$in":[2]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(2);
				var block = ob[1];
                var rule = {"sg.test3":{"$in":["myval"]}, "up.p":{"$in":["Android"]}, "s":{"$in":[2]}};
                block.should.have.property("_id");
                block.should.have.property("type", "event");
                block.should.have.property("key", "test");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[1] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests with blocked platform and segmentation but without sum', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1,
                    "segmentation":{
                        "test3":"myval"
                    }
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 18 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:18,e:18,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 9 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:9});
				});
			});
		});
    });
    
    describe('Verify we can make requests with blocked platform and segmentation but other sum', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1,
                    "sum":1,
                    "segmentation":{
                        "test3":"myval"
                    }
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 19 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:19,e:19,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 10 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:10});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with blocked platform and segment and value and sum', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1,
                    "segmentation":{
                        "test2":"myval"
                    }
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 20 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:20,e:20,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 10 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:10});
				});
			});
		});
    });
    
    describe('Add blocking event by custom property', function(){
		it('should success', function(done){
            var block = {type:"event", key:"*", rule:{"custom.key":{"$in":["val"]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.with.lengthOf(3);
				var block = ob[2];
                var rule = {"custom.key":{"$in":["val"]}};
                block.should.have.property("_id");
                block.should.have.property("type", "event");
                block.should.have.property("key", "*");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[2] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Apply custom property to user', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var user = {custom:{key:"val"}};
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&user_details='+JSON.stringify(user))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with custom property stored', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 21 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:21,e:22,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 10 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:10});
				});
			});
		});
    });
    
    //Reset app
    describe('Reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID, period:"reset"};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
    
    //Testing upgrading property

    describe('Verify we can make requests', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 1 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:1,e:1,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 1 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:1});
				});
			});
		});
    });
    
    describe('Add blocking all by app version', function(){
		it('should success', function(done){
            var block = {type:"all", rule:{"up.av":{"$in":[APP_VERSION.replace(".",":")]}}};
			request
			.get('/i/blocks/create?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&blocks="+JSON.stringify(block))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.deepEqual({result:"Success"});
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify blocking rule', function(){
		it('should have provided settings', function(done){
            var rule = {"up.av":{"$in":[APP_VERSION.replace(".",":")]}};
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				var block = ob[0];
                block.should.have.property("_id");
                block.should.have.property("type", "all");
                block.should.have.property("rule", escape_html(JSON.stringify(rule)));
                BLOCKS[0] = block._id;
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify we can make requests without app version', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 2 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:2,e:2,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 2 event', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:2});
				});
			});
		});
    });
    
    describe('Verify we can make requests with another app version in metrics', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var metrics = {
                    "_app_version": APP_VERSION2
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 3 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:3,e:3,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 3 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:3});
				});
			});
		});
    });
    
    describe('verify we can make requests from another stored app_version', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:4,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('Verify we cannot make requests with app version in metrics', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var metrics = {
                    "_app_version": APP_VERSION
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:4,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('verify we upgraded app version to blocked one', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 4 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:4,e:4,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 4 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:4});
				});
			});
		});
    });
    
    describe('Verify we can make requests with new app version in metrics', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
                var metrics = {
                    "_app_version": APP_VERSION22
                };
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(metrics)+'&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 5 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:5,e:5,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 5 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:5});
				});
			});
		});
    });
    
    describe('verify we upgraded app version to new/unblocked one', function(){
		describe('GET request', function(){
			it('should success', function(done){
                var params = [{
					"key": "test",
					"count": 1
				}];
				request
				.get('/i?device_id='+BLOCKED_DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&events='+JSON.stringify(params))
				.expect(200)
				.end(function(err, res){
					if (err) return done(err);
					var ob = JSON.parse(res.text);
					ob.should.have.property('result','Success');
					setTimeout(done, 5000);
				});
			});
		});
		
		describe('Verify sessions', function(){
			it('should have 6 session with 1 user', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=sessions')
				.expect(200)
				.end(function(err, res){
					testUtils.validateSessionData(err, res, done, {meta:{"countries":["Unknown"],"f-ranges":["0"],"l-ranges":["0"]}, f:{"0":1},l:{"0":1},u:1,n:1,t:6,e:6,Unknown:true});
				});
			});
		});
        
        describe('verify specific event', function(){
			it('should have 6 events', function(done){
				request
				.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=events&event=test')
				.expect(200)
				.end(function(err, res){
					testUtils.validateEvents(err, res, done, {c:6});
				});
			});
		});
    });
    
    //Reset app
    describe('Reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID, period:"reset"};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
    
    describe('Empty blocks', function(){
		it('should success', function(done){
			request
			.get('/o/blocks?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.be.empty;
				setTimeout(done, 100)
			});
		});
	});
});