var plugin = {},
	common = require('../../../api/utils/common.js'),
	drill = require('./parts/data/drill.js'),
	crypto = require('crypto'),
    Promise = require("bluebird"),
    plugins = require('../../pluginManager.js');

plugins.setConfigs("drill", {
    list_limit: 100,
    custom_property_limit: 20,
    projection_limit: 10,
    big_list_limit: 128000,
    record_sessions: true,
    record_views: true,
    record_actions: true,
    record_pushes: true,
    record_crashes: true,
    record_star_rating: true
});

var countlyConfig;
try{
    countlyConfig = require('../config');
}
catch(err){
    countlyConfig = require('../../../api/config', 'dont-encolse');
}

(function (plugin) {
	plugin.openDrillDb = function(){
        if (!common.drillDb) {
            if (typeof countlyConfig.mongodb === "string") {
                countlyConfig.mongodb = countlyConfig.mongodb.replace(new RegExp('countly$'), 'countly_drill');
            } else{
                countlyConfig.mongodb.db = 'countly_drill';
            }

            common.drillDb = plugins.dbConnection(countlyConfig);
        }
	};
	plugin.drill = drill;
	plugins.register("/worker", plugin.openDrillDb);
	plugins.register("/o", function(ob){
		var params = ob.params;
		var validate = ob.validateUserForDataReadAPI;
		if(params.qstring.method == 'drill_bookmarks'){
			if (!params.qstring.event_key) {
                common.returnMessage(params, 400, 'Missing parameter event_key');
                return false;
            }
            validate(params, drill.fetchDrillBookmarks);
			return true;
		}
		else if(params.qstring.method == 'segmentation_meta'){
            validate(params, drill.fetchSegmentationMeta);
			return true;
		}
        else if(params.qstring.method == 'segmentation_big_meta'){
            validate(params, drill.fetchSegmentationBigMeta);
			return true;
		}
		else if(params.qstring.method == 'segmentation'){
            validate(params, drill.fetchSegmentationData);
			return true;
		}
        else if(params.qstring.method == 'segmentation_users'){
            validate(params, drill.fetchUserList);
			return true;
		}
	});

    plugins.register("/i/drill", function(ob){
		var params = ob.params,
			paths = ob.paths,
			validateUserForWriteAPI = ob.validateUserForWriteAPI;

        switch (paths[3]) {
            case 'add_bookmark':
                validateUserForWriteAPI(function (params) {
					if (!((params.member.user_of && params.member.user_of.indexOf(params.qstring.app_id) != -1) ||
                        (params.member.admin_of && params.member.admin_of.indexOf(params.qstring.app_id) != -1) || params.member.global_admin)) {
						common.returnMessage(params, 200, 'User does not have right to create bookmark');
						return false;
					}
					var argProps = {
							'app_id': 	{ 'required': true, 'type': 'String', 'min-length': 1 },
							'event_key': 	{ 'required': true, 'type': 'String', 'min-length': 1 },
							'query_obj': 	{ 'required': true, 'type': 'String', 'min-length': 1 },
							'query_text': 	{ 'required': false, 'type': 'String' },
							'by_val': 	{ 'required': false, 'type': 'String' },
							'by_val_text': 	{ 'required': false, 'type': 'String' }
						},
						newAtt = {};
					if (!(newAtt = common.validateArgs(params.qstring, argProps))) {
						common.returnMessage(params, 200, 'Not enough args');
						return false;
					}
                    var bookmarkId = crypto.createHash('md5').update(newAtt.app_id + newAtt.event_key + newAtt.query_obj + (newAtt.by_val || "")).digest('hex'),
					eventAppId = crypto.createHash('md5').update(newAtt.app_id + newAtt.event_key).digest('hex');

                    var setObj = {
                        event_app_id: eventAppId,
                        app_id: newAtt.app_id,
                        event_key: newAtt.event_key
                    };

                    if (newAtt.by_val && newAtt.by_val_text) {
                        setObj.by_val = newAtt.by_val;
                        setObj.by_val_text = newAtt.by_val_text;
                    }

                    if (newAtt.query_obj && newAtt.query_text) {
                        setObj.query_obj = newAtt.query_obj;
                        setObj.query_text = newAtt.query_text;
                    }

                    common.drillDb.collection('drill_bookmarks').update({"_id":bookmarkId}, {'$set': setObj}, {upsert: true}, function (err, bookmarks) {
                        setObj._id = bookmarkId;
                        plugins.dispatch("/systemlogs", {params:params, action:"drill_add_bookmark", data:setObj});
                        common.returnMessage(params, 200, "Success");
                    });
				}, params);
                break;
            case 'delete_bookmark':
                validateUserForWriteAPI(function (params) {
					if (!((params.member.user_of && params.member.user_of.indexOf(params.qstring.app_id) != -1) ||
                        (params.member.admin_of && params.member.admin_of.indexOf(params.qstring.app_id) != -1) || params.member.global_admin)) {
						common.returnMessage(params, 200, 'User does not have right to create bookmark');
						return false;
					}
					var argProps = {
							'app_id': 	{ 'required': true, 'type': 'String', 'min-length': 1 },
							'bookmark_id': 	{ 'required': true, 'type': 'String', 'min-length': 1 }
						},
						newAtt = {};

					if (!(newAtt = common.validateArgs(params.qstring, argProps))) {
						common.returnMessage(params, 200, 'Not enough args');
						return false;
					}
                    common.drillDb.collection('drill_bookmarks').findOne({"_id":newAtt.bookmark_id}, function (err, bookmark) {
                        common.drillDb.collection('drill_bookmarks').remove({"_id":newAtt.bookmark_id}, function (err, bookmarks) {
                            if(bookmark)
                                plugins.dispatch("/systemlogs", {params:params, action:"drill_remove_bookmark", data:bookmark});
                            common.returnMessage(params, 200, "Success");
                        });
                    });
				}, params);
                break;
            default:
                common.returnMessage(params, 404, 'Invalid endpoint');
                break;
        }
		return true;
	});

	plugins.register("/session/duration", function(ob){
        if(plugins.getConfig("drill").record_sessions){
            var params = ob.params;
            var session_duration = ob.session_duration;
            if(params.app_user["lsid"])
                drill.updateDuration(params.app_user["lsid"], session_duration, params);
        }
	});
    plugins.register("/plugins/drill", function(ob){
        drill.processCustomEvents(ob.dbAppUser, ob.params, ob.events);
	});
    plugins.register("/view/duration", function(ob){
        if(plugins.getConfig("drill").record_views){
            var params = ob.params;
            var duration = ob.duration;
            if(params.app_user["lvid"])
                drill.updateViewDuration(params.app_user["lvid"], duration, params);
        }
	});
    plugins.register("/sdk/end", function(ob){
		var params = ob.params;
        if(!params.cancelRequest){
            if (params.qstring.events) {
                drill.processEvents(params.app_user, params);
            }
            // If the last end_session is received less than 15 seconds ago we will ignore
            // current begin_session request and mark this user as having an ongoing session
            var lastEndSession = params.app_user && params.app_user[common.dbUserMap['last_end_session_timestamp']] || 0;

            if (plugins.getConfig("drill").record_sessions && params.qstring.begin_session && (params.qstring.ignore_cooldown || !lastEndSession || (params.time.timestamp - lastEndSession) > plugins.getConfig("api").session_cooldown)) {
                var timestamp = parseInt(params.qstring.timestamp);
                if (params.qstring.events && params.qstring.events.length && Array.isArray(params.qstring.events) && params.qstring.events[0].timestamp) {
                    var et = parseInt(params.qstring.events[0].timestamp);
                    if (et && (!timestamp || et < timestamp)) {
                        timestamp = et;
                    }
                }
                var events = [{
                    key: "[CLY]_session",
                    count: 1
                }];

                if (timestamp) {
                    events[0].timestamp = timestamp;
                }

                drill.processCustomEvents(params.app_user, params, events);
            }
        }
        return true;
	});
	plugins.register("/i/device_id", function(ob){
		var params = ob.params;
		var appId = params.app_id;
		var oldUid = ob.oldUser.uid;
		var newUid = ob.newUser.uid;
        if(oldUid != newUid){
            //change uid for existing events
            common.db.collection('events').findOne({'_id': common.db.ObjectID(appId)}, function(err, events) {
                if (!err && events && events.list) {
                    common.arrayAddUniq(events.list, plugins.internalDrillEvents);
                    for (var i = 0; i < events.list.length; i++) {
                        var collectionNameWoPrefix = crypto.createHash('sha1').update(events.list[i] + appId).digest('hex');
                        common.drillDb.collection("drill_events" + collectionNameWoPrefix).update({uid:oldUid}, {'$set': {uid:newUid}}, {multi:true}, function(err) {});
                    }
                }
            });
            for(var i = 0; i < plugins.internalDrillEvents.length; i++){
                common.drillDb.collection("drill_events" + crypto.createHash('sha1').update(plugins.internalDrillEvents[i] + appId).digest('hex')).update({uid:oldUid}, {'$set': {uid:newUid}}, {multi:true},function(err) {});
            }
        }
	});
	plugins.register("/i/apps/create", function(ob){
		// add 10 min TTL index to drill_cache collection
		common.drillDb.collection('drill_cache').ensureIndex({"lu": 1}, {expireAfterSeconds: 600}, function() {});
	});
	plugins.register("/i/apps/delete", function(ob){
        return new Promise(function(resolve, reject){
            var appId = ob.appId;
            common.db.collection('events').findOne({'_id': common.db.ObjectID(appId)}, function(err, events) {
                resolve();
                if (!err && events && events.list) {
                    common.arrayAddUniq(events.list, plugins.internalDrillEvents);
                    for (var i = 0; i < events.list.length; i++) {
                        var collectionNameWoPrefix = crypto.createHash('sha1').update(events.list[i] + appId).digest('hex');
                        common.drillDb.collection("drill_events" + collectionNameWoPrefix).drop(function() {});
                    }
                }
            });
            common.drillDb.collection('drill_bookmarks').remove({'app_id':appId}, function() {});
            common.drillDb.collection('drill_cache').remove({'app_id':appId}, function() {});
            common.drillDb.collection("drill_events" + crypto.createHash('sha1').update("[CLY]_session" + appId).digest('hex')).drop(function() {});
        });
	});

	plugins.register("/i/apps/reset", function(ob){
        return new Promise(function(resolve, reject){
           var appId = ob.appId;
            common.db.collection('events').findOne({'_id': common.db.ObjectID(appId)}, function(err, events) {
                resolve();
                if (!err && events && events.list) {
                    common.arrayAddUniq(events.list, plugins.internalDrillEvents);
                    for (var i = 0; i < events.list.length; i++) {
                        var collectionNameWoPrefix = crypto.createHash('sha1').update(events.list[i] + appId).digest('hex');
                        common.drillDb.collection("drill_events" + collectionNameWoPrefix).drop(function() {});
                    }
                }
            });
            common.drillDb.collection('drill_bookmarks').remove({'app_id':appId}, function() {});
            common.drillDb.collection('drill_cache').remove({'app_id':appId}, function() {});
            common.drillDb.collection("drill_events" + crypto.createHash('sha1').update("[CLY]_session" + appId).digest('hex')).drop(function() {});
        });
	});

    plugins.register("/i/apps/clear_all", function(ob){
        return new Promise(function(resolve, reject){
           var appId = ob.appId;
            common.db.collection('events').findOne({'_id': common.db.ObjectID(appId)}, function(err, events) {
                resolve();
                if (!err && events && events.list) {
                    common.arrayAddUniq(events.list, plugins.internalDrillEvents);
                    for (var i = 0; i < events.list.length; i++) {
                        var collectionNameWoPrefix = crypto.createHash('sha1').update(events.list[i] + appId).digest('hex');
                        common.drillDb.collection("drill_events" + collectionNameWoPrefix).drop(function() {});
                    }
                }
            });
            common.drillDb.collection('drill_cache').remove({'app_id':appId}, function() {});
            common.drillDb.collection("drill_events" + crypto.createHash('sha1').update("[CLY]_session" + appId).digest('hex')).drop(function() {});
        });
    });

    plugins.register("/i/apps/clear", function(ob){
		var appId = ob.appId;
		common.db.collection('events').findOne({'_id': common.db.ObjectID(appId)}, function(err, events) {
            if (!err && events && events.list) {
                common.arrayAddUniq(events.list, plugins.internalDrillEvents);
                for (var i = 0; i < events.list.length; i++) {
                    var collectionNameWoPrefix = crypto.createHash('sha1').update(events.list[i] + appId).digest('hex');
                    common.drillDb.collection("drill_events" + collectionNameWoPrefix).remove({ts:{$lt:ob.moment.valueOf()}}, function() {});
                }
            }
        });
        common.drillDb.collection('drill_cache').remove({'app_id':appId}, function() {});
		common.drillDb.collection("drill_events" + crypto.createHash('sha1').update("[CLY]_session" + appId).digest('hex')).remove({ts:{$lt:ob.moment.valueOf()}}, function() {});
        return true;
	});
}(plugin));

module.exports = plugin;