var drill = {},
    common = require('../../../../../api/utils/common.js'),
    log = common.log('drill:drill'),
    async = require('async'),
    moment = require('moment'),
	momentiso = require('moment-isocalendar'),
	countlyCommon = require('../../../../../api/lib/countly.common.js'),
	_ = require('underscore'),
    crypto = require('crypto'),
    plugins = require('../../../../pluginManager.js');

(function (drill) {

	drill.fetchSegmentationData = function(params) {
        //check request parameters if they all exist and in correct format
        if (!params.qstring.event) {
            common.returnMessage(params, 400, 'Missing request parameter: event');
            return false;
        }

        if (params.qstring.queryObject) {
            try {
                params.qstring.queryObject = JSON.parse(params.qstring.queryObject);
            } catch (SyntaxError) {
                console.log('Parsing custom queryObject failed!');
                common.returnMessage(params, 400, 'Bad request parameter: queryObject');
                return false;
            }
        } else {
            common.returnMessage(params, 400, 'Missing request parameter: queryObject');
            return false;
        }

        if (params.qstring.period) {
            //check if period comes from datapicker
            if (params.qstring.period.indexOf(",") !== -1) {
                try {
                    params.qstring.period = JSON.parse(params.qstring.period);
                } catch (SyntaxError) {
                    console.log('Parsing custom period failed!');
                    common.returnMessage(params, 400, 'Bad request parameter: period');
                    return false;
                }
            }
            else{
                switch (params.qstring.period){
                    case "month":
                    case "day":
                    case "yesterday":
                    case "hour":
                    case "7days":
                    case "30days":
                    case "60days":
                        break;
                    default:
                        common.returnMessage(params, 400, 'Bad request parameter: period');
                        return false;
                        break;
                }
            }
        } else {
            common.returnMessage(params, 400, 'Missing request parameter: period');
            return false;
        }

        if (params.qstring.bucket) {
            switch (params.qstring.bucket){
                case "hourly":
                case "daily":
                case "weekly":
                case "monthly":
                    break;
                default:
                    common.returnMessage(params, 400, 'Bad request parameter: bucket');
                    return false;
                    break;
            }
        } else {
            common.returnMessage(params, 400, 'Missing request parameter: bucket');
            return false;
        }

        countlyCommon.setPeriod(params.qstring.period);
        countlyCommon.setTimezone(params.appTimezone);

        var periodObj = countlyCommon.periodObj,
            queryObject = params.qstring.queryObject,//note that params.qstring.queryObject is JSON parsed during request check above
            projectionKey = params.qstring.projectionKey,
            resultObject = {data:{},meta:[]},
            now = params.time.now.toDate(),
            collectionName = "drill_events" + crypto.createHash('sha1').update(params.qstring.event + params.qstring.app_id).digest('hex'),
            cacheId,
            isCached = false;

        //create current period array if it does not exist
        if (!periodObj.currentPeriodArr) {
            periodObj.currentPeriodArr = [];

            //create a period array that starts from the beginning of the current year until today
            if (params.qstring.period == "month") {
                for (var i = 0; i < (now.getMonth() + 1); i++) {
                    var daysInMonth = moment().month(i).daysInMonth();

                    for (var j = 0; j < daysInMonth; j++) {
                        periodObj.currentPeriodArr.push(periodObj.activePeriod + "." + (i + 1) + "." + (j + 1));

                        // If current day of current month, just break
                        if ((i == now.getMonth()) && (j == (now.getDate() - 1))) {
                            break;
                        }
                    }
                }
            }
            //create a period array that starts from the beginning of the current month until today
            else if(params.qstring.period == "day") {
                for(var i = 0; i < now.getDate(); i++) {
                    periodObj.currentPeriodArr.push(periodObj.activePeriod + "." + (i + 1));
                }
            }
            //create one day period array
            else{
                periodObj.currentPeriodArr.push(periodObj.activePeriod);
            }
        }

        //get timestamps of start of days (DD-MM-YYYY-00:00) with respect to apptimezone for both beginning and end of period arrays
        var tmpArr;
        queryObject.ts = {};

        tmpArr = periodObj.currentPeriodArr[0].split(".");
        queryObject.ts.$gte = new Date(Date.UTC(parseInt( tmpArr[0]),parseInt(tmpArr[1])-1,parseInt(tmpArr[2]) ));
        queryObject.ts.$gte.setTimezone(params.appTimezone);
        queryObject.ts.$gte = queryObject.ts.$gte.getTime() + queryObject.ts.$gte.getTimezoneOffset()*60000;

        tmpArr = periodObj.currentPeriodArr[periodObj.currentPeriodArr.length - 1].split(".");
        queryObject.ts.$lt = new Date(Date.UTC(parseInt( tmpArr[0]),parseInt(tmpArr[1])-1,parseInt(tmpArr[2]) ));
        queryObject.ts.$lt.setDate(queryObject.ts.$lt.getDate() + 1);
        queryObject.ts.$lt.setTimezone(params.appTimezone);
        queryObject.ts.$lt = queryObject.ts.$lt.getTime() + queryObject.ts.$lt.getTimezoneOffset()*60000;

        if (projectionKey) {
            cacheId = common.crypto.createHash('sha1').update(
                    params.qstring.event +
                    params.qstring.app_id +
                    JSON.stringify(params.qstring.queryObject) +
                    projectionKey +
                    params.qstring.bucket +
                    periodObj.currentPeriodArr[0] +
                    periodObj.currentPeriodArr[periodObj.currentPeriodArr.length - 1]).digest('hex');
        } else {
            cacheId = common.crypto.createHash('sha1').update(
                    params.qstring.event +
                    params.qstring.app_id +
                    JSON.stringify(params.qstring.queryObject) +
                    params.qstring.bucket +
                    periodObj.currentPeriodArr[0] +
                    periodObj.currentPeriodArr[periodObj.currentPeriodArr.length - 1]).digest('hex');
        }

        var prepResult = function () {
            if (!isCached) {
                resultObject = replaceDateNames(resultObject, projectionKey, "toCache");
                resultObject._id = cacheId;
                resultObject.app_id = params.qstring.app_id;
                resultObject.lu = new Date();
                common.drillDb.collection("drill_cache").insert(resultObject,function(err,res){});
                resultObject = replaceDateNames(resultObject, projectionKey, "toResult");
            }

            //check if segment types are date and convert them to date strings if they are
            if (projectionKey && (projectionKey != "up.cc" || !plugins.getConfig("frontend", params.member.settings).use_google)) {
                var isSegmentTypeDate = true;

                for (var i= 0; i < resultObject.meta.length; i++) {
                    if (!(_.isNumber(resultObject.meta[i]) && ((resultObject.meta[i] + "").length == 10 || (resultObject.meta[i] + "").length == 13))) {
                        isSegmentTypeDate = false;
                        break;
                    }
                }

                if (isSegmentTypeDate) {
                    var tmpDate;

                    for (var i = 0; i < resultObject.meta.length; i++) {
                        if ((resultObject.meta[i] + "").length == 10) {
                            tmpDate = new Date(resultObject.meta[i] * 1000);
                        } else {
                            tmpDate = new Date(resultObject.meta[i]);
                        }

                        tmpDate.setTimezone(params.appTimezone);
                        tmpDate = moment(tmpDate).format("D MMM, HH:mm");

                        if (resultObject.meta.indexOf(tmpDate) != -1) {
                            resultObject[tmpDate].s += resultObject[resultObject.meta[i]].s || 0;
                            resultObject[tmpDate].dur += resultObject[resultObject.meta[i]].dur || 0;
                            resultObject[tmpDate].t += resultObject[resultObject.meta[i]].t;
                            resultObject[tmpDate].u += resultObject[resultObject.meta[i]].u;

                            for(var date in resultObject.data){
                                if(resultObject.data[date][resultObject.meta[i]]){
                                    if(resultObject.data[date][tmpDate]){
                                        resultObject.data[date][tmpDate].s += resultObject.data[date][resultObject.meta[i]].s || 0;
                                        resultObject.data[date][tmpDate].dur += resultObject.data[date][resultObject.meta[i]].dur || 0;
                                        resultObject.data[date][tmpDate].t += resultObject.data[date][resultObject.meta[i]].t;
                                        resultObject.data[date][tmpDate].u = resultObject.data[date][tmpDate].meta.length;
                                        delete resultObject.data[date][resultObject.meta[i]];
                                    }
                                    else{
                                        resultObject.data[date][tmpDate] = resultObject.data[date][resultObject.meta[i]];
                                        delete resultObject.data[date][resultObject.meta[i]];
                                    }
                                }
                            }

                            delete resultObject[resultObject.meta[i]];
                            resultObject.meta[i] = "";
                        } else {
                            resultObject[tmpDate] = resultObject[resultObject.meta[i]];

                            for(var date in resultObject.data){
                                if(resultObject.data[date][resultObject.meta[i]]){
                                    resultObject.data[date][tmpDate] = resultObject.data[date][resultObject.meta[i]];
                                    delete resultObject.data[date][resultObject.meta[i]];
                                }
                            }

                            delete resultObject[resultObject.meta[i]];
                            resultObject.meta[i] = tmpDate;
                        }
                    }
                    resultObject.meta = _.compact(resultObject.meta);
                }
            }

            //sort segment values with respect to their total values in descending order, this is to achieve better look in bar graph
            if (projectionKey) {
                resultObject.meta = _.sortBy(resultObject.meta,function(seg) { return -1 * resultObject[seg].t; });
            }

            delete resultObject._id;
            common.returnOutput(params,resultObject);
        };

        common.drillDb.collection("drill_cache").findOne({_id:cacheId}, {_id:0}, function(cacheError, cacheResult) {
            //b: bucket
            //c: count
            //dp: data points
            //s: segmentation
            //t:times
            //u:users
            //udid: unique device ids
            //ut: users total
            if (cacheResult) {
                isCached = true;
                resultObject = replaceDateNames(cacheResult,projectionKey,"toResult");
                prepResult();
            } else {
                if (projectionKey) {
                    queryObject[projectionKey] = {$exists:true};
                }

                if (projectionKey && projectionKey == "up.cc" && plugins.getConfig("frontend", params.member.settings).use_google) {
                    common.drillDb.collection(collectionName).aggregate([
                        {
                            $match: queryObject
                        },
                        {
                            $group: {
                                _id: { s: '$' + projectionKey },
                                uid: { $addToSet: '$uid' },
                                t: { $sum: "$" + common.dbEventMap['count'] },
                                s: { $sum: "$" + common.dbEventMap['sum'] },
                                dur: { $sum: "$" + common.dbEventMap['duration'] }
                            }
                        },
                        {
                            $unwind: "$uid"
                        },
                        {
                            $group: {
                                _id: { s: "$_id.s" },
                                u: { $sum: 1 },
                                t: { $first: "$t" },
                                s: { $first: "$s" },
                                dur: { $first: "$dur" }
                            }
                        }
                    ],{allowDiskUse:true}, function(error,result){
                        if (result && result.length != 0) {
                            var tmpArrEach,
                                tmpSegVal;

                            for(var i= 0; i < result.length; i++){
                                tmpArrEach = result[i];
                                tmpSegVal = tmpArrEach._id.s;

                                if(resultObject.meta.indexOf(tmpSegVal) == -1) {
                                    resultObject.meta.push(tmpSegVal);
                                }

                                if(resultObject[tmpSegVal]) {
                                    resultObject[tmpSegVal].t += tmpArrEach.t;
                                    resultObject[tmpSegVal].s += tmpArrEach.s || 0;
                                    resultObject[tmpSegVal].dur += tmpArrEach.dur || 0;
                                } else {
                                    resultObject[tmpSegVal] = {t: tmpArrEach.t, u: tmpArrEach.u, s: tmpArrEach.s, dur: tmpArrEach.dur}
                                }
                            }
                        }

                        resultObject.chart = "map";
                        delete resultObject.data;
                        prepResult();
                    });
                } else if (projectionKey) {
                    common.drillDb.collection(collectionName).aggregate([
                        {
                            $match: queryObject
                        },
                        {
                            $group: {
                                _id: { b: '$' + getBucketAbbreviation(params.qstring.bucket), s: '$' + projectionKey },
                                udid: { $addToSet: '$uid' },
                                t: { $sum: "$" + common.dbEventMap['count'] },
                                s: { $sum: "$" + common.dbEventMap['sum'] },
                                dur: { $sum: "$" + common.dbEventMap['duration'] },
								m: {$first: "$m"}
                            }
                        },
                        {
                            $unwind: "$udid"
                        },
                        {
                            $group: {
                                _id: "$_id.s",
                                dp: { $addToSet: {"b": '$_id.b', "uid": '$udid'}},
                                u: { $sum:1 },
                                t: { $addToSet: {"b":"$_id.b", "c":"$t"} },
                                s: { $addToSet: {"b":"$_id.b", "c":"$s"} },
                                dur: { $addToSet: {"b":"$_id.b", "c":"$dur"} },
								m: {$first: "$m"}
                            }
                        },
                        {
                            $sort: { u: -1 }
                        },
                        {
                            $limit: plugins.getConfig("drill").projection_limit || 10
                        },
                        {
                            $unwind: "$dp"
                        },
                        {
                            $group: {
                                _id: { b:"$dp.b", s:"$_id" },
                                u: { $sum:1 },
                                ut: { $first: "$u" },
                                t: { $first: "$t" },
                                s: { $first: "$s" },
                                dur: { $first: "$dur" },
								m: {$first: "$m"}
                            }
                        },
                        {
                            $unwind: "$t"
                        },
                        {
                            $project: {
                                _id: 1,
                                u: 1,
                                ut: 1,
                                isEq: { $eq: ["$_id.b", "$t.b"] },
                                t: "$t.c",
                                s: "$s",
                                dur: "$dur",
								m: "$m"
                            }
                        },
                        {
                            $match: { "isEq":true }
                        },
                        {
                            $unwind: "$s"
                        },
                        {
                            $project: {
                                _id: 1,
                                u: 1,
                                ut: 1,
                                isEq: { $eq: ["$_id.b", "$s.b"] },
                                t: "$t",
                                s: "$s.c",
                                dur: "$dur",
								m: "$m"
                            }
                        },
                        {
                            $match: { "isEq":true }
                        },
                        {
                            $unwind: "$dur"
                        },
                        {
                            $project: {
                                _id: 1,
                                u: 1,
                                ut: 1,
                                isEq: { $eq: ["$_id.b", "$dur.b"] },
                                t: "$t",
                                s: "$s",
                                dur: "$dur.c",
								m: "$m"
                            }
                        },
                        {
                            $match: { "isEq":true }
                        }
                    ],{allowDiskUse:true}, function(error,result) {
                        if (result && result.length != 0) {
                            var tmpArrEach,
                                tmpSegVal,
                                tmpBuckVal;
							var curYear = new Date(queryObject.ts.$lt).getFullYear();
							var prevYear = curYear-1;
                            for (var i= 0; i < result.length; i++) {
                                tmpArrEach = result[i];
								if(tmpArrEach._id.b == prevYear+":w53" && tmpArrEach.m == prevYear+":m12"){
									tmpArrEach._id.b = curYear+":w1";
								}
								else if(tmpArrEach._id.b == prevYear+":w1" && tmpArrEach.m == prevYear+":m12"){
									tmpArrEach._id.b = curYear+":w1";
								}
                                tmpSegVal = tmpArrEach._id.s;
                                tmpBuckVal = tmpArrEach._id.b.replace(/:/g,'.');

                                if(resultObject.meta.indexOf(tmpSegVal) == -1){
                                    resultObject.meta.push(tmpSegVal);
                                }

                                if(! resultObject.data[tmpBuckVal]){
                                    resultObject.data[tmpBuckVal] = {};
                                }

								if(!resultObject.data[tmpBuckVal][tmpSegVal])
									resultObject.data[tmpBuckVal][tmpSegVal] = {u: tmpArrEach.u, t: tmpArrEach.t, s: tmpArrEach.s, dur: tmpArrEach.dur};
								else{
									resultObject.data[tmpBuckVal][tmpSegVal].u += tmpArrEach.u;
									resultObject.data[tmpBuckVal][tmpSegVal].t += tmpArrEach.t;
									resultObject.data[tmpBuckVal][tmpSegVal].s += tmpArrEach.s || 0;
									resultObject.data[tmpBuckVal][tmpSegVal].dur += tmpArrEach.dur || 0;
								}

                                if(resultObject[tmpSegVal]) {
                                    resultObject[tmpSegVal].t += tmpArrEach.t;
                                    resultObject[tmpSegVal].s += tmpArrEach.s || 0;
                                    resultObject[tmpSegVal].dur += tmpArrEach.dur || 0;
                                } else {
                                    resultObject[tmpSegVal] = {t: tmpArrEach.t, u: tmpArrEach.ut, s: tmpArrEach.s, dur: tmpArrEach.dur}
                                }
                            }
                        }

                        prepResult();
                    });
                } else {
                    common.drillDb.collection(collectionName).aggregate([
                        {
                            $match: queryObject
                        },
                        {
                            $group: {
                                _id: { b: '$' + getBucketAbbreviation(params.qstring.bucket) },
                                uid: { $addToSet: '$uid' },
                                t: { $sum: "$" + common.dbEventMap['count'] },
                                s: { $sum: "$" + common.dbEventMap['sum'] },
                                dur: { $sum: "$" + common.dbEventMap['duration'] },
								m: {$first: "$m"}
                            }
                        },
                        {
                            $unwind: "$uid"
                        },
                        {
                            $group: {
                                _id: { b:"$_id.b" },
                                u: { $sum:1 },
                                t: { $first:"$t" },
                                s: { $first:"$s" },
                                dur: { $first:"$dur" },
								m: {$first: "$m"}
                            }
                        }
                    ],{allowDiskUse:true}, function(error,result) {
                        if (result && result.length != 0) {
                            var tmpArrEach;
							var curYear = new Date(queryObject.ts.$lt).getFullYear();
							var prevYear = curYear-1;
                            for(var i= 0; i < result.length; i++){
                                tmpArrEach = result[i];
								if(tmpArrEach._id.b == prevYear+":w53" && tmpArrEach.m == prevYear+":m12"){
									tmpArrEach._id.b = curYear+":w1";
								}
								else if(tmpArrEach._id.b == prevYear+":w1" && tmpArrEach.m == prevYear+":m12"){
									tmpArrEach._id.b = curYear+":w1";
								}
								tmpArrEach._id.b = tmpArrEach._id.b.replace(/:/g,'.');
								if(!resultObject.data[tmpArrEach._id.b])
									resultObject.data[tmpArrEach._id.b] = {u: tmpArrEach.u, t: tmpArrEach.t, s: tmpArrEach.s, dur: tmpArrEach.dur};
								else{
									resultObject.data[tmpArrEach._id.b].u += tmpArrEach.u;
									resultObject.data[tmpArrEach._id.b].t += tmpArrEach.t;
									resultObject.data[tmpArrEach._id.b].s += tmpArrEach.s || 0;
									resultObject.data[tmpArrEach._id.b].dur += tmpArrEach.dur || 0;
								}
                            }
                        }

                        prepResult();
                    });
                }
            }
        });
    };

	drill.fetchSegmentationMeta = function(params) {

        var collectionName = "drill_events" + crypto.createHash('sha1').update(params.qstring.event + params.qstring.app_id).digest('hex');
        var result;
        common.drillDb.collection(collectionName).findOne( {"_id": "meta"},{_id:0} ,function(err,meta1){
            result = meta1;
            common.drillDb.collection(collectionName).findOne( {"_id": "meta_v2"},{_id:0} ,function(err,meta2){
                if(meta2){
                    if(!result)
                        result = {};
                    for(var i in meta2){
                        if(meta2[i] && typeof meta2[i] === "object"){
                            if(typeof result[i] === "undefined"){
                                result[i] = {};
                            }
                            for(var j in meta2[i]){
                                if(typeof result[i][j] === "undefined"){
                                    result[i][j] = {};
                                    if(meta2[i][j].values){
                                        result[i][j].values = Object.keys(meta2[i][j].values).map(function(arg){return common.convertToType(common.db.decode(arg));});
                                    }
                                }
                                else if(meta2[i][j].values){
                                    common.arrayAddUniq(result[i][j].values, Object.keys(meta2[i][j].values).map(function(arg){return common.convertToType(common.db.decode(arg));}));
                                }
                                result[i][j].type = meta2[i][j].type;
                            }
                        }
                        else if(typeof result[i] === "undefined"){
                            result[i] = meta2[i];
                        }
                    }
                }
                common.returnOutput(params,result);
            });
        });
    };

    drill.fetchSegmentationBigMeta = function(params) {
        if(params.qstring.prop){
            var collectionName = "drill_events" + crypto.createHash('sha1').update(params.qstring.event + params.qstring.app_id).digest('hex');
            var result = [];
            var reg;
            if(params.qstring.search){
                reg = new RegExp(params.qstring.search);
            }
            common.drillDb.collection(collectionName).findOne( {"_id": "meta_v2_"+params.qstring.prop},{_id:0} ,function(err,meta){
                for(var i in meta){
                    if(!reg || reg.test(i))
                        result.push(common.convertToType(common.db.decode(i)));
                    if(result.length >= plugins.getConfig("drill").list_limit){
                        break;
                    }
                }
                common.returnOutput(params,result);
            });
        }
        else{
            common.returnOutput(params,[]);
        }
    };

	drill.fetchDrillBookmarks = function(params) {
		var eventAppId = common.crypto.createHash('md5').update(params.qstring.app_id + params.qstring.event_key).digest('hex');

		common.drillDb.collection("drill_bookmarks").find({event_app_id: eventAppId}).toArray(function (err, result) {
			common.returnOutput(params, result || []);
		});
	};

    drill.fetchUserList = function(params) {
        drill.fetchUsers(params, (error, uids) => {
            if (error) {
                common.returnMessage(params, error[0], error[1]);
            } else {
                common.returnOutput(params, uids);
            }
        });
    };

    drill.fetchUsers = function(params, callback) {
        //check request parameters if they all exist and in correct format
        if (!params.qstring.event) {
            callback([400, 'Missing request parameter: event'])
            return false;
        }

        if (params.qstring.queryObject) {
            try {
                params.qstring.queryObject = JSON.parse(params.qstring.queryObject);
            } catch (SyntaxError) {
                console.log('Parsing custom queryObject failed!');
                callback([400, 'Bad request parameter: queryObject'])
                return false;
            }
        } else {
            callback([400, 'Missing request parameter: queryObject'])
            return false;
        }

        if (params.qstring.period) {
            //check if period comes from datapicker
            if (params.qstring.period.indexOf(",") !== -1) {
                try {
                    params.qstring.period = JSON.parse(params.qstring.period);
                } catch (SyntaxError) {
                    console.log('Parsing custom period failed!');
                    callback([400, 'Bad request parameter: period'])
                    return false;
                }
            }
            else{
                switch (params.qstring.period){
                    case "month":
                    case "day":
                    case "yesterday":
                    case "hour":
                    case "7days":
                    case "30days":
                    case "60days":
                        break;
                    default:
                        callback([400, 'Bad request parameter: period'])
                        return false;
                        break;
                }
            }
        } else {
            callback([400, 'Missing request parameter: period'])
            return false;
        }

        if (params.qstring.bucket) {
            switch (params.qstring.bucket){
                case "hourly":
                case "daily":
                case "weekly":
                case "monthly":
                    break;
                default:
                    callback([400, 'Bad request parameter: bucket'])
                    return false;
                    break;
            }
        } else {
            callback([400, 'Missing request parameter: bucket'])
            return false;
        }

        countlyCommon.setPeriod(params.qstring.period);
        countlyCommon.setTimezone(params.appTimezone);

        var periodObj = countlyCommon.periodObj,
            queryObject = params.qstring.queryObject,//note that params.qstring.queryObject is JSON parsed during request check above
            projectionKey = params.qstring.projectionKey,
            resultObject = {data:{},meta:[]},
            now = params.time.now.toDate(),
            collectionName = "drill_events" + crypto.createHash('sha1').update(params.qstring.event + params.qstring.app_id).digest('hex'),
            cacheId,
            isCached = false;

        //create current period array if it does not exist
        if (!periodObj.currentPeriodArr) {
            periodObj.currentPeriodArr = [];

            //create a period array that starts from the beginning of the current year until today
            if (params.qstring.period == "month") {
                for (var i = 0; i < (now.getMonth() + 1); i++) {
                    var daysInMonth = moment().month(i).daysInMonth();

                    for (var j = 0; j < daysInMonth; j++) {
                        periodObj.currentPeriodArr.push(periodObj.activePeriod + "." + (i + 1) + "." + (j + 1));

                        // If current day of current month, just break
                        if ((i == now.getMonth()) && (j == (now.getDate() - 1))) {
                            break;
                        }
                    }
                }
            }
            //create a period array that starts from the beginning of the current month until today
            else if(params.qstring.period == "day") {
                for(var i = 0; i < now.getDate(); i++) {
                    periodObj.currentPeriodArr.push(periodObj.activePeriod + "." + (i + 1));
                }
            }
            //create one day period array
            else{
                periodObj.currentPeriodArr.push(periodObj.activePeriod);
            }
        }

        //get timestamps of start of days (DD-MM-YYYY-00:00) with respect to apptimezone for both beginning and end of period arrays
        var tmpArr;
        queryObject.ts = {};

        tmpArr = periodObj.currentPeriodArr[0].split(".");
        queryObject.ts.$gte = new Date(Date.UTC(parseInt( tmpArr[0]),parseInt(tmpArr[1])-1,parseInt(tmpArr[2]) ));
        queryObject.ts.$gte.setTimezone(params.appTimezone);
        queryObject.ts.$gte = queryObject.ts.$gte.getTime() + queryObject.ts.$gte.getTimezoneOffset()*60000;

        tmpArr = periodObj.currentPeriodArr[periodObj.currentPeriodArr.length - 1].split(".");
        queryObject.ts.$lt = new Date(Date.UTC(parseInt( tmpArr[0]),parseInt(tmpArr[1])-1,parseInt(tmpArr[2]) ));
        queryObject.ts.$lt.setDate(queryObject.ts.$lt.getDate() + 1);
        queryObject.ts.$lt.setTimezone(params.appTimezone);
        queryObject.ts.$lt = queryObject.ts.$lt.getTime() + queryObject.ts.$lt.getTimezoneOffset()*60000;

        if (projectionKey) {
            queryObject[projectionKey] = {$exists:true};
        }

        common.drillDb.collection(collectionName).aggregate([
            {
                $match: queryObject
            },
            {
                $group: {
                    _id : null,
                    uid: { $addToSet: '$uid' },
                }
            }
        ],{allowDiskUse:true}, function(error,result){
            log.d('Drilled user list: %j / %d (%j)', error, result && result[0] && result[0].uid ? result[0].uid.length : result, queryObject);
            if (error) {
                callback([500, error]);
            } else if (result && result.length != 0 && result[0].uid && result[0].uid.length != 0) {
                callback(null, result[0].uid);
            } else {
                callback(null, []);
            }
        });
    };

    drill.processEvents = function(dbAppUser, params) {
        common.db.collection("events").findOne({'_id':params.app_id}, {list:1}, function (err, eventColl) {
            var appEvents = (eventColl && eventColl.list)? eventColl.list : [];
            processEvents(appEvents, dbAppUser, params);
        });
    };

    drill.processCustomEvents = function(dbAppUser, params, events){
        processEvents([], dbAppUser, params, events);
    };

	drill.updateDuration = function(id, session_duration, params){
		var hashedCollectionName = "drill_events" + crypto.createHash('sha1').update("[CLY]_session" + params.app_id).digest('hex');
        common.drillDb.collection(hashedCollectionName).update({'_id': id}, {'$inc': {'sd': session_duration}}, {'upsert': true}, function(err, res) {});
	};

    drill.updateViewDuration = function(id, duration, params){
		var hashedCollectionName = "drill_events" + crypto.createHash('sha1').update("[CLY]_view" + params.app_id).digest('hex');
        common.drillDb.collection(hashedCollectionName).update({'_id': id}, {'$inc': {'dur': parseFloat(duration)}}, {'upsert': true}, function(err, res) {});
	};

    function processEvents(appEvents, dbAppUser, params, events) {
        events = events || params.qstring.events || [];
        // Vars for detailed event db write
        var dbEventObject = {},
            upWithMeta= fillUserProperties(dbAppUser),
            dbEventUpMeta = upWithMeta.upMeta,
            segKeyAsFieldName = "",
            date,
            momentDate,
            eventIndex,
            eventIndexer = {},
            dbInsertArray = [],
            dbMetaArray = [],
            hashedCollectionName = "",
            eventKey = "";

        if ( dbAppUser && dbAppUser[common.dbUserMap['user_id']] ){
            dbEventObject[common.dbUserMap['user_id']] = dbAppUser[common.dbUserMap['user_id']];
        }
        dbEventObject[common.dbUserMap['device_id']] = params.qstring.device_id;
        dbEventObject[common.dbEventMap['user_properties']] = upWithMeta.up;
        dbEventObject["custom"] = upWithMeta.upCustom;
        dbEventObject["cmp"] = upWithMeta.upCampaign;

        for (var i=0; i < events.length; i++) {

            var currEvent = events[i];
            if (!currEvent.key || (currEvent.key.indexOf('[CLY]_') === 0 && plugins.internalDrillEvents.indexOf(currEvent.key) === -1)) {
                continue;
            }

            if(currEvent.key == "[CLY]_session" && !plugins.getConfig("drill").record_sessions){
                continue;
            }

            if(currEvent.key == "[CLY]_view" && !plugins.getConfig("drill").record_views){
                continue;
            }

            if(currEvent.key == "[CLY]_action" && !plugins.getConfig("drill").record_actions){
                continue;
            }

            if((currEvent.key == "[CLY]_push_action" || currEvent.key == "[CLY]_push_open" || currEvent.key == "[CLY]_push_sent") && !plugins.getConfig("drill").record_pushes){
                continue;
            }

            if(currEvent.key == "[CLY]_crash" && !plugins.getConfig("drill").record_crashes){
                continue;
            }

            if(currEvent.key == "[CLY]_star_rating" && !plugins.getConfig("drill").record_star_rating){
                continue;
            }

            if (currEvent.key.indexOf("[CLY]_") != 0 &&
                plugins.getConfig("api").event_limit &&
                appEvents.length >= plugins.getConfig("api").event_limit &&
                appEvents.indexOf(currEvent.key) === -1) {
                continue;
            }

            currEvent.key = common.fixEventKey(currEvent.key);

            if (!currEvent.key) {
                continue;
            }
            eventKey = currEvent.key;

            if (events[i].timestamp) {
                params.time = common.initTimeObj(params.appTimezone, events[i].timestamp);
            }
            else{
                params.time = common.initTimeObj(params.appTimezone, params.qstring.timestamp);
            }

            dbEventObject[common.dbEventMap['timestamp']] = params.time.mstimestamp;



            date = new Date(dbEventObject[common.dbEventMap['timestamp']]);
			date.setTimezone(params.appTimezone);
            momentDate = moment(date);

            dbEventObject.cd = new Date();
            dbEventObject.d = momentDate.year() + ":" + (momentDate.month() + 1) + ":" + momentDate.format("D");
            dbEventObject.w = momentDate.isoyear() + ":w" + momentDate.isoweek();
            dbEventObject.m = momentDate.year() + ":m" +  (momentDate.month() + 1);
            dbEventObject.h = momentDate.year() + ":" + (momentDate.month() + 1) + ":" + momentDate.format("D") + ":h" + momentDate.format("H");

            events[i].hour = (typeof events[i].hour != "undefined") ? events[i].hour : params.qstring.hour;
            if (typeof events[i].hour != "undefined") {
                events[i].hour = parseInt(events[i].hour);
                if(events[i].hour == 24)
                    events[i].hour = 0;
                if(events[i].hour >= 0 && events[i].hour < 24){
                    upWithMeta.up.hour = events[i].hour;
                    upWithMeta.upMetaTypes["up.hour.type"] = "l";
                    upWithMeta.upMeta["up.hour.values."+events[i].hour] = true;
                }
            }

            events[i].dow = (typeof events[i].dow != "undefined") ? events[i].dow : params.qstring.dow;
            if (typeof events[i].dow != "undefined") {
                events[i].dow = parseInt(events[i].dow);
                if(events[i].dow == 0)
                    events[i].dow = 7;
                if(events[i].dow > 0 && events[i].dow <= 7){
                    upWithMeta.up.dow = events[i].dow;
                    upWithMeta.upMetaTypes["up.dow.type"] = "l";
                    upWithMeta.upMeta["up.dow.values."+events[i].dow] = true;
                }
            }

            dbEventObject[common.dbEventMap['segmentations']] = {};

            if (eventIndexer[eventKey]) {
                eventIndex = eventIndexer[currEvent.key];
            } else {
                hashedCollectionName = "drill_events" + crypto.createHash('sha1').update(eventKey + params.app_id).digest('hex');
                var collection = common.drillDb.collection(hashedCollectionName);

                dbInsertArray.push({collection:collection, dbEventObjectArr: [], key:eventKey, app_id: params.app_id, uid: params.app_user_id});
                dbMetaArray.push({eventName:eventKey, app_id:params.app_id, collection:collection, dbEventUpMeta:dbEventUpMeta, upMetaTypes:upWithMeta.upMetaTypes, sgMeta:{}, upCustom:upWithMeta.upCustom, upCampaign:upWithMeta.upCampaign, upMetaBig: upWithMeta.upMetaBig});

                eventIndex = dbInsertArray.length -1;
                eventIndexer[eventKey] = eventIndex;
            }

            if (currEvent.segmentation) {
                var tmpSegVal;
                for (var segKey in currEvent.segmentation) {
                    if (currEvent.segmentation[segKey] == null || typeof currEvent.segmentation[segKey] == "undefined") {
                        continue;
                    }

                    segKeyAsFieldName = segKey.replace(/^\$|\./g, "");

                    //max number check
                    if (common.isNumber(currEvent.segmentation[segKey])) {
                        if(currEvent.segmentation[segKey].length && currEvent.segmentation[segKey].length <= 16)
                            tmpSegVal = parseFloat(currEvent.segmentation[segKey]);
                        else
                            tmpSegVal = currEvent.segmentation[segKey];
                    } else{
                        tmpSegVal = (currEvent.segmentation[segKey]+"").replace(/^\$/, "");
                    }

                    dbEventObject[common.dbEventMap['segmentations']][segKeyAsFieldName] = tmpSegVal;

                    if (!dbMetaArray[eventIndex].sgMeta[segKeyAsFieldName]) {
                        dbMetaArray[eventIndex].sgMeta[segKeyAsFieldName] = [];
                    }

                    if (dbMetaArray[eventIndex].sgMeta[segKeyAsFieldName].indexOf(tmpSegVal) == -1) {
                        dbMetaArray[eventIndex].sgMeta[segKeyAsFieldName].push(tmpSegVal);
                    }
                }
            }

            if (currEvent.sum && common.isNumber(currEvent.sum)) {
                currEvent.sum = parseFloat(parseFloat(currEvent.sum).toFixed(5));
            }

            if (currEvent.dur && common.isNumber(currEvent.dur)) {
                currEvent.dur = parseFloat(currEvent.dur);
            }

            dbEventObject[common.dbEventMap['sum']] = currEvent.sum || 0;
            dbEventObject[common.dbEventMap['duration']] = currEvent.dur || 0;
            dbEventObject[common.dbEventMap['count']] = currEvent.count || 1;

            dbInsertArray[eventIndex].dbEventObjectArr.push(new clonedObject(dbEventObject));
        }

        function insertEventCollection(dbEvent, callback){
            dbEvent.collection.insert(dbEvent.dbEventObjectArr,{multi:true},function(err,res){
                if(err)
                    console.log(err, dbEvent);
                else if(dbEvent.key == "[CLY]_session"){
                    res = res.ops;
                    common.updateAppUser(params, {'$set': {lsid:res[0]._id}});
                }
                else if(dbEvent.key == "[CLY]_view"){
                    res = res.ops;
                    common.updateAppUser(params, {'$set': {lvid:res[0]._id}});
                }
            });
            callback(false);
        }
        async.map(dbInsertArray, insertEventCollection, function(){});
        async.map(dbMetaArray, setSegmentTypes, function(){});
    }

    function setSegmentTypes(dbMeta, callback) {
        dbMeta.collection.findOne({"_id": "meta_v2"}, {_id:0, up:1, sg:1, custom:1, cmp:1}, function(err, result) {
            if (!err && (!result || result.length == 0)) {
                dbMeta.collection.ensureIndex({ts:-1},function(err,res){});
                dbMeta.collection.ensureIndex({uid:1},function(err,res){});
            }

            if (!err) {
                var addToSetObject = {},
                    setObject = {},
                    listSize = plugins.getConfig("drill").list_limit,
                    newSgObj = dbMeta.sgMeta;

                setObject.e = dbMeta.eventName;
                setObject.app_id = dbMeta.app_id + "";

                //check up limit here
                for (var prop in dbMeta.dbEventUpMeta) {
                    var up = prop.split(".values.")[0].replace(common.dbEventMap['user_properties'] + ".", "");
                    if(!result || !result.up || !result.up[up] || !result.up[up].values || Object.keys(result.up[up].values).length < listSize ){
                        setObject[prop] = dbMeta.dbEventUpMeta[prop];
                    }
                }

                for (var prop in dbMeta.upMetaTypes) {
                    setObject[prop] = dbMeta.upMetaTypes[prop];
                }

                var isNumber, isDate,setString;

                for (var segmentKey in newSgObj) {
                    var isSgInResult = (result && common.dbEventMap['segmentations'] in result),
                        isSgKeyInResult = (isSgInResult && (segmentKey in result[common.dbEventMap['segmentations']])),
                        resSgKeyObj = (isSgKeyInResult)? result[common.dbEventMap['segmentations']][segmentKey] : {};

                    if (!resSgKeyObj.values || !(Object.keys(resSgKeyObj.values).length > listSize)) {
                        for(var i = 0; i <  newSgObj[segmentKey].length; i++){
                            if(common.db.encode(newSgObj[segmentKey][i]+"").trim() != "")
                                setObject[common.dbEventMap['segmentations'] + "." + segmentKey + ".values." + common.db.encode(newSgObj[segmentKey][i]+"")] = true;
                        }
                    }

                    setString = common.dbEventMap['segmentations'] + "." + segmentKey + ".type";
                    isNumber = true;
                    isDate = true;

                    for (var index in newSgObj[segmentKey]) {
                        if (!common.isNumber(newSgObj[segmentKey][index])) {
                            isNumber = false;
                            break;
                        }

                        if (isDate && newSgObj[segmentKey][index].toString().length != 10 && newSgObj[segmentKey][index].toString().length != 13) {
                            isDate = false;
                        }
                    }

                    if (isNumber) {
                        if (isDate) {
                            if (!(resSgKeyObj.type)) {
                                setObject[setString] = "d";
                            }
                        } else {
                            if (!(resSgKeyObj.type) || (resSgKeyObj.type == "d")) {
                                setObject[setString] = "n";
                            }
                        }
                    } else {
                        if (resSgKeyObj.type) {
                            if (resSgKeyObj.type == "l") {
                                if (resSgKeyObj.values && resSgKeyObj.values.length >= (listSize - newSgObj[segmentKey].length)) {
                                    setObject[setString] = "s";
                                }
                            } else if (resSgKeyObj.type != "s") {
                                if (!resSgKeyObj.values || Object.keys(resSgKeyObj.values).length < (listSize - newSgObj[segmentKey].length)) {
                                    setObject[setString] = "l";
                                } else {
                                    setObject[setString] = "s";
                                }
                            }
                        } else {
                            setObject[setString] = "l";
                        }
                    }
                }

                //adding user custom properties
                var isNumber, isDate,setString;

                for (var segmentKey in dbMeta.upCustom) {
                    var isSgInResult = (result && "custom" in result),
                        isSgKeyInResult = (isSgInResult && (segmentKey in result["custom"])),
                        resSgKeyObj = (isSgKeyInResult)? result["custom"][segmentKey] : {};

                    //limiting amount of custom property keys in drill
                    if(isSgKeyInResult || !isSgInResult || (isSgInResult && Object.keys(result["custom"]).length <= plugins.getConfig("drill").custom_property_limit)){
                        if (!resSgKeyObj.values || !(Object.keys(resSgKeyObj.values).length > listSize)) {
                            if(Array.isArray(dbMeta.upCustom[segmentKey]))
                                for(var i = 0; i < dbMeta.upCustom[segmentKey].length; i++){
                                    if(common.db.encode(dbMeta.upCustom[segmentKey][i]+"").trim() != "")
                                        setObject["custom." + segmentKey + ".values." + common.db.encode(dbMeta.upCustom[segmentKey][i]+"")] = true;
                                }
                            else if(common.db.encode(dbMeta.upCustom[segmentKey]+"").trim() != "")
                                setObject["custom." + segmentKey + ".values." + common.db.encode(dbMeta.upCustom[segmentKey]+"")] = true;
                        }

                        setString = "custom." + segmentKey + ".type";
                        isNumber = true;
                        isDate = true;

                        if (!common.isNumber(dbMeta.upCustom[segmentKey]) || dbMeta.upCustom[segmentKey].toString().length > 16) {
                            isNumber = false;
                        }
                        else if (isDate && dbMeta.upCustom[segmentKey].toString().length != 10 && dbMeta.upCustom[segmentKey].toString().length != 13) {
                            isDate = false;
                        }


                        if (isNumber) {
                            if (isDate) {
                                if (!(resSgKeyObj.type)) {
                                    setObject[setString] = "d";
                                }
                            } else {
                                if (!(resSgKeyObj.type) || (resSgKeyObj.type == "d")) {
                                    setObject[setString] = "n";
                                }
                            }
                        } else {
                            if (resSgKeyObj.type) {
                                if (resSgKeyObj.type == "l") {
                                    if (resSgKeyObj.values && resSgKeyObj.values.length >= listSize) {
                                        setObject[setString] = "s";
                                    }
                                } else if (resSgKeyObj.type != "s") {
                                    if (!resSgKeyObj.values || Object.keys(resSgKeyObj.values).length < listSize) {
                                        setObject[setString] = "l";
                                    } else {
                                        setObject[setString] = "s";
                                    }
                                }
                            } else {
                                setObject[setString] = "l";
                            }
                        }
                    }
                }

                //adding user campaign info
                var isNumber, isDate,setString;

                for (var segmentKey in dbMeta.upCampaign) {
                    var isSgInResult = (result && "cmp" in result),
                        isSgKeyInResult = (isSgInResult && (segmentKey in result["cmp"])),
                        resSgKeyObj = (isSgKeyInResult)? result["cmp"][segmentKey] : {};

                    //limiting amount of custom property keys in drill
                    if(isSgKeyInResult || !isSgInResult || (isSgInResult && Object.keys(result["cmp"]).length <= plugins.getConfig("drill").custom_property_limit)){

                        if (!resSgKeyObj.values || !(Object.keys(resSgKeyObj.values).length > listSize)) {
                            if(Array.isArray(dbMeta.upCampaign[segmentKey]))
                                for(var i = 0; i < dbMeta.upCampaign[segmentKey].length; i++){
                                    if(common.db.encode(dbMeta.upCampaign[segmentKey][i]+"").trim() != "")
                                        setObject["cmp." + segmentKey + ".values." + common.db.encode(dbMeta.upCampaign[segmentKey][i]+"")] = true;
                                }
                            else if(common.db.encode(dbMeta.upCampaign[segmentKey]+"").trim() != "")
                                setObject["cmp." + segmentKey + ".values." + common.db.encode(dbMeta.upCampaign[segmentKey]+"")] = true;
                        }

                        setString = "cmp." + segmentKey + ".type";
                        if (resSgKeyObj.type) {
                            if (resSgKeyObj.type == "l") {
                                if (resSgKeyObj.values && resSgKeyObj.values.length >= listSize) {
                                    setObject[setString] = "s";
                                }
                            } else if (resSgKeyObj.type != "s") {
                                if (!resSgKeyObj.values || Object.keys(resSgKeyObj.values).length < listSize) {
                                    setObject[setString] = "l";
                                } else {
                                    setObject[setString] = "s";
                                }
                            }
                        } else {
                            setObject[setString] = "l";
                        }
                    }
                }

				if(Object.keys(setObject).length > 0)
					dbMeta.collection.update({"_id":"meta_v2"}, {$set:setObject}, {upsert: true},function(err,res){});
            }
        });
        //process big list meta
        if(dbMeta.upMetaBig){
            async.each(Object.keys(dbMeta.upMetaBig), function(item, done){
                //get value for limit
                dbMeta.collection.findOne({_id:"meta_v2_"+item}, function(err, res){
                    if(!res || Object.keys(res).length < plugins.getConfig("drill").big_list_limit){
                        //limit not reached, insert new meta stuff
                        dbMeta.collection.update({_id:"meta_v2_"+item}, {$set:dbMeta.upMetaBig[item]}, {upsert: true}, function(err,res){
                            done();
                        });
                    }
                });
            }, function(){
                callback(false);
            });
        }
        else{
            callback(false);
        }
    }

    function fillUserProperties(dbAppUser) {

        var userProperties = {},
            userCustom = {},
            userCampaign = {},
            metaObject = {},
            metaBigListObject = {},
            metaTypesObject = {};

        if (!dbAppUser) {
            return {up : userProperties, upMeta: metaObject, upMetaBig: metaBigListObject, upMetaTypes: metaTypesObject, upCustom: userCustom, upCampaign:userCampaign };
        }

        var countlyUP = [
            { name:"first_seen", type: "d" },
            { name:"last_seen", type: "d" },
            { name:"total_session_duration", type: "n" },
            { name:"session_count", type: "n" },
            { name:"device", type: "l" },
            { name:"city", type: "l" },
            { name:"country_code", type: "l" },
            { name:"platform", type: "l" },
            { name:"platform_version", type: "l" },
            { name:"app_version", type: "l" },
            { name:"carrier", type: "l" },
            { name:"resolution", type: "l" },
            { name:"dnst", type: "l" },
            { name:"brw", type: "l" },
            { name:"la", type: "l" },
            { name:"src", type: "bl" },
            { name:"name", type: "s" },
            { name:"username", type: "s" },
            { name:"email", type: "s" },
            { name:"organization", type: "s" },
            { name:"phone", type: "s" },
            { name:"gender", type: "l" },
            { name:"byear", type: "n" },
            { name:"lp", type: "d" },
            { name:"lpa", type: "n" },
            { name:"tp", type: "n" },
            { name:"tpc", type: "n" },
            { name:"lat", type: "n" },
            { name:"lng", type: "n" },
            { name:"lv", type: "bl" },
            { name:"cdfs", type: "n" },
            { name:"cwfs", type: "n" },
            { name:"cmfs", type: "n" }
        ];

        for (var i = 0; i < countlyUP.length; i++) {
            var shortRep = common.dbUserMap[countlyUP[i].name] || countlyUP[i].name;

            if(dbAppUser[shortRep]){
                userProperties[shortRep] = dbAppUser[shortRep];
                metaTypesObject[common.dbEventMap['user_properties'] + "." + shortRep + ".type"] = countlyUP[i].type;

                if (countlyUP[i].type == "l") {
                    if(common.db.encode(dbAppUser[shortRep]+"").trim() != "")
                        metaObject[common.dbEventMap['user_properties'] + "." + shortRep + ".values." + common.db.encode(dbAppUser[shortRep]+"")] = true;
                }
                else if (countlyUP[i].type == "bl") {
                    if(common.db.encode(dbAppUser[shortRep]+"").trim() != ""){
                        if(!metaBigListObject[common.dbEventMap['user_properties'] + "." + shortRep])
                            metaBigListObject[common.dbEventMap['user_properties'] + "." + shortRep] = {};
                        metaBigListObject[common.dbEventMap['user_properties'] + "." + shortRep][common.db.encode(dbAppUser[shortRep]+"")] = true;
                    }
                }
            }

        }

        if(dbAppUser.custom){
            var key;
            for(var i in dbAppUser.custom){
                key = common.fixEventKey(i);

                if (!key || key == "_id" || key == "bv" || key == "ip" || key == "os" || key == "r" || key == "cty" || key == "last_click") {
                    continue;
                }

                var tmpVal;
                if (common.isNumber(dbAppUser.custom[i])) {
                    if(dbAppUser.custom[i].length && dbAppUser.custom[i].length <= 16)
                        tmpVal = parseFloat(dbAppUser.custom[i]);
                    else
                        tmpVal = dbAppUser.custom[i];
                }
                else if(Array.isArray(dbAppUser.custom[i])){
                    tmpVal = dbAppUser.custom[i];
                }
                else{
                    tmpVal = dbAppUser.custom[i];
                }

                userCustom[key] = tmpVal;
            }
        }

        //add referral campaign data if any
        //legacy campaign data
        if(dbAppUser.cmp){
            var key;
            for(var i in dbAppUser.cmp){
                key = common.fixEventKey(i);

                if (!key) {
                    continue;
                }

                var tmpVal;
                if (common.isNumber(dbAppUser.cmp[i])) {
                    if(dbAppUser.cmp[i].length && dbAppUser.cmp[i].length <= 16)
                        tmpVal = parseFloat(dbAppUser.cmp[i]);
                    else
                        tmpVal = dbAppUser.cmp[i];
                }
                else if(Array.isArray(dbAppUser.cmp[i])){
                    tmpVal = dbAppUser.cmp[i];
                }
                else{
                    tmpVal = dbAppUser.cmp[i];
                }

                userCampaign[key] = tmpVal;
            }
        }
        else{
            userCampaign["c"] = "Organic";
        }

        return {
            up : userProperties,
            upMeta: metaObject,
            upMetaBig: metaBigListObject,
            upCustom: userCustom,
            upCampaign: userCampaign,
            upMetaTypes: metaTypesObject
        };
    }

    function clonedObject(dbEventObject) {
        for (var prop in dbEventObject) {
            this[prop] = dbEventObject[prop];
        }
    }

	function getBucketAbbreviation(bucket){
        var abbreviation;
        switch (bucket){
            case "hourly":
                abbreviation= "h";
                break;
            case "daily":
                abbreviation= "d";
                break;
            case "weekly":
                abbreviation= "w";
                break;
            case "monthly":
                abbreviation= "m";
                break;
        }
        return abbreviation;
    }

	//switch between '.' s and ':' s in date names, ':' used for database objects, '.' used for response
    function replaceDateNames(resultObject, projectionKey, toType) {
        var tmpDate, tmpSegVal;
        for(var date in resultObject.data){
            if(toType == "toCache"){
                if(projectionKey){
                    for(var segVal in resultObject.data[date]){
                        if(segVal.indexOf('.') != -1){
                            tmpSegVal = segVal.replace(/\./g,'&#46;');
                            resultObject.data[date][tmpSegVal] = resultObject.data[date][segVal];
                            delete resultObject.data[date][segVal];
                        }
                    }
                }
                if(date.indexOf('.') != -1){
                    tmpDate = date.replace(/\./g,':');
                    resultObject.data[tmpDate] = resultObject.data[date];
                    delete resultObject.data[date];
                }
            }
            else if(toType == "toResult"){
                if(projectionKey){
                    for(var segVal in resultObject.data[date]){
                        if(segVal.indexOf('&#46;') != -1){
                            tmpSegVal = segVal.replace(/&#46;/g,'.');
                            resultObject.data[date][tmpSegVal] = resultObject.data[date][segVal];
                            delete resultObject.data[date][segVal];
                        }
                    }
                }
                if(date.indexOf(':') != -1){
                    tmpDate = date.replace(/:/g,'.');
                    resultObject.data[tmpDate] = resultObject.data[date];
                    delete resultObject.data[date];
                }
            }
        }
        if(projectionKey){
            if(toType == "toCache"){
                for(var i= 0; i < resultObject.meta.length; i++){
                    if(_.isString(resultObject.meta[i]) && resultObject.meta[i].indexOf('.') != -1){
                        tmpSegVal = resultObject.meta[i].replace(/\./g,'&#46;');
                        resultObject[tmpSegVal] = resultObject[resultObject.meta[i]];
                        delete resultObject[resultObject.meta[i]];
                        resultObject.meta[i] = tmpSegVal;
                    }
                }
            }
            else if(toType == "toResult"){
                for(var i= 0; i < resultObject.meta.length; i++){
                    if(_.isString(resultObject.meta[i]) && resultObject.meta[i].toString().indexOf('&#46;') != -1){
                        tmpSegVal = resultObject.meta[i].replace(/&#46;/g,'.');
                        resultObject[tmpSegVal] = resultObject[resultObject.meta[i]];
                        delete resultObject[resultObject.meta[i]];
                        resultObject.meta[i] = tmpSegVal;
                    }
                }
            }
        }
        return resultObject;
    }

}(drill));

module.exports = drill;