var plugin = {},
	crypto = require('crypto'),
	async = require('async'),
	countlyConfig = require('../../../frontend/express/config', 'dont-enclose'),
    plugins = require('../../pluginManager.js');
    
var config;
try{
    config = require('../config');
}
catch(err){
    config = countlyConfig;
}

(function (plugin) {
	plugin.init = function(app, countlyDb){
		if (typeof config.mongodb === "string") {
			config.mongodb = config.mongodb.replace(new RegExp('countly$'), 'countly_drill');
		} else{
			config.mongodb.db = 'countly_drill';
            config.mongodb.max_pool_size = countlyConfig.mongodb.max_pool_size;
		}
		var countlyDrillDb = plugins.dbConnection(config);
        
        function deleteEvent(req, key, app_id){
            var collectionNameWoPrefix = crypto.createHash('sha1').update(key + app_id).digest('hex');
			if (!req.session.gadm) {
				countlyDb.collection('members').findOne({"_id":countlyDb.ObjectID(req.session.uid)}, function (err, member) {
					if (!err && member.admin_of && member.admin_of.indexOf(app_id) != -1) {
						countlyDrillDb.collection("drill_events" + collectionNameWoPrefix).drop(function(){});
						countlyDrillDb.collection('drill_bookmarks').remove({'app_id':app_id, 'event_key':key},function(err,res){});
					}
				});
			} else {
				countlyDrillDb.collection("drill_events" + collectionNameWoPrefix).drop(function(){});
				countlyDrillDb.collection('drill_bookmarks').remove({'app_id':app_id, 'event_key':key},function(err,res){});
			}
        };
		
		app.post(countlyConfig.path+'/events/delete', function (req, res, next) {
			if (!req.session.uid || !req.body.app_id || !req.body.event_key) {
				res.end();
				return false;
			}
			
            deleteEvent(req, req.body.event_key, req.body.app_id);
			
			next();
		});
        
        app.post(countlyConfig.path+'/events/delete_multi', function (req, res, next) {
			if (!req.session.uid || !req.body.app_id || !req.body.events) {
				res.end();
				return false;
			}
			
            var events = JSON.parse(req.body.events);
            async.each(events, function(key, callback){
                deleteEvent(req, key, req.body.app_id, function(result){
                    callback();
                })
            }, function(err, results) {});
			
			next();
		});
	};
    
    plugin.renderDashboard = function(ob){
        ob.data.countlyGlobal.list_limit = plugins.getConfig("drill").list_limit;
        ob.data.countlyGlobal.custom_property_limit = plugins.getConfig("drill").custom_property_limit;
        ob.data.countlyGlobal.record_sessions = plugins.getConfig("drill").record_sessions;
        ob.data.countlyGlobal.record_crashes = plugins.getConfig("drill").record_crashes;
    };
}(plugin));

module.exports = plugin;