var request = require('supertest');
var should = require('should');
var crypto = require('crypto');
var testUtils = require("../../test/testUtils");
var pluginManager = require("../../plugins/pluginManager.js"); 
request = request(testUtils.url);

var APP_KEY = "";
var API_KEY_ADMIN = "";
var APP_ID = "";
var DEVICE_ID = "1234567890";

describe('Testing Drill', function(){
	//{"data":{},"meta":[],"lu":"2015-01-20T12:00:06.176Z"}
	describe('Empty segmentation', function(){
		it('should success', function(done){
			API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject=%7B%7D&period=30days&bucket=daily")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("data",{});
				ob.should.have.property("meta",[]);
				ob.should.have.property("lu");
				setTimeout(done, 100)
			});
		});
	});
    describe('Empty projection', function(){
		it('should success', function(done){
			API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject=%7B%7D&period=30days&bucket=daily&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("data",{});
				ob.should.have.property("meta",[]);
				ob.should.have.property("lu");
				setTimeout(done, 100)
			});
		});
	});
    describe('Empty country projection', function(){
		it('should success', function(done){
			API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject=%7B%7D&period=30days&bucket=daily&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("meta",[]);
				ob.should.have.property("lu");
				setTimeout(done, 100)
			});
		});
	});
	describe('Empty segmentation meta', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation_meta")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				res.text.should.eql("null");
				setTimeout(done, 100)
			});
		});
	});
	describe('Start session', function(){
		it('should success', function(done){
            var params = {"_device": "Nexus 5x"};
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
	describe('Session duration', function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+'&session_duration=30')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
	describe('End session', function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+'&end_session=1')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	//{"data":{"2015.1.20":{"u":1,"t":1,"s":0}},"meta":[],"lu":"2015-01-20T10:11:43.376Z"}
    //change query to avoid hitting cache
	describe('Verify hourly session segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=hour&bucket=hourly")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(1);
					ob.data[i].t.should.equal(1);
					ob.data[i].s.should.equal(0);
					ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify daily session segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=daily")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(1);
					ob.data[i].t.should.equal(1);
					ob.data[i].s.should.equal(0);
					ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify weekly session segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=weekly")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(1);
					ob.data[i].t.should.equal(1);
					ob.data[i].s.should.equal(0);
					ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify monthly session segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=month&bucket=monthly")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(1);
					ob.data[i].t.should.equal(1);
					ob.data[i].s.should.equal(0);
					ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify hourly session segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=hour&bucket=hourly&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify daily session segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=daily&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify weekly session segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=weekly&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify monthly session segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=month&bucket=monthly&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify hourly session segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=hour&bucket=hourly&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify daily session segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=daily&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify weekly session segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=weekly&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify monthly session segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cty\":{\"$in\":[\"Unknown\"]}}&period=month&bucket=monthly&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify session segmentation meta', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation_meta")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('e',"[CLY]_session");
				ob.should.have.property('app_id',APP_ID);
				ob.should.have.property('up',{"fs":{"type":"d"},"ls":{"type":"d"},"sc":{"type":"n"},"cty":{"type":"l","values":["Unknown"]},"cc":{"type":"l","values":["Unknown"]},"d":{"type":"l","values":["Nexus 5x"]}});
				setTimeout(done, 100)
			});
		});
	});
    describe('Start session', function(){
		it('should success', function(done){
            var params = {"_device": "iPhone7,1"};
			request
			.get('/i?device_id='+DEVICE_ID+'1&app_key='+APP_KEY+'&begin_session=1&metrics='+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
	describe('Session duration', function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+DEVICE_ID+'1&app_key='+APP_KEY+'&session_duration=30')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
	describe('End session', function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+DEVICE_ID+'1&app_key='+APP_KEY+'&end_session=1')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
    //{"data":{"2015.1.20":{"u":1,"t":1,"s":0}},"meta":[],"lu":"2015-01-20T10:11:43.376Z"}
	describe('Verify hourly session segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=hour&bucket=hourly")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(2);
					ob.data[i].t.should.equal(2);
					ob.data[i].s.should.equal(0);
                    ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify daily session segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=daily")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(2);
					ob.data[i].t.should.equal(2);
					ob.data[i].s.should.equal(0);
                    ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify weekly session segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=weekly")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(2);
					ob.data[i].t.should.equal(2);
					ob.data[i].s.should.equal(0);
                    ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify monthly session segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=month&bucket=monthly")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(2);
					ob.data[i].t.should.equal(2);
					ob.data[i].s.should.equal(0);
                    ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify hourly session segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=hour&bucket=hourly&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x", "iPhone7,1"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
                    ob.data[i]["iPhone7,1"].u.should.equal(1);
					ob.data[i]["iPhone7,1"].t.should.equal(1);
					ob.data[i]["iPhone7,1"].s.should.equal(0);
					ob.data[i]["iPhone7,1"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify daily session segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=daily&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x", "iPhone7,1"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
                    ob.data[i]["iPhone7,1"].u.should.equal(1);
					ob.data[i]["iPhone7,1"].t.should.equal(1);
					ob.data[i]["iPhone7,1"].s.should.equal(0);
					ob.data[i]["iPhone7,1"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify weekly session segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=weekly&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x", "iPhone7,1"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
                    ob.data[i]["iPhone7,1"].u.should.equal(1);
					ob.data[i]["iPhone7,1"].t.should.equal(1);
					ob.data[i]["iPhone7,1"].s.should.equal(0);
					ob.data[i]["iPhone7,1"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify monthly session segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=month&bucket=monthly&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x", "iPhone7,1"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
                    ob.data[i]["iPhone7,1"].u.should.equal(1);
					ob.data[i]["iPhone7,1"].t.should.equal(1);
					ob.data[i]["iPhone7,1"].s.should.equal(0);
					ob.data[i]["iPhone7,1"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify hourly session segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=hour&bucket=hourly&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify daily session segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=daily&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify weekly session segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=30days&bucket=weekly&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify monthly session segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation&queryObject={\"up.cc\":{\"$in\":[\"Unknown\"]}}&period=month&bucket=monthly&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify session segmentation meta', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=%5BCLY%5D_session&method=segmentation_meta")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('e',"[CLY]_session");
				ob.should.have.property('app_id',APP_ID);
				ob.should.have.property('up',{"fs":{"type":"d"},"ls":{"type":"d"},"sc":{"type":"n"},"cty":{"type":"l","values":["Unknown"]},"cc":{"type":"l","values":["Unknown"]},"d":{"type":"l","values":["Nexus 5x","iPhone7,1"]}});
				setTimeout(done, 100)
			});
		});
	});
	describe('Creating event', function(){
		it('should success', function(done){
			var params = [{
				"key": "test",
				"count": 1,
				"segmentation": {
					"version": "1.0",
					"country": "Turkey"
				}
			}];
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&events="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify hourly event segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=hour&bucket=hourly")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(1);
					ob.data[i].t.should.equal(1);
					ob.data[i].s.should.equal(0);
					ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify daily event segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=30days&bucket=daily")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(1);
					ob.data[i].t.should.equal(1);
					ob.data[i].s.should.equal(0);
					ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify weekly event segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=30days&bucket=weekly")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(1);
					ob.data[i].t.should.equal(1);
					ob.data[i].s.should.equal(0);
					ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify monthly event segmentation', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=month&bucket=monthly")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',[]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i].u.should.equal(1);
					ob.data[i].t.should.equal(1);
					ob.data[i].s.should.equal(0);
					ob.data[i].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify hourly event segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=hour&bucket=hourly&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify daily event segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=30days&bucket=daily&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify weekly event segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=30days&bucket=weekly&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify monthly event segmentation with projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=month&bucket=monthly&projectionKey=up.d")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Nexus 5x"]);
				ob.should.have.property('data').and.not.eql({});
				for(var i in ob.data){
					ob.data[i]["Nexus 5x"].u.should.equal(1);
					ob.data[i]["Nexus 5x"].t.should.equal(1);
					ob.data[i]["Nexus 5x"].s.should.equal(0);
					ob.data[i]["Nexus 5x"].dur.should.equal(0);
				}
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    
    describe('Verify hourly event segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=hour&bucket=hourly&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify daily event segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=30days&bucket=daily&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify weekly event segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=30days&bucket=weekly&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify monthly event segmentation with country projection', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation&queryObject=%7B%7D&period=month&bucket=monthly&projectionKey=up.cc")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('meta',["Unknown"]);
				ob.should.have.property('lu');
				setTimeout(done, 100)
			});
		});
	});
//{"e":"test","app_id":"54b6b5f55b15821c6dade6ea","up":{"fs":{"type":"d"},"ls":{"type":"d"},"tsd":{"type":"n"},"sc":{"type":"n"},"cty":{"type":"l","values":["Unknown"]},"cc":{"type":"l","values":["Unknown"]}},"sg":{"version":{"type":"n","values":[1]},"country":{"type":"l","values":["Turkey"]}}}

	describe('Verify event segmentation meta', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&event=test&method=segmentation_meta")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('e',"test");
				ob.should.have.property('app_id',APP_ID);
				ob.should.have.property('sg',{"version":{"type":"n","values":[1]},"country":{"type":"l","values":["Turkey"]}});
				ob.should.have.property('up');
                var up = ob.up;
                up.should.have.property("fs",{"type":"d"});
                up.should.have.property("ls",{"type":"d"});
                up.should.have.property("tsd",{"type":"n"});
                up.should.have.property("sc",{"type":"n"});
                up.should.have.property("cty",{"type":"l","values":["Unknown"]});
                up.should.have.property("cc",{"type":"l","values":["Unknown"]});
                up.should.have.property("d",{"type":"l","values":["Nexus 5x"]});
				setTimeout(done, 100)
			});
		});
	});
	describe('Reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
    
    describe('Testing user merge scenarios for drill events', function(){
        describe('Single user (anonymous and then logged in with merge) single device', function(){
            describe('Anonymous user session', function(){
                it('should success', function(done){
                    db = pluginManager.dbConnection("countly_drill");
                    var params = {"_os": "Android", "_os_version":"4.1", "_device":"Samsung S7"};
                    var events = [{"key": "UserA1", "count":1}];
                    request
                    .get('/i?device_id='+DEVICE_ID+'1&app_key='+APP_KEY+"&begin_session=1&metrics="+JSON.stringify(params)+"&events="+JSON.stringify(events))
                    .expect(200)
                    .end(function(err, res){
                        if (err) return done(err);
                        var ob = JSON.parse(res.text);
                        ob.should.have.property('result','Success');
                        setTimeout(done, 10000)
                    });
                });
            });
            describe('Verify user event', function(){
                it('should have event data', function(done){
                    var id = crypto.createHash('sha1').update("UserA1"+APP_ID).digest('hex');
                    db.collection('drill_events' + id).find({}).toArray(function (err, events){
                        events[0].should.have.property("did",DEVICE_ID+'1');
                        events[0].should.have.property("uid", "1");
                        setTimeout(done, 5000)
                    });
                });
            });
            describe('User Logs in', function(){
                it('should success', function(done){
                    request
                    .get('/i?device_id='+DEVICE_ID+'2&old_device_id='+DEVICE_ID+'1&app_key='+APP_KEY)
                    .expect(200)
                    .end(function(err, res){
                        if (err) return done(err);
                        var ob = JSON.parse(res.text);
                        ob.should.have.property('result','Success');
                        setTimeout(done, 5000)
                    });
                });
            });
            describe('Verify logged in user', function(){
                it('should have reassigned event data', function(done){
                    var id = crypto.createHash('sha1').update("UserA1"+APP_ID).digest('hex');
                    db.collection('drill_events' + id).find({}).toArray(function (err, events){
                        events[0].should.have.property("did",DEVICE_ID+'1');
                        events[0].should.have.property("uid", "1");
                        setTimeout(done, 5000)
                    });
                });
            });
        });
        
        describe('Single user (anonymous and then logged in with merge) multi device', function(){
            describe('Anonymous user session', function(){
                it('should success', function(done){
                    var params = {"_os": "iOS", "_os_version":"9.1", "_device":"iPad"};
                    var events = [{"key": "UserA1", "count":1}];
                    request
                    .get('/i?device_id='+DEVICE_ID+'3&app_key='+APP_KEY+"&begin_session=1&metrics="+JSON.stringify(params)+"&events="+JSON.stringify(events))
                    .expect(200)
                    .end(function(err, res){
                        if (err) return done(err);
                        var ob = JSON.parse(res.text);
                        ob.should.have.property('result','Success');
                        setTimeout(done, 10000)
                    });
                });
            });
            describe('Verify user event', function(){
                it('should have event data', function(done){
                    var id = crypto.createHash('sha1').update("UserA1"+APP_ID).digest('hex');
                    db.collection('drill_events' + id).find({}).toArray(function (err, events){
                        var docs = [];
                        for(var i = 0; i < events.length; i++){
                            if((events[i]._id+"").substr(0, "meta".length) !== "meta"){
                                docs.push(events[i]);
                            }
                        }
                        docs[0].should.have.property("did",DEVICE_ID+'1');
                        docs[0].should.have.property("uid", "1");
                        docs[1].should.have.property("did",DEVICE_ID+'3');
                        docs[1].should.have.property("uid", "2");
                        setTimeout(done, 5000)
                    });
                });
            });
            describe('User Logs in', function(){
                it('should success', function(done){
                    request
                    .get('/i?device_id='+DEVICE_ID+'2&old_device_id='+DEVICE_ID+'3&app_key='+APP_KEY)
                    .expect(200)
                    .end(function(err, res){
                        if (err) return done(err);
                        var ob = JSON.parse(res.text);
                        ob.should.have.property('result','Success');
                        setTimeout(done, 5000)
                    });
                });
            });
            describe('Verify logged in user event', function(){
                it('should have reassigned event data', function(done){
                    var id = crypto.createHash('sha1').update("UserA1"+APP_ID).digest('hex');
                    db.collection('drill_events' + id).find({}).toArray(function (err, events){
                        var docs = [];
                        for(var i = 0; i < events.length; i++){
                            if((events[i]._id+"").substr(0, "meta".length) !== "meta"){
                                docs.push(events[i]);
                            }
                        }
                        docs[0].should.have.property("did",DEVICE_ID+'1');
                        docs[0].should.have.property("uid", "1");
                        docs[1].should.have.property("did",DEVICE_ID+'3');
                        docs[1].should.have.property("uid", "1");
                        setTimeout(done, 5000)
                    });
                });
            });
        });
        
        describe('Single user (multi account merge)', function(){
            describe('First account', function(){
                it('should success', function(done){
                    var params = {"_os": "Android", "_os_version":"5.1", "_device":"Sony Xperia"};
                    var events = [{"key": "UserA1", "count":1}];
                    request
                    .get('/i?device_id='+DEVICE_ID+'4&app_key='+APP_KEY+"&begin_session=1&metrics="+JSON.stringify(params)+"&events="+JSON.stringify(events))
                    .expect(200)
                    .end(function(err, res){
                        if (err) return done(err);
                        var ob = JSON.parse(res.text);
                        ob.should.have.property('result','Success');
                        setTimeout(done, 10000)
                    });
                });
            });
            describe('Verify user event', function(){
                it('should have event data', function(done){
                    var id = crypto.createHash('sha1').update("UserA1"+APP_ID).digest('hex');
                    db.collection('drill_events' + id).find({}).toArray(function (err, events){
                        var docs = [];
                        for(var i = 0; i < events.length; i++){
                            if((events[i]._id+"").substr(0, "meta".length) !== "meta"){
                                docs.push(events[i]);
                            }
                        }
                        docs[0].should.have.property("did",DEVICE_ID+'1');
                        docs[0].should.have.property("uid", "1");
                        docs[1].should.have.property("did",DEVICE_ID+'3');
                        docs[1].should.have.property("uid", "1");
                        docs[2].should.have.property("did",DEVICE_ID+'4');
                        docs[2].should.have.property("uid", "3");
                        setTimeout(done, 5000)
                    });
                });
            });
            describe('Second account', function(){
                it('should success', function(done){
                    var params = {"_os": "iOS", "_os_version":"8.1", "_device":"iPhone"};
                    var events = [{"key": "UserA1", "count":1}];
                    request
                    .get('/i?device_id='+DEVICE_ID+'5&app_key='+APP_KEY+"&begin_session=1&metrics="+JSON.stringify(params)+"&events="+JSON.stringify(events))
                    .expect(200)
                    .end(function(err, res){
                        if (err) return done(err);
                        var ob = JSON.parse(res.text);
                        ob.should.have.property('result','Success');
                        setTimeout(done, 10000)
                    });
                });
            });
            describe('Verify user event', function(){
                it('should have event data', function(done){
                    var id = crypto.createHash('sha1').update("UserA1"+APP_ID).digest('hex');
                    db.collection('drill_events' + id).find({}).toArray(function (err, events){
                        var docs = [];
                        for(var i = 0; i < events.length; i++){
                            if((events[i]._id+"").substr(0, "meta".length) !== "meta"){
                                docs.push(events[i]);
                            }
                        }
                        docs[0].should.have.property("did",DEVICE_ID+'1');
                        docs[0].should.have.property("uid", "1");
                        docs[1].should.have.property("did",DEVICE_ID+'3');
                        docs[1].should.have.property("uid", "1");
                        docs[2].should.have.property("did",DEVICE_ID+'4');
                        docs[2].should.have.property("uid", "3");
                        docs[3].should.have.property("did",DEVICE_ID+'5');
                        docs[3].should.have.property("uid", "4");
                        setTimeout(done, 5000)
                    });
                });
            });
            describe('Merge accounts', function(){
                it('should success', function(done){
                    request
                    .get('/i?device_id='+DEVICE_ID+'5&old_device_id='+DEVICE_ID+'4&app_key='+APP_KEY)
                    .expect(200)
                    .end(function(err, res){
                        if (err) return done(err);
                        var ob = JSON.parse(res.text);
                        ob.should.have.property('result','Success');
                        setTimeout(done, 5000)
                    });
                });
            });
            describe('Verify logged in user event', function(){
                it('should have reassigned event data', function(done){
                    var id = crypto.createHash('sha1').update("UserA1"+APP_ID).digest('hex');
                    db.collection('drill_events' + id).find({}).toArray(function (err, events){
                        var docs = [];
                        for(var i = 0; i < events.length; i++){
                            if((events[i]._id+"").substr(0, "meta".length) !== "meta"){
                                docs.push(events[i]);
                            }
                        }
                        docs[0].should.have.property("did",DEVICE_ID+'1');
                        docs[0].should.have.property("uid", "1");
                        docs[1].should.have.property("did",DEVICE_ID+'3');
                        docs[1].should.have.property("uid", "1");
                        docs[2].should.have.property("did",DEVICE_ID+'4');
                        docs[2].should.have.property("uid", "4");
                        docs[3].should.have.property("did",DEVICE_ID+'5');
                        docs[3].should.have.property("uid", "4");
                        setTimeout(done, 5000)
                    });
                });
            });
        });
        
        describe('reset app', function(){
            describe('reseting data', function(){
                it('should reset data', function(done){
                    this.timeout(20000);
                    var params = {app_id:APP_ID};
                    request
                    .get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
                    .expect(200)
                    .end(function(err, res){
                        if (err) return done(err);
                        var ob = JSON.parse(res.text);
                        ob.should.have.property('result', 'Success');
                        done();
                        db.close();
                    });
                });
            });
        });
    });
});