var plugin = {},
	common = require('../../../api/utils/common.js'),
	log = common.log('funnels:api'),
	_ = require('underscore'),
	countlyCommon = require('../../../api/lib/countly.common.js'),
	crypto = require('crypto'),
    plugins = require('../../pluginManager.js');

plugins.setConfigs("funnels", {
    funnel_step_limit: 8
});

//intersection of two map arrays, uses intersecting keys as result keys and values of second array for result values
function intersectUserTimes(x, y) {
    var obj = {},
        totals = {};

    for (var i = x.length-1; i >= 0; -- i) {
        if (obj[x[i]._id]) {
            obj[x[i]._id]++;
        } else {
            obj[x[i]._id] = 1;
        }
    }

    for (var i = y.length-1; i >= 0; -- i) {
        totals[y[i]._id] = y[i].t;

        if (obj[y[i]._id]) {
            obj[y[i]._id]++;
        } else {
            obj[y[i]._id] = 1;
        }
    }

    var res = [];
    for (var k in obj) {
        if (obj[k] > 1) {
            res.push({
                _id: k,
                t: totals[k]
            });
        }
    }

    return res;
};

(function (plugin) {
	plugins.register('/i/apps/create', function(ob){
		// add 10 min TTL index to drill_cache collection
		common.db.collection('funnels_cache').ensureIndex({date: 1}, {expireAfterSeconds: 600}, function() {});
	});

	//read api call
	plugins.register("/o", function(ob){
		var params = ob.params;
		var validate = ob.validateUserForDataReadAPI;
		if(params.qstring.method == 'funnel'){
            validate(params, function(params){
				if (!params.qstring.funnel) {
					common.returnMessage(params, 400, 'Missing request parameter: funnel');
					return false;
				}

				if (params.qstring.period) {
					//check if period comes from datepicker
					if (params.qstring.period.indexOf(",") !== -1) {
						try {
							params.qstring.period = JSON.parse(params.qstring.period);
						} catch (SyntaxError) {
							log.w('Parsing custom period failed!');
							common.returnMessage(params, 400, 'Bad request parameter: period');
							return false;
						}
					}
					else {
						switch (params.qstring.period) {
							case "month":
							case "day":
							case "yesterday":
							case "hour":
							case "7days":
							case "30days":
							case "60days":
								break;
							default:
								common.returnMessage(params, 400, 'Bad request parameter: period');
								return false;
								break;
						}
					}
				} else {
					common.returnMessage(params, 400, 'Missing request parameter: period');
					return false;
				}

				countlyCommon.setPeriod(params.qstring.period);
				countlyCommon.setTimezone(params.appTimezone);

				var queryObject = {
						ts: {}
					},
					periodObj = countlyCommon.periodObj;

				//get timestamps of start of days (DD-MM-YYYY-00:00) with respect to apptimezone for both beginning and end of period arrays
				var tmpArr;

				tmpArr = periodObj.currentPeriodArr[0].split(".");
				queryObject.ts.$gte = new Date(Date.UTC(parseInt(tmpArr[0]), parseInt(tmpArr[1]) - 1, parseInt(tmpArr[2])));
				queryObject.ts.$gte.setTimezone(params.appTimezone);
				queryObject.ts.$gte = queryObject.ts.$gte.getTime() + queryObject.ts.$gte.getTimezoneOffset() * 60000;

				tmpArr = periodObj.currentPeriodArr[periodObj.currentPeriodArr.length - 1].split(".");
				queryObject.ts.$lt = new Date(Date.UTC(parseInt(tmpArr[0]), parseInt(tmpArr[1]) - 1, parseInt(tmpArr[2])));
				queryObject.ts.$lt.setDate(queryObject.ts.$lt.getDate() + 1);
				queryObject.ts.$lt.setTimezone(params.appTimezone);
				queryObject.ts.$lt = queryObject.ts.$lt.getTime() + queryObject.ts.$lt.getTimezoneOffset() * 60000;

				queryObject._id = { $ne: "meta" };
				queryObject.uid = { $exists: true};

				//check funnel database for the requested funnel
				common.db.collection("funnels").findOne({_id: params.qstring.funnel}, log.callback(function(funnelsResult) {
					if (funnelsResult) {
						if(common.drillDb)
							plugins.cachedStepThroughFunnel(params.qstring, null, funnelsResult, 0, queryObject, 'initial', 0, function(err, data){
								if (err) {
									if (err.length === 2) {
										common.returnMessage(params, err[0], err[1]);
									} else {
										common.returnMessage(params, 400, err);
									}
								} else {
									common.returnOutput(params, data);
								}
							});
						else{
							var ret = {};
							ret.steps = [];
							ret.total_users = 0;
							ret.success_users = 0;
							ret.success_rate = 0;
							for(var i = 0, l = funnelsResult.steps.length; i < l; i++){
								ret.steps.push({step: funnelsResult.steps[i], users: 0, times: 0, percent: 0});
							}
							common.returnOutput(params, ret);
						}
					} else {
						common.returnMessage(params, 400, 'Requested funnel does not exist');
						return false;
					}
				}));

			});
			return true;
		}
		else if(params.qstring.method == 'get_funnels'){
            validate(params, function(params){
				common.db.collection('funnels').find({app_id:params.qstring.app_id}, {app_id:0}).toArray(function(err,result){
				// common.db.collection('funnels').find({app_id:params.qstring.app_id, flow: {$exists: false}}, {app_id:0}).toArray(function(err,result){
					if (result) {
						common.returnOutput(params, _.sortBy(result, function(funnel) {
                            if (funnel.order) {
                                return funnel.order;
                            } else {
                                return funnel.name;
                            }
                        }));
					} else {
						common.returnOutput(params, []);
					}
				});
			});
			return true;
		}
	});

	plugins.cachedStepThroughFunnel = function (qstring, funnelReturnObject, funnel, funnelStepNo, queryObject, funnelResultArr, minTs, callback) {
        if(typeof qstring.users_for_step != "undefined"){
            plugin.stepThroughFunnel(qstring, funnelReturnObject, funnel, 0, queryObject, 'initial',  minTs, function(err, data){
                if (err) {
                    if (callback) { callback(err); }
                } else {
                    if (callback) { callback(null, data); }
                }
            });
        }
        else{
            common.db.collection('funnels_cache').findOne({_id: funnel._id + qstring.period, date: {$gt: Date.now() - 1 * 60 * 1000}}, function(err, cache){
                if (cache) {
                    if (callback) { callback(null, cache.data); }
                } else {
                    plugin.stepThroughFunnel(qstring, funnelReturnObject, funnel, 0, queryObject, 'initial',  minTs, function(err, data){
                        if (err) {
                            if (callback) { callback(err); }
                        } else {
                            var cache = {
                                _id: funnel._id + qstring.period,
                                date: new Date(),
                                data: data
                            };
                            common.db.collection('funnels_cache').save(cache, function(){});
                            if (callback) { callback(null, data); }
                        }
                    });
                }
            });
        }
	};

	// plugins.stepThroughFunnel = function (app_id, funnelStepsArr, funnelStepNo, queryObject, funnelResultArr, minTs) {
	plugin.stepThroughFunnel = function (qstring, funnelReturnObject, funnel, funnelStepNo, queryObject, funnelResultArr, minTs, callback) {
		log.d('======================= stepping through funnel', arguments);
		funnelReturnObject = funnelReturnObject || {steps: [], total_users: 0, success_users: 0, success_rate: 0.0};
		minTs = minTs || 0;

		var tmpQueryObject = {};

		if (minTs && queryObject.ts.$gte < minTs) {
			queryObject.ts.$gte = minTs;
		}

		if (funnelResultArr == 'initial') {
			// Apply the filter only to the first event
			if (qstring.filter) {
				log.d('filter %j', qstring.filter);

				try {
					tmpQueryObject = JSON.parse(qstring.filter);
					log.d('filter parsed %j', tmpQueryObject);
				} catch (SyntaxError) {}
			}
			tmpQueryObject.ts = queryObject.ts;
			tmpQueryObject._id = queryObject._id;
			tmpQueryObject.uid = queryObject.uid;
		} else {
			tmpQueryObject = queryObject;
		}

		var collName = 'drill_events' + (crypto.createHash('sha1').update(funnel.steps[funnelStepNo] + funnel.app_id).digest('hex'));


		// Until the last step we need to find the minimum timestamp in the result set and
		// feed it to the next stepThroughFunnel call. This ensures that the proceeding
		// funnel step query respects the minimum timestamp of the previous step
		if (funnelStepNo != funnel.steps.length) {
			common.drillDb.collection(collName).aggregate([
				{
					$match: tmpQueryObject
				},
				{
					$group: {
						_id: 0,
						minTs: {
							$min: '$ts'
						}
					}
				}
			], function (minErr, minDoc) {
				log.d('first match result for %j(%j) / %j / %j is %j', collName, funnel.steps[funnelStepNo], funnel.steps, tmpQueryObject, minDoc);
				var resultMinTs = 0;

				if (minDoc && minDoc[0]) {
					resultMinTs = minDoc[0].minTs;
				}

				getStepData(resultMinTs);
			});
		} else {
			getStepData(0);
		}

		function getStepData(stepMinTs) {
			var totalTimes = 0, i, l;
			common.drillDb.collection(collName).aggregate([
				{
					$match: tmpQueryObject
				},
				{
					$group: {
						_id: '$uid',
						t: {
							$sum: '$c'
						}
					}
				}
			], function (err, result) {
				log.d('getStepData result for %j(%j) / %j / %j is %j', collName, funnel.steps[funnelStepNo], funnel.steps, tmpQueryObject, result);
                if(typeof qstring.users_for_step != "undefined" && qstring.users_for_step == funnelStepNo){
                    var res = [];
                    if (funnelResultArr == "initial") {
                        for(var i = 0; i < result.length; i++){
                            res.push(result[i]._id);
                        }
                    }
                    else{
                        var middleResultArr = intersectUserTimes(funnelResultArr, result);
                        for(var i = 0; i < middleResultArr.length; i++){
                            res.push(middleResultArr[i]._id);
                        }
                    }

                    return callback ? callback(null, res) : undefined;
                }
				funnelStepNo++;

				//beginning event of the funnel
				if (funnelResultArr == 'initial') {
					if (funnelStepNo != funnel.steps.length) {
						totalTimes = 0;
						if (result) {
							for (i = result.length - 1; i >= 0; --i) {
								totalTimes += result[i].t;
							}

							funnelReturnObject.steps.push({step: funnel.steps[funnelStepNo - 1], users: result.length, times: totalTimes, percent: (result.length) ? 100 : 0});
							funnelReturnObject.total_users = result.length;
							plugin.stepThroughFunnel(qstring, funnelReturnObject, funnel, funnelStepNo, queryObject, result, stepMinTs, callback);
						} else {
							//common.returnMessage(params, 500, 'Server error, try again.');
							var ret = {};
							ret.steps = [];
							ret.total_users = 0;
							ret.success_users = 0;
							ret.success_rate = 0;
							for(i = 0, l = funnel.steps.length; i < l; i++){
								ret.steps.push({step: funnel.steps[i], users: 0, times: 0, percent: 0});
							}
							return callback ? callback(null, ret) : undefined;
						}
					} else {
						return callback ? callback([400, 'Requested funnel has only one event, add more events to funnel list']) : undefined;
					}
				} else if (funnelResultArr) {
					//main loop which executes as long as the events are not empty
					var middleResultArr = intersectUserTimes(funnelResultArr, result);

					totalTimes = 0;
					for (i = middleResultArr.length - 1; i >= 0; --i) {
						totalTimes += middleResultArr[i].t;
					}

					funnelReturnObject.steps.push({step: funnel.steps[funnelStepNo - 1], users: middleResultArr.length, times: totalTimes, percent: (Math.round(common.safeDivision(middleResultArr.length, funnelReturnObject.total_users) * 1000) / 10)});
					if (funnelStepNo != funnel.steps.length) {
						plugin.stepThroughFunnel(qstring, funnelReturnObject, funnel, funnelStepNo, queryObject, middleResultArr, stepMinTs, callback);
					} else {
						funnelReturnObject.success_users = middleResultArr.length;
						funnelReturnObject.success_rate = (Math.round(common.safeDivision(funnelReturnObject.success_users, funnelReturnObject.total_users) * 1000) / 10);
						return callback ? callback(null, funnelReturnObject) : undefined;
					}
				} else {
					//if an event in the middle of funnel is empty fill the rest of the events with zeros and return
					while (funnelStepNo != funnel.steps.length) {
						funnelReturnObject.steps.push({step: funnel.steps[funnelStepNo - 1], users: 0, times: 0});
					}

					funnelReturnObject.steps.push({step: funnel.steps[funnelStepNo - 1], users: 0, times: 0, percent: 0});
					funnelReturnObject.success_users = 0;
					funnelReturnObject.success_rate = 0;
					return callback ? callback(null, funnelReturnObject) : undefined;
				}
			});
		}
	}


	plugins.register("/i/apps/delete", function(ob){
		var appId = ob.appId;
		common.db.collection("funnels").remove({app_id:appId},function(){});
	});

	plugins.register("/i/apps/reset", function(ob){
		var appId = ob.appId;
		common.db.collection("funnels").remove({app_id:appId},function(){});
	});
}(plugin));

module.exports = plugin;