var plugin = {},
	crypto = require('crypto'),
	countlyConfig = require('../../../frontend/express/config', 'dont-enclose'),
	async = require('async'),
    plugins = require('../../pluginManager.js');
    
var countlyFunnelConfig
try{
    countlyFunnelConfig = require('../config');
}
catch(err) {
    countlyFunnelConfig = {};
}

(function (plugin) {
	plugin.init = function(app, countlyDb){
		app.post(countlyConfig.path+'/funnel/add', function (req, res, next) {
			if (!req.session.uid || !req.body.app_id || !req.body.steps || req.body.steps.length < 2 || !req.body.funnel_name) {
				res.end();
				return false;
			}
		
			if (!req.session.gadm) {
				countlyDb.collection('members').findOne({"_id":countlyDb.ObjectID(req.session.uid)}, function (err, member) {
					if (!err && member.user_of && member.user_of.indexOf(req.body.app_id) != -1) {
						saveFunnel();
						return true;
					} else {
						res.send(false);
						return false;
					}
				});
			} else {
				saveFunnel();
				return true;
			}
		
			function saveFunnel() {
				var funnelId = crypto.createHash('md5').update(JSON.stringify(req.body.steps) + req.body.app_id).digest('hex');
		
				countlyDb.collection('funnels').findOne({"_id": funnelId}, function(err, funnel) {
					if (funnel) {
						res.send(funnel._id);
					} else {
						var newFunnel = {
							_id: funnelId,
							app_id: req.body.app_id,
							name: req.body.funnel_name,
							steps: req.body.steps
						};
		
						countlyDb.collection('funnels').save(newFunnel, function (err, funnel) {});
                        plugins.callMethod("logAction", {req:req, user:{_id:req.session.uid, email:req.session.email}, action:"funnel_added", data:newFunnel});
						res.send(funnelId);
					}
				});
			}
		});
		
		app.post(countlyConfig.path+'/funnel/edit', function (req, res, next) {
			if (!req.session.uid || !req.body.app_id || !req.body.funnel_map) {
				res.end();
				return false;
			}
		
			try {
				req.body.funnel_map = JSON.parse(req.body.funnel_map);
			} catch (SyntaxError) {
				res.end();
				return false;
			}
		
			var updateObjArr = [];
		
			if (!req.session.gadm) {
				countlyDb.collection('members').findOne({"_id":countlyDb.ObjectID(req.session.uid)}, function (err, member) {
					if (!err && member.admin_of && member.admin_of.indexOf(req.body.app_id) != -1) {
						for (var funnelId in req.body.funnel_map) {
							updateObjArr.push({
								appId: req.body.app_id,
								funnelId: funnelId,
								funnelName: req.body.funnel_map[funnelId].name,
                                funnelOrder: req.body.funnel_map[funnelId].order
							});
						}
		
						async.map(updateObjArr, updateFunnel, function (err, ret) {
							res.send(true);
							return true;
						});
					} else {
						res.send(false);
						return false;
					}
				});
			} else {
				for (var funnelId in req.body.funnel_map) {
					updateObjArr.push({
						appId: req.body.app_id,
						funnelId: funnelId,
						funnelName: req.body.funnel_map[funnelId].name,
                        funnelOrder: req.body.funnel_map[funnelId].order
					});
				}
		
				async.map(updateObjArr, updateFunnel, function (err, ret) {
					res.send(true);
					return true;
				});
			}
		
			function updateFunnel(obj, callback) {
                var updatePart = {
                    '$set': {
                        "name": obj.funnelName
                    }
                };

                if (obj.funnelOrder) {
                    updatePart["$set"]["order"] = obj.funnelOrder;
                }

				countlyDb.collection('funnels').update({"_id": obj.funnelId, "app_id": obj.appId}, updatePart, function (err, funnels) {
                    plugins.callMethod("logAction", {req:req, user:{_id:req.session.uid, email:req.session.email}, action:"funnel_updated", data:obj});
					callback(false);
				});
			}
		});
		
		app.post(countlyConfig.path+'/funnel/delete', function (req, res, next) {
			if (!req.session.uid || !req.body.app_id || !req.body.funnel_id) {
				res.end();
				return false;
			}
		
            countlyDb.collection('funnels').findOne({"_id": req.body.funnel_id, "app_id": req.body.app_id}, function(err, funnel) {
                if (!req.session.gadm) {
                    countlyDb.collection('members').findOne({"_id":countlyDb.ObjectID(req.session.uid)}, function (err, member) {
                        if (!err && member.admin_of && member.admin_of.indexOf(req.body.app_id) != -1) {
                            countlyDb.collection('funnels').remove({"_id": req.body.funnel_id, "app_id": req.body.app_id}, function(err, result) {
                                res.send(true);
                                return true;
                            });
                        } else {
                            res.send(false);
                            return false;
                        }
                    });
                } else {
                    countlyDb.collection('funnels').remove({"_id": req.body.funnel_id, "app_id": req.body.app_id}, function(err, result) {
                        res.send(true);
                        return true;
                    });
                }
                plugins.callMethod("logAction", {req:req, user:{_id:req.session.uid, email:req.session.email}, action:"funnel_deleted", data:funnel});
            });
		});
	};
    
    plugin.renderDashboard = function(ob){
        ob.data.countlyGlobal.funnel_step_limit = plugins.getConfig("funnels").funnel_step_limit;
    };
}(plugin));

module.exports = plugin;