(function(countlyFunnel, $, undefined) {

    //Private Properties
    var _activeFunnels = [],
        _activeFunnel = "",
        _activeFunnelData = {},
        _filter = {},
        _activeAppId = 0,
        _activeAppKey = 0,
        _initialized = false;

    //Public Methods
    countlyFunnel.initialize = function() {
		if(!_initialized){
			_activeAppKey = countlyCommon.ACTIVE_APP_KEY;
            _initialized = true;
		}
		else if(_activeAppKey != countlyCommon.ACTIVE_APP_KEY){
			countlyFunnel.reset();
		}
        return $.when(countlyFunnel.initFunnels()).then(
            function () {
                if (countlyFunnel.getFunnels().length) {
                    return countlyFunnel.initCurrentFunnel();
                } else {
                    return true;
                }
            }
        );
    };

    countlyFunnel.initCurrentFunnel = function() {
        _activeAppId = countlyCommon.ACTIVE_APP_ID;

        if (!countlyCommon.DEBUG) {
            _activeAppKey = countlyCommon.ACTIVE_APP_KEY;
            _initialized = true;

            var period = countlyCommon.getPeriod();
            if (Object.prototype.toString.call(period) === '[object Array]'){
                period = JSON.stringify(period);
            }

            return $.when($.ajax({
                type: "GET",
                url: countlyCommon.API_PARTS.data.r,
                data: {
                    "api_key": countlyGlobal.member.api_key,
                    "app_id" : countlyCommon.ACTIVE_APP_ID,
                    "method" : "funnel",
                    "funnel": _activeFunnel,
                    "filter": JSON.stringify(_filter),
                    "period": period
                },
                dataType: "jsonp",
                success: function(json) {
                    _activeFunnelData = json;
                    setMeta();
                }
            })).then(function() {
                var funnelData = countlyFunnel.getFunnelData();

                if (funnelData && typeof countlySegmentation != "undefined" && funnelData.steps && funnelData.steps[0] && funnelData.steps[0].key) {
                    countlySegmentation.reset();
                    return countlySegmentation.initialize(funnelData.steps[0].key);
                } else {
                    return true;
                }
            }, function(){return $.when(false);});

        } else {
            _activeFunnels = {};
            return true;
        }
    };

    countlyFunnel.initFunnels = function () {
        if (!countlyCommon.DEBUG) {
            return $.ajax({
                type:"GET",
                url:countlyCommon.API_PARTS.data.r,
                data:{
                    "api_key": countlyGlobal.member.api_key,
                    "app_id": countlyCommon.ACTIVE_APP_ID,
                    "method": "get_funnels"
                },
                dataType:"jsonp",
                success:function (json) {
                    _activeFunnels = json;

                    if (!_activeFunnel && countlyFunnel.getFunnels()[0]) {
                        _activeFunnel = countlyFunnel.getFunnels()[0]._id;
                    }
                }
            });
        } else {
            _activeFunnels = {};
            return true;
        }
    };

    countlyFunnel.reset = function () {
        _activeFunnels = [];
        _activeFunnel = "";
        _activeFunnelData = {};
        _filter = {};
        _activeAppId = 0;
        _activeAppKey = 0;
        _initialized = false;
    };

    countlyFunnel.getActiveFunnel = function () {
        return _activeFunnel;
    };

    countlyFunnel.getActiveFunnelName = function () {
        var funnelIds = _.pluck(_activeFunnels, "_id");

        if (funnelIds.indexOf(_activeFunnel) !== -1) {
            return _activeFunnels[funnelIds.indexOf(_activeFunnel)].name;
        } else {
            return "";
        }
    };

    countlyFunnel.setActiveFunnel = function (activeFunnel, callback) {
        for (var i = 0; i < _activeFunnels.length; i++) {
            if (_activeFunnels[i]._id == activeFunnel) {
                _activeFunnels[i].is_active = true;
            } else {
                delete _activeFunnels[i].is_active;
            }
        }

        _activeFunnel = activeFunnel;

        if (callback) {
            $.when(countlyFunnel.initialize()).then(callback);
        }
    };

    countlyFunnel.setFilter = function (filterObj, callback) {
        _filter = filterObj;

        if (callback) {
            $.when(countlyFunnel.initialize()).then(callback);
        }
    };

    countlyFunnel.getFunnels = function() {
        var funnelIds = _.pluck(_activeFunnels, "_id");

        if (funnelIds.indexOf(_activeFunnel) !== -1) {
            _activeFunnels[funnelIds.indexOf(_activeFunnel)].is_active = true;
        }

        return _.sortBy(_activeFunnels, 'order');
    };

    countlyFunnel.getFunnelData = function() {
        if (_activeFunnelData && _activeFunnelData.steps && _activeFunnelData.steps.length) {
            for (var i = 0; i < _activeFunnelData.steps.length; i++) {
                _activeFunnelData.steps[i].key = _activeFunnelData.steps[i].step;
                _activeFunnelData.steps[i].step = countlyEvent.getEventLongName(_activeFunnelData.steps[i].step);
            }
        }

        return _activeFunnelData;
    };
    
    countlyFunnel.getRequestData = function() {
        var period = countlyCommon.getPeriod();
        if (Object.prototype.toString.call(period) === '[object Array]'){
            period = JSON.stringify(period);
        }
        return {
                    "api_key": countlyGlobal.member.api_key,
                    "app_id" : countlyCommon.ACTIVE_APP_ID,
                    "method" : "funnel",
                    "funnel": _activeFunnel,
                    "filter": JSON.stringify(_filter),
                    "period": period
                };
    };

    countlyFunnel.deleteFunnel = function(funnelId) {
        var funnelIds = _.pluck(_activeFunnels, "_id");

        if (funnelIds.indexOf(funnelId) !== -1) {
            delete _activeFunnels[funnelIds.indexOf(funnelId)];
        }

        _activeFunnels = _.compact(_activeFunnels);
    };

    countlyFunnel.setFunnels = function(funnelMap) {
        for (var funnelId in funnelMap) {
            for (var i = 0; i < _activeFunnels.length; i++) {
                if (_activeFunnels[i]._id == funnelId) {
                    if (funnelMap[funnelId].name) {
                        _activeFunnels[i].name = funnelMap[funnelId].name;
                    }
                    _activeFunnels[i].order = funnelMap[funnelId].order;
                    break;
                }
            }
        }
    };

    function setMeta() {

    }

}(window.countlyFunnel = window.countlyFunnel || {}, jQuery));