window.FunnelView = countlyView.extend({
    beforeRender: function() {
        if (!_.isEmpty(this.filterObj)) {
            this.filterObj = {};
            countlyFunnel.setFilter(this.filterObj);
        }

        if(this.template)
			return $.when(countlyFunnel.initialize()).then(function () {});
		else{
			var self = this;
			return $.when($.get(countlyGlobal["path"]+'/funnels/templates/funnels.html', function(src){
				self.template = Handlebars.compile(src);
			}), countlyFunnel.initialize()).then(function () {});
		}
    },
    initialize:function () {},
    initFunnelView: function() {
        var self = this;

        $("#show-new-funnel-view").on("click", function() {
            $("#view-funnel").fadeOut("fast", function() {
                $("#new-funnel").fadeIn("fast");
                $("#new-funnel-name").val("");
                $("#new-funnel").find(".funnel-events").empty();
                $("#new-funnel").find(".step-funnel-icons").empty();
                $("#add-step").trigger("click");

                if (countlyFunnel.getFunnels().length == 0) {
                    $("#funnel-cancel").addClass("disabled");
                } else {
                    $("#funnel-cancel").removeClass("disabled");
                }
            });
        });

        this.drawFunnelView();
    },
    initNewFunnelView: function() {
        resetChanges();

        if (countlyFunnel.getFunnels().length == 0) {
            $("#funnel-cancel").addClass("disabled");
        } else {
            $("#funnel-cancel").removeClass("disabled");
        }

        var newFunnelStep = $(".new-funnel-step").clone(),
            stepFunnelIcon = $(".step-funnel-icon").clone(),
            stepId = 1,
            usedEvents = [],
            self = this;

        newFunnelStep.removeClass("new-funnel-step");
        stepFunnelIcon.removeClass("step-funnel-icon");

        $("#add-step").on("click", function() {
            var limit = countlyGlobal.funnel_step_limit || 8;
            if (getStepCount() == limit) {
                return false;
            }

            if (getStepCount() != 0 && !$(".funnel-events>div:visible:last-child").find(".cly-text-select .text").data("value")) {
                return false;
            }

            var tmpStepFunnelIcon = stepFunnelIcon.clone(),
                tmpNewFunnelStep = newFunnelStep.clone();

            tmpNewFunnelStep.attr("data-step", stepId);
            tmpStepFunnelIcon.attr("data-step", stepId);
            stepId++;

            $(".step-funnel-icons").append(tmpStepFunnelIcon.fadeIn("slow"));
            $("#new-funnel").find(".funnel-events").append(tmpNewFunnelStep.fadeIn("slow"));
            adjustStepInfo();
        });

        $("#new-funnel").on("click", ".delete-step", function() {
            if (getStepCount() == 1) {
                return false;
            }

            var stepNo = $(this).parents(".step-icon").data("step");
            $(this).parents(".step-icon").fadeOut("fast", function() { $(this).remove(); });

            $("#new-funnel").find(".funnel-events>div[data-step='" + stepNo + "']").fadeOut("fast", function() {
                $(this).remove();
                adjustStepInfo();
            });
        });

        $("#new-funnel").on("focus keyup", ".funnel-events .cly-text-select input", function() {
            var allEvents = countlyEvent.getEvents(),
                eventStr = "";

            for (var i = 0; i < allEvents.length; i++) {
                if (usedEvents.indexOf(allEvents[i].key) === -1) {
                    var tmpItem = $("<div>");

                    tmpItem.addClass("item");
                    tmpItem.attr("data-value", allEvents[i].key);
                    tmpItem.text(allEvents[i].name);

                    eventStr += tmpItem.prop('outerHTML');
                }
            }

            $(this).parents(".cly-text-select").find(".select-items>div").html(eventStr);
            adjustStepInfo();
        });

        $("#new-funnel").on("mouseup", ".select-items .item", function() {
            setTimeout(function(){
                adjustStepInfo()
            }, 0);
        });

        $("#funnel-cancel").on("click", function() {
            if ($(this).hasClass("disabled")) {
                return;
            }

            $("#new-funnel").fadeOut("fast", function() {
                $("#view-funnel").fadeIn("fast");
                resetChanges();
            });
        });

        $("#funnel-save").on("click", function() {
            if (getStepCount() < 2 || $(this).hasClass("disabled")) {
                return false;
            }
            
            var funnelNameEl = $("#new-funnel-name");

            if (!funnelNameEl.val() || funnelNameEl.val() == funnelNameEl.attr("placeholder")) {
                funnelNameEl.val(funnelNameEl.attr("placeholder"));
                funnelNameEl.addClass("required-color");
                return false;
            }

            $(this).find(".text").text(jQuery.i18n.map["common.saving"]);
            $(this).addClass("disabled");

            $.ajax({
                type:"POST",
                url:countlyGlobal["path"]+"/funnel/add",
                data:{
                    app_id: countlyCommon.ACTIVE_APP_ID,
                    funnel_name: funnelNameEl.val(),
                    steps: getCurrentSteps(),
                    _csrf: countlyGlobal['csrf_token']
                },
                success:function (result) {
                    if (result && countlyFunnel.getFunnels().length) {
                        countlyFunnel.setActiveFunnel(result);
                        countlyFunnel.initFunnels();
                        self.loadAndDraw(true);
                    } else {
                        $.when(countlyFunnel.initialize()).then(function() {
                            self.drawFunnelView();
                            self.refreshFunnelList();
                            resetChanges();
                        });
                    }

                    self.resetFilter();
                }
            });
        });

        $("#new-funnel-name").on("focus", function() {
            if ($(this).val() == $(this).attr("placeholder")) {
                $(this).val("");
            }

            $(this).removeClass("required-color");
        });

        function getCurrentSteps() {
            var steps = [];
            $(".funnel-events>div:visible").each(function(index) {
                var stepEventKey = $(this).find(".cly-text-select .text").data("value");

                if (stepEventKey) {
                    steps.push(stepEventKey);
                }
            });

            return steps;
        }

        function adjustStepInfo() {
            var step = 1,
                stepLength = $(".funnel-events>div:visible").length,
                stepsWithDataCount = 0;

            $(".funnel-events>div:visible").each(function(index) {
                $(".step-funnel-icons").find(".data-part:nth-child(" + (index + 1) + ") .connector>span").text(step++);

                if ($(this).find(".cly-text-select .text").data("value")) {
                    stepsWithDataCount++;
                }

                if (index != (stepLength - 1)) {
                    $(this).find(".cly-text-select").addClass("disabled");
                    $(this).find(".cly-text-select input").attr("readonly","readonly");
                } else {
                    $(this).find(".cly-text-select").removeClass("disabled");
                    $(this).find(".cly-text-select input").removeAttr("readonly");
                }
            });

            if (stepsWithDataCount >= 2) {
                $("#funnel-save").removeClass("disabled");
            } else {
                $("#funnel-save").addClass("disabled");
            }

            usedEvents = getCurrentSteps();
        }

        function getStepCount() {
            return $(".funnel-events>div:visible").length;
        }

        function resetChanges() {
            $("#new-funnel-name").val("");
            $("#new-funnel").find(".funnel-events").empty();
            $("#new-funnel").find(".step-funnel-icons").empty();
            $("#funnel-save").removeClass("disabled");
            $("#funnel-save").find(".text").text(jQuery.i18n.map["common.save"]);
        }
    },
    drawFunnelView: function() {
        $("#view-funnel").find(".funnel-name-title").text(countlyFunnel.getActiveFunnelName());

        var funnelData = countlyFunnel.getFunnelData();

        if (funnelData.steps && funnelData.steps.length) {
            if ($("#view-funnel").is(":hidden")) {
                $("#view-funnel").show();
                $("#new-funnel").hide();
                $("#funnel-save").find(".text").text(jQuery.i18n.map["common.save"]);
            }

            $("#view-funnel").find(".total-users").data("target", funnelData.total_users);
            $("#view-funnel").find(".success_users").data("target", funnelData.success_users);
            $("#view-funnel").find(".success_rate").data("target", funnelData.success_rate);

            var funnelStepInfo = $("#view-funnel").find(".funnel-step-info").clone(),
                funnelStepData = $("#view-funnel").find(".funnel-step-data").clone();

            funnelStepInfo.removeClass("funnel-step-info");
            funnelStepData.removeClass("funnel-step-data");

            $("#view-funnel").find(".funnel-events").empty();
            $("#view-funnel").find(".funnel-step-data-container").empty();

            for (var i = 0; i < funnelData.steps.length; i++) {
                var tmpFunnelStepInfo = funnelStepInfo.clone(),
                    tmpFunnelStepData = funnelStepData.clone();

                tmpFunnelStepInfo.find(".step-name").text(funnelData.steps[i].step.toUpperCase());

                tmpFunnelStepData.find(".connector span").text(i+1);
                tmpFunnelStepData.find(".step-users").data("target", funnelData.steps[i].users);
                tmpFunnelStepData.find(".step-times").data("target", funnelData.steps[i].times);
                tmpFunnelStepData.find(".step-users-percent").data("percent", funnelData.steps[i].percent);
                tmpFunnelStepData.find(".step-users-percent-bar").data("percent", funnelData.steps[i].percent);
                tmpFunnelStepData.addClass("funnel-step-"+i);
                
                if(typeof countlyUserdata != "undefined" && funnelData.steps[i].users > 0) {
                    var request = countlyFunnel.getRequestData();
                    request.users_for_step = i;
                    tmpFunnelStepData.find(".users").wrap("<a href='#/users/request/"+JSON.stringify(request)+"' class='funnel-user-list'></a>");
                }

                $("#view-funnel").find(".funnel-events").append(tmpFunnelStepInfo.show());
                $("#view-funnel").find(".funnel-step-data-container").append(tmpFunnelStepData.show());
            }

            this.animateNumbers(2000);

            $("#view-funnel").find(".funnel-events>div").hide();

            $("#view-funnel").find(".funnel-events>div").each(function(i) {
                if (!$(this).hasClass("funnel-step-info")) {
                    var el = $(this);
                    setTimeout(function() { el.fadeIn(); }, (i * 300));
                }
            });
        } else {
            $("#view-funnel").hide();
            $("#new-funnel").fadeIn("fast");

            setTimeout(function(){ $("#add-step").trigger("click"); }, 0);
        }
    },
    refreshFunnelView: _.debounce(function() {
        var funnelData = countlyFunnel.getFunnelData();

        if (funnelData.steps && funnelData.steps.length) {
            $("#view-funnel").find(".total-users").data("target", funnelData.total_users);
            $("#view-funnel").find(".success_users").data("target", funnelData.success_users);
            $("#view-funnel").find(".success_rate").data("target", funnelData.success_rate);

            for (var i = 0; i < funnelData.steps.length; i++) {
                var tmpFunnelStepData = $($("#view-funnel").find(".funnel-step-data-container>div")[i]);

                tmpFunnelStepData.find(".step-users").data("target", funnelData.steps[i].users);
                tmpFunnelStepData.find(".step-times").data("target", funnelData.steps[i].times);
                tmpFunnelStepData.find(".step-users-percent").data("percent", funnelData.steps[i].percent);
                tmpFunnelStepData.find(".step-users-percent-bar").data("percent", funnelData.steps[i].percent);

                var request = countlyFunnel.getRequestData();

                // We can't filter users based on event segments so if
                // filter contains an event segment we don't attach a user profile link
                if (funnelData.steps[i].users > 0 && (request.filter + "").indexOf("sg.") == -1) {
                    if (typeof countlyUserdata != "undefined") {
                        var request = countlyFunnel.getRequestData();

                        request.users_for_step = i;
                        tmpFunnelStepData.find(".users").parent("a").attr("href", "#/users/request/" + JSON.stringify(request));
                    }
                } else {
                    tmpFunnelStepData.find(".users").parent("a").attr("href", "");
                }
            }

            this.animateNumbers(1000);
        }
    }, 500),
    loadAndDraw: function(refreshFunnelList) {
        var self = this;

        $.when(countlyFunnel.initCurrentFunnel()).then(function() {
            self.drawFunnelView();

            if (refreshFunnelList) {
                self.refreshFunnelList();
            }
        });
    },
    loadAndRefresh: function() {
        var self = this;

        $.when(countlyFunnel.initCurrentFunnel()).then(function() {
            self.refreshFunnelView();
        });
    },
    resetFilter: function() {
        this.filterObj = {};
        $(".filter-view-container").hide();
        $("#filter-view").replaceWith(this.filterBlockClone.clone(true));
        $("#current-filter").hide();
        $("#no-filter").show();
    },
    animateNumbers: function(duration) {
        var animDur = (duration)? duration : 2000;

        $(".moving-bar").each(function(i) {
            var target = $(this).data("percent");
            target *= 0.75;
            target = (target == 0)? 1 : target;

            var self = $(this);
            setTimeout(function() {
                self.css({
                    transition: "width " + ((animDur == 2000)? 1.8 : 0.8) + "s",
                    width: target + "%"
                });
            }, 0);
        });

        $('.moving-number').each(function(i) {
            var el = $(this),
                currVal = el.text().replace("%","") || 0,
                target = el.data("percent");

            jQuery({someValue: currVal, currEl: el}).animate({someValue: target}, {
                duration: animDur,
                easing:'easeInOutQuint',
                step: function() {
                    this.currEl.text(Math.round(this.someValue) + "%");
                }
            });
        });

        $('.num-animation').each(function(i) {
            var el = $(this),
                currVal = el.text() || 0,
                target = el.data("target");

            jQuery({someValue: currVal, currEl: el}).animate({someValue: target}, {
                duration: animDur,
                easing:'easeInOutQuint',
                step: function() {
                    this.currEl.text(Math.round(this.someValue));
                }
            });
        });
    },
    pageScript:function () {
        var self = this;

        $(document).off("click", "#edit-funnels-save");
        $(document).on("click", "#edit-funnels-save", function () {
            var funnelMap = {};

            $(".dialog .events-table tbody tr").each(function (index) {
                var $currFunnel = $(this),
                    funnelID = $currFunnel.data("id"),
                    newName = $currFunnel.find(".funnel-name").val();

                if (!funnelMap[funnelID]) {
                    funnelMap[funnelID] = {}
                }

                if (newName) {
                    funnelMap[funnelID]["name"] = newName;
                }

                funnelMap[funnelID]["order"] = index + 1;
            });

            $.ajax({
                type:"POST",
                url:countlyGlobal["path"]+"/funnel/edit",
                data:{
                    "app_id":countlyCommon.ACTIVE_APP_ID,
                    "funnel_map":JSON.stringify(funnelMap),
                    _csrf:countlyGlobal['csrf_token']
                },
                success:function (result) {
                    countlyFunnel.setFunnels(funnelMap);
                    $("#view-funnel").find(".funnel-name-title").text(countlyFunnel.getActiveFunnelName());
                    self.refreshFunnelList();
                }
            });
        });

        $(document).off("click", ".dialog .delete-funnel");
        $(document).on("click", ".dialog .delete-funnel", function() {
            var funnelId = $(this).data("id"),
                funnelName = $(this).parents("tr").find(".funnel-name").val();

            var dialog = $(this).parents(".dialog");

            if (funnelId) {
                CountlyHelpers.confirm(jQuery.i18n.prop('funnel.delete-confirm', funnelName), "red", function (result) {
                    if (result) {
                        $.ajax({
                            type:"POST",
                            url:countlyGlobal["path"]+"/funnel/delete",
                            data:{
                                "funnel_id":funnelId,
                                "app_id":countlyCommon.ACTIVE_APP_ID,
                                _csrf:countlyGlobal['csrf_token']
                            },
                            success:function (result) {
                                // Reset view only if we are deleting active funnel
                                if (funnelId == countlyFunnel.getActiveFunnel()) {
                                    countlyFunnel.reset();
                                    self.renderCommon(true);
                                } else {
                                    countlyFunnel.deleteFunnel(funnelId);
                                    self.refreshFunnelList();
                                }

                                $(".dialog #edit-funnels-table-container tr").each(function() {
                                    if($(this).find(".delete-event").data("id") == funnelId){
                                        $(this).remove();

                                        CountlyHelpers.changeDialogHeight(dialog);
                                    }
                                });
                            }
                        });
                    }
                });
            }
        });

        $(document).off("click", ".funnel-container");
        $(document).on("click", ".funnel-container", function(){
            if ($(this).hasClass("active")) {
                return;
            }

            $(".funnel-container").removeClass("active");
            $(this).addClass("active");

            self.resetFilter();

            countlyFunnel.setFilter(self.filterObj);
            countlyFunnel.setActiveFunnel($(this).data("id"));
            self.loadAndDraw();
        });

        app.localize();
    },
    initDrill: function() {
        var self = this;
        this.filterObj = {};
        $("#view-funnel").on("click", "#toggle-filter", function() {
            if ($(".filter-view-container").is(":visible")) {
                $("#filter-view").hide();
                $(".filter-view-container").hide();
            } else {
                $("#filter-view").show();
                $(".filter-view-container").show();
                adjustFilters();
            }
        });

        var usedFilters = {},
            currEvent = "";

        function getFilters() {
            usedFilters = {};

            $(".query:visible").each(function (index) {
                var filterType = $(this).find(".filter-name .text").data("type");

                // number and date types can be used multiple times for range queries
                if (filterType != "n" && filterType != "d") {
                    usedFilters[$(this).find(".filter-name .text").data("value")] = true;
                }
            });

            var filters = countlySegmentation.getFilters(currEvent),
                allFilters = "";

            if (filters.length == 0) {
                CountlyHelpers.alert(jQuery.i18n.map["drill.no-filters"], "black");
            }

            for (var i = 0; i < filters.length; i++) {
                if(typeof filters[i].id != "undefined"){
                    if (usedFilters[filters[i].id] == true) {
                        continue;
                    }
    
                    var tmpItem = $("<div>");
    
                    tmpItem.addClass("item");
                    tmpItem.attr("data-type", filters[i].type);
                    tmpItem.attr("data-value", filters[i].id);
                    tmpItem.text(filters[i].name);
    
                    allFilters += tmpItem.prop('outerHTML');
                }
                else{
                    var tmpItem = $("<div>");
    
                    tmpItem.addClass("group");
                    tmpItem.text(filters[i].name);
    
                    allFilters += tmpItem.prop('outerHTML');
                }
            }

            return allFilters;
        }

        function adjustFilters() {
            if (!$("#filter-view").is(":visible")) {
                return;
            }

            var queryCount = $("#filter-blocks").find(".query:visible").length;

            if (queryCount == 0) {
                $("#filter-add-container").trigger("click");
            }

            $(".query:visible").each(function (index) {
                $(this).find(".connector div").hide();

                if (queryCount > 1) {
                    $(this).find(".and-or").show();

                    if (index == 0) {
                        $(this).find(".and-or").hide();
                        $(this).find(".connector .middle-bottom").show();
                    } else if (index == (queryCount - 1)) {
                        $(this).find(".connector .middle-top").show();
                    } else {
                        $(this).find(".connector .top").show();
                        $(this).find(".connector .middle-bottom").show();
                    }

                    if (index == (queryCount - 1)) {
                        $(this).find(".filter-connector").removeClass("disabled");
                    } else {
                        $(this).find(".filter-connector").addClass("disabled");
                    }
                } else {
                    $(this).find(".and-or").hide();
                }

                if ($(this).find(".filter-connector .text").data("value") == "BY") {
                    $(this).find(".cly-select").removeClass("disabled");
                    $(this).find(".filter-connector").addClass("disabled");
                    $(this).prev(".query").find(".filter-connector").removeClass("disabled");
                    $(this).prev(".query").find(".filter-name").removeClass("disabled");

                    if ($(this).prev(".query").find(".filter-name .text").text() != "Select a Filter") {
                        $(this).prev(".query").find(".filter-type").removeClass("disabled");
                    }
                }
            });

            setTimeout(function() { $("#filter-blocks").removeClass("empty"); }, 1000);
        }

        function getFilterObjAndByVal() {
            var filterObj = {},
                filterObjTypes = {},
                byVal = "",
                byValText = "",
                bookmarkText = "";

            $(".query:visible").each(function (index) {
                var tmpConnector = $(this).find(".filter-connector .text").data("value"),
                tmpText = $(this).find(".filter-name .text").text(),
                tmpTypeText = $(this).find(".filter-type .text").text(),
                tmpValText = $(this).find(".filter-value.num input").val() ||
                    $(this).find(".filter-value.date input").val() ||
                    $(this).find(".filter-value.string input").val() ||
                    $(this).find(".filter-value .text").text(),
                tmpName = $(this).find(".filter-name .text").data("value"),
                tmpType = $(this).find(".filter-type .text").data("value"),
                tmpVal = $(this).find(".filter-value.num input").val() ||
                    $(this).find(".filter-value.date input").data("timestamp") ||
                    $(this).find(".filter-value.string input").val() ||
                    $(this).find(".filter-value .text").data("value"),
                tmpDataType = $(this).find(".filter-name .text").data("type");

                if(typeof tmpVal == "boolean")
                    tmpVal = tmpVal+"";
                if (tmpConnector != "BY") {
                    if (!tmpVal) {
                        return true;
                    }

                    if (tmpConnector && index != 0) {
                        bookmarkText += " [" + tmpConnector + "] ";
                    }

                    bookmarkText += tmpText + " " + tmpTypeText + " " + tmpValText;

                    if (!filterObj[tmpName]) {
                        filterObj[tmpName] = [];
                    }

                    if (!filterObjTypes[tmpName]) {
                        filterObjTypes[tmpName] = [];
                    }

                    if (tmpDataType == "d" || tmpDataType == "n") {
                        tmpVal = parseInt(tmpVal, 10);
                    }

                    var exp = {};

                    if (tmpType == "=" || tmpType == "!=") {
                        exp = tmpVal;
                    } else {
                        exp[tmpType] = tmpVal;
                    }

                    filterObj[tmpName].push(exp);
                    filterObjTypes[tmpName].push(tmpType);
                } else {
                    byVal = tmpName;
                    byValText = tmpText;
                }
            });

            var dbFilter = {};

            for (var filter in filterObj) {
                dbFilter[filter] = {};
                for (var i = 0; i < filterObj[filter].length; i++) {
                    if(_.isObject(filterObj[filter][i])) {
                        for (var tmpFilter in filterObj[filter][i]) {
                            dbFilter[filter][tmpFilter] = filterObj[filter][i][tmpFilter];
                        }
                    } else if (filterObjTypes[filter][i] == "!=") {
                        if (!dbFilter[filter]["$nin"]) {
                            dbFilter[filter]["$nin"] = [];
                        }

                        dbFilter[filter]["$nin"].push(filterObj[filter][i]);
                    } else {
                        if (!dbFilter[filter]["$in"]) {
                            dbFilter[filter]["$in"] = [];
                        }

                        dbFilter[filter]["$in"].push(filterObj[filter][i]);
                    }
                }
            }

            return {
                bookmarkText: bookmarkText,
                dbFilter: dbFilter,
                byVal: byVal,
                byValText: byValText
            };
        }

        function initApply() {
            var filterData = getFilterObjAndByVal();

            if (_.isEmpty(filterData.dbFilter) && filterData.byVal == "") {
                $("#apply-filter").addClass("disabled");
            } else {
                $("#apply-filter").removeClass("disabled");
            }
        }

        $("#segmentation").on("click", ".filter-connector", function() {
            var byUsed = false;
            $(".query:visible").each(function (index) {
                byUsed = byUsed || ($(this).find(".filter-connector .text").data("value") == "BY");
            });

            if (byUsed) {
                $(this).find(".item.by").hide();
            } else {
                $(this).find(".item.by").show();
            }
        });

        $("#segmentation").on("click", ".filter-name", function() {
            if ($(this).parents(".query").find(".filter-connector .text").data("value") == "OR") {
                var existingFilters = [],
                    orFilters = [],
                    orFilterStr = "";

                $(".query:visible:not(.by)").each(function (index) {
                    if ($(this).find(".filter-name .text").text() == jQuery.i18n.map["drill.select-filter"]) {
                        return false;
                    }

                    if (existingFilters.indexOf($(this).find(".filter-name .text").data("value")) == -1) {
                        orFilters.push({
                            type: $(this).find(".filter-name .text").data("type"),
                            id: $(this).find(".filter-name .text").data("value"),
                            name: $(this).find(".filter-name .text").text()
                        });

                        existingFilters.push($(this).find(".filter-name .text").data("value"));
                    }
                });

                for (var i = 0; i < orFilters.length; i++) {
                    var tmpItem = $("<div>");

                    tmpItem.addClass("item");
                    tmpItem.attr("data-type", orFilters[i].type);
                    tmpItem.attr("data-value", orFilters[i].id);
                    tmpItem.text(orFilters[i].name);

                    orFilterStr += tmpItem.prop('outerHTML');
                }

                $(this).find(".select-items>div").html(orFilterStr);
            } else {
                $(this).find(".select-items>div").html(getFilters());
            }

            initApply();
        });

        $("#segmentation").on("click", ".filter-name .item", function() {
            if ($(this).parents(".query").find(".filter-connector .text").data("value") != "BY") {
                $(this).parents(".filter-name").siblings(".filter-type").replaceWith($("#defalt-filter-block").clone(true).find(".filter-type"));
                $(this).parents(".filter-name").siblings(".filter-type").removeClass("disabled");

                $(this).parents(".filter-name").siblings(".filter-value").addClass("hidden");

                if ($(this).data("type") == "l") {
                    $(this).parents(".filter-name").siblings(".filter-value.list").removeClass("hidden");
                    $(this).parents(".filter-name").siblings(".filter-value.list").removeClass("big-list");
                    $(this).parents(".filter-name").siblings(".filter-value").find(".text").text("");
                    $(this).parents(".filter-name").siblings(".filter-value").find(".text").data("value","");
                    $(this).parents(".filter-name").siblings(".filter-value").removeClass("disabled");
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.num").hide();
                } else if ($(this).data("type") == "bl") {
                    $(this).parents(".filter-name").siblings(".filter-value.list").removeClass("hidden");
                    $(this).parents(".filter-name").siblings(".filter-value.list").addClass("big-list");
                    $(this).parents(".filter-name").siblings(".filter-value").find(".text").text("");
                    $(this).parents(".filter-name").siblings(".filter-value").find(".text").data("value","");
                    $(this).parents(".filter-name").siblings(".filter-value").removeClass("disabled");
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.num").hide();
                } else if ($(this).data("type") == "n") {
                    $(this).parents(".filter-name").siblings(".filter-value.num").removeClass("hidden");
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.num").show();
                }  else if ($(this).data("type") == "s") {
                    $(this).parents(".filter-name").siblings(".filter-value.string").removeClass("hidden");
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.num").hide();
                }  else if ($(this).data("type") == "d") {
                    $(this).parents(".filter-name").siblings(".filter-value.date").removeClass("hidden");
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.eq").hide();
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.num").hide();
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.lt").show();
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.lte").hide();
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.gte").show();
                    $(this).parents(".filter-name").siblings(".filter-type").find(".item.gte").trigger("click");
                    $(this).parents(".filter-name").siblings(".filter-value.date").find(".open-date-picker").trigger("click");
                }
            }
            
            $(this).parents(".query").find(".filter-value.num input").val("");
            $(this).parents(".query").find(".filter-value.date input").val("");
            $(this).parents(".query").find(".filter-value.date input").data("timestamp", null)
            $(this).parents(".query").find(".filter-value.string input").val("");
            
            var self = $(this).parents(".query").find(".filter-value .select-items>div");
            self.html("<div></div>");
            function setUpFilterValues(filterValues, filterNames, biglist){
                var filterValStr = "";
                
                if(biglist){
                    var tmpItem = $("<div>");
                    tmpItem.addClass("warning");
                    tmpItem.text(jQuery.i18n.map["drill.big-list-warning"]);       
                    filterValStr += tmpItem.prop('outerHTML');
                }
    
                if (jQuery.isArray(filterValues)) {
                    for (var i = 0; i < filterValues.length; i++) {
                        var tmpItem = $("<div>");
        
                        tmpItem.addClass("item");
                        tmpItem.attr("data-value", filterValues[i]);
                        tmpItem.text(filterNames[i]);
        
                        filterValStr += tmpItem.prop('outerHTML');
                    }
                } else {
                    for (var p in filterValues) {
                        var tmpItem = $("<div>");
                        tmpItem.addClass("group");
                        tmpItem.text(p);
            
                        filterValStr += tmpItem.prop('outerHTML');
                        for (var i = 0; i < filterValues[p].length; i++) {
                            var tmpItem = $("<div>");
            
                            tmpItem.addClass("item");
                            tmpItem.attr("data-value", filterValues[p][i]);
                            tmpItem.text(countlyCommon.decode(filterValues[p][i]));
            
                            filterValStr += tmpItem.prop('outerHTML');
                        }
                    }
                }
    
                self.html(filterValStr);
            }
            
            if($(this).data("type") == "bl"){
                var prop = $(this).data("value");
                $(this).parents(".query").find(".filter-value").addClass("loading");

                countlySegmentation.getBigListMetaData(prop, null, function(values, names){
                    self.parents(".query").find(".filter-value").removeClass("loading");
                    setUpFilterValues(values, names,  names.length >= countlyGlobal["list_limit"]);
                    var timeout = null;
                    self.parents(".query").find(".filter-value").on("keyup", ".search input", function(e) {
                        e.stopPropagation();
                        var search = $(this).val();
                        var parent = $(this).parents(".filter-value").find(".select-items");
                        if(!parent.find(".table-loader").length)
                            parent.prepend("<div class='table-loader'></div>");
                        if(timeout)
                        {
                            clearTimeout(timeout);
                            timeout = null;
                        }
                        timeout = setTimeout(function(){
                            countlySegmentation.getBigListMetaData(prop, search, function(values, names){
                                parent.find(".table-loader").remove();
                                setUpFilterValues(values, names, names.length >= countlyGlobal["list_limit"]);
                            });                                    
                        }, 1000);
                    });
                });
            }
            else{
                setUpFilterValues(countlySegmentation.getFilterValues($(this).data("value")), countlySegmentation.getFilterNames($(this).data("value")));
            }

            var selectedItem = $(this).parents(".cly-select").find(".text");
            selectedItem.text($(this).text());
            selectedItem.data("value", $(this).data("value"));
            selectedItem.data("type", $(this).data("type"));
        });

        $("#segmentation").on("click", ".filter-value .item", function() {
            setTimeout(function() { initApply(); }, 0);
        });

        $("#segmentation").on("keyup", ".filter-value input",function(event) {
            initApply();
        });

        $("#segmentation").on("keydown", ".filter-value.num input",function(event) {
            // Allow: backspace, delete, tab, escape, and enter
            if ( event.keyCode == 46 || event.keyCode == 8 || event.keyCode == 9 || event.keyCode == 27 || event.keyCode == 13 ||
                // Allow: Ctrl+A
                (event.keyCode == 65 && event.ctrlKey === true) ||
                // Allow: home, end, left, right
                (event.keyCode >= 35 && event.keyCode <= 39)) {
                // let it happen, don't do anything
                return;
            }
            else {
                // Ensure that it is a number and stop the keypress
                if (event.shiftKey || (event.keyCode < 48 || event.keyCode > 57) && (event.keyCode < 96 || event.keyCode > 105 )) {
                    event.preventDefault();
                }
            }
        });

        $("#segmentation").on("keydown", ".filter-value.date input",function(event) {
            event.preventDefault();
        });

        $("#segmentation").on("click", ".filter-connector .item", function() {
            $(this).parents(".query").find(".filters").replaceWith($("#defalt-filter-block").clone(true).find(".filters"));

            $(".query:visible:not(:last)").find(".filter-name").addClass("disabled");
            $(".query:visible:not(:last)").find(".filter-type").addClass("disabled");
            adjustFilters();

            $(this).parents(".query").removeClass("by");

            if ($(this).data("value") == "BY") {
                $(this).parents(".query").addClass("by");
                $(this).parents(".query").find(".filter-type").hide();
                $(this).parents(".query").find(".filter-value").hide();
                $(this).parents(".query").find(".filter-connector").addClass("disabled");
            } else if ($(this).data("value") == "AND") {

            } else if ($(this).data("value") == "OR") {

            }

            initApply();
        });

        $("#segmentation").on("click", ".query .delete", function() {
            $(this).parent(".query").remove();
            $(".query:visible:last-child").find(".filter-name").removeClass("disabled");
            $(".query:visible:last-child").find(".filter-type").removeClass("disabled");
            adjustFilters();
            initApply();
            var filterData = getFilterObjAndByVal();

            self.filterObj = filterData.dbFilter;
            countlyFunnel.setFilter(self.filterObj);

            if (_.isEmpty(filterData.dbFilter)) {
                $("#no-filter").show();
                $("#current-filter").hide().find(".text").text("");
            } else {
                $("#no-filter").hide();
                $("#current-filter").show().find(".text").text(filterData.bookmarkText);
            }

            self.loadAndRefresh();
        });

        $("#segmentation").on("click", ".open-date-picker", function(e) {
            $(this).next(".date-picker").show();
            $(this).next(".date-picker").find(".calendar").datepicker({
                numberOfMonths:1,
                showOtherMonths:true,
                maxDate:moment().subtract('days', 1).toDate(),
                onSelect:function (selectedDate) {
                    var instance = $(this).data("datepicker"),
                        date = $.datepicker.parseDate(instance.settings.dateFormat || $.datepicker._defaults.dateFormat, selectedDate, instance.settings),
                        currMoment = moment(date);

                    $(this).parents(".date-picker").prev(".open-date-picker").val(countlyCommon.formatDate(moment(date),"DD MMMM, YYYY"));
                    $(this).parents(".date-picker").prev(".open-date-picker").data("timestamp", moment(currMoment.format("DD MMMM, YYYY"), "DD MMMM, YYYY").unix());
                    $(".date-picker").hide();
                    initApply();
                }
            });

            $.datepicker.setDefaults($.datepicker.regional[""]);
            $(this).next(".date-picker").find(".calendar").datepicker("option", $.datepicker.regional[countlyCommon.BROWSER_LANG]);

            $(this).next(".date-picker").click(function(e) {
                e.stopPropagation();
            });

            e.stopPropagation();
        });

        $("#filter-add-container").on("click", function() {
            if ($(".query:visible:not(.by):last").find(".filter-name .text").text() == jQuery.i18n.map["drill.select-filter"]) {
                $(".query:visible:not(.by):last").find(".filter-name").addClass("req");
                return false;
            }

            var filterBlocks = $("#filter-blocks"),
                defaultFilters = $("#defalt-filter-block").html();

            filterBlocks.append(defaultFilters);
            $(".query:visible:not(:last)").find(".filter-name").addClass("disabled");
            $(".query:visible:not(:last)").find(".filter-type").addClass("disabled");

            var byFilter = $("#filter-blocks").find(".query.by").clone(true);
            $("#filter-blocks").find(".query.by").remove();
            $("#filter-blocks").append(byFilter);

            adjustFilters();
        });

        $("#apply-filter").on("click", function() {
            if ($(this).hasClass("disabled")) {
                return true;
            }

            var filterData = getFilterObjAndByVal();
            var needData = false;
            if (!_.isEqual(self.filterObj, filterData.dbFilter)) {
                // If the query is not changed we don't need to get data
                needData = true;
            }

            self.filterObj = filterData.dbFilter;

            /*
            self.byVal = filterData.byVal;
            self.activeSegmentForTable = "";
            self.draw(needData, false);
            */

            if (needData) {
                countlyFunnel.setFilter(self.filterObj);
                self.loadAndRefresh();
            }

            $("#no-filter").hide();
            $("#current-filter").show().find(".text").text(filterData.bookmarkText);
        });

        $("#remove-filter").on("click", function(e) {
            self.filterObj = {};

            var wasFilterVisible = $("#filter-view").is(":visible");

            $("#filter-view").replaceWith(self.filterBlockClone.clone(true));
            if (wasFilterVisible) {
                $(".filter-view-container").show();
                $("#filter-view").show();
                adjustFilters();
            }

            $("#current-filter").hide();
            $("#no-filter").show();

            countlyFunnel.setFilter(self.filterObj);
            self.loadAndRefresh();

            e.stopPropagation();
        });

        $(window).click(function() {
            $(".date-picker").hide();
        });
    },
    renderCommon:function (isRefresh) {
        var self = this;
        
        if (countlyEvent.getEvents().length == 0) {
            CountlyHelpers.alert(jQuery.i18n.map["funnels.no-events"], "black");
        }

        if (!isRefresh) {

            if (countlyFunnel.getFunnels().length == 0 && !countlyGlobal['admin_apps'][countlyCommon.ACTIVE_APP_ID]) {
                window.location = "dashboard#/";
                CountlyHelpers.alert(jQuery.i18n.map["funnels.no-funnel"], "black");
                return true;
            }

            this.templateData = {
                "page-title":countlyFunnel.getActiveFunnelName(),
                "funnels": countlyFunnel.getFunnels()
            };

            $(this.el).html(this.template(this.templateData));

            this.initFunnelView();
            this.initNewFunnelView();
            this.pageScript();
            this.initDrill();

            setTimeout(function() {
                self.filterBlockClone = $("#filter-view").clone(true);
            }, 0);

            $('#funnel-configure-button').tooltipster({
                animation: 'fade',
                animationDuration: 100,
                delay: 100,
                theme: 'tooltipster-borderless',
                side: "right",
                distance: 10,
                trigger: 'custom',
                triggerOpen: {
                    mouseenter: true,
                    touchstart: true
                },
                triggerClose: {
                    mouseleave: true,
                    touchleave: true
                },
                interactive: true,
                contentAsHTML: true,
                content: $("#funnel-configure-options")
            });

            $("#edit-funnels").on("click", function() {
                CountlyHelpers.popup("#edit-funnels-container", "funnels");

                $(".dialog #edit-funnels-table-container").slimScroll({
                    height:'100%',
                    start:'top',
                    wheelStep:10,
                    position:'right',
                    disableFadeOut:true
                });

                $(".dialog .events-table").sortable({
                    items:"tbody tr",
                    revert:true,
                    handle:"td:first-child",
                    helper:function (e, elem) {
                        elem.children().each(function () {
                            $(this).width($(this).width());
                        });

                        elem.addClass("moving");

                        return elem;
                    },
                    cursor:"move",
                    containment:"parent",
                    tolerance:"pointer",
                    placeholder:"event-row-placeholder",
                    stop:function (e, elem) {
                        elem.item.removeClass("moving");
                    }
                });
            });

            if (countlyGlobal['admin_apps'][countlyCommon.ACTIVE_APP_ID]) {
                $("#funnel-configure-button").show();
            }

        } else {
            $.when(this.beforeRender()).then(function () {
                self.initFunnelView();
                self.initNewFunnelView();
                self.refreshFunnelList();
            });
        }
    },
    dateChanged:function() {
        this.loadAndRefresh();
    },
    refresh:function () {},
    refreshFunnelList: function() {
        var self = this;

        this.templateData.funnels = countlyFunnel.getFunnels();

        var newPage = $("<div>" + self.template(self.templateData) + "</div>");

        $(self.el).find("#event-nav .scrollable").html(function () {
            return newPage.find("#event-nav .scrollable").html();
        });

        $(self.el).find("#edit-funnels-container").replaceWith(newPage.find("#edit-funnels-container"));

        app.localize();
    }
});

//register views
app.funnelView = new FunnelView();

app.route("/funnels", "funnels", function () {
	this.renderWhenReady(this.funnelView);
});

$( document ).ready(function() {
	var menu = '<a href="#/funnels" class="item analytics" id="sidebar-funnels">'+
        '<div class="logo ion-funnel"></div>'+
        '<div class="text" data-localize="sidebar.funnels"></div>'+
    '</a>';
	if($('.sidebar-menu:not(#iot-type) #management-menu').length)
		$('.sidebar-menu:not(#iot-type) #management-menu').before(menu);
	else
		$('.sidebar-menu:not(#iot-type)').append(menu);
    //check if configuration view exists
    if(app.configurationsView){
        app.configurationsView.registerLabel("funnels", "sidebar.funnels");
    }
});