var request = require('supertest');
var should = require('should');
var crypto = require('crypto');
var testUtils = require("../../test/testUtils");
request = request.agent(testUtils.url);

var APP_KEY = "";
var API_KEY_ADMIN = "";
var APP_ID = "";
var DEVICE_ID = "1234567890";
var steps = [];
steps.push("Step1");
steps.push("Step2");
steps.push("Step3");
var FUNNEL_ID = "";

describe('Testing Funnels', function(){
	//{"data":{},"meta":[],"lu":"2015-01-20T12:00:06.176Z"}
	describe('Empty get_funnels', function(){
		it('should success', function(done){
			API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=get_funnels")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.eql([]);
				setTimeout(done, 1000)
			});
		});
	});
	describe('Creating funnel', function(){
		before(function( done ){
			testUtils.login( request );
			testUtils.waitLogin( done );
		});
		it('should create funnel', function(done){
			request
			.post("/funnel/add")
			.send({app_id:APP_ID, funnel_name:"Test", steps:steps, _csrf:testUtils.getCSRF()})
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				setTimeout(done, 1000)
			});
		});
	});
	//[{"_id":"98defb36e40006e1895b4d35597459f9","name":"Test","steps":["Step1","Step2","Step3"]}]
	describe('Verify funnel', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=get_funnels")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text)[0];
				ob.should.have.property("_id");
				ob.should.have.property("name", "Test");
				ob.should.have.property("steps", steps);
                FUNNEL_ID = ob._id;	
                setTimeout(done, 1000)
			});
		});
	});
	//{"steps":[{"step":"Step1","users":0,"times":0,"percent":0},{"step":"Step2","users":0,"times":0,"percent":0},{"step":"Step3","users":0,"times":0,"percent":0}],"total_users":0,"success_users":0,"success_rate":0}
	describe('Verify funnel users', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=funnel&funnel="+FUNNEL_ID+"&period=month")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("total_users", 0);
				ob.should.have.property("success_users", 0);
				ob.should.have.property("success_rate", 0);
				ob.should.have.property("steps", [{"step":"Step1","users":0,"times":0,"percent":0},{"step":"Step2","users":0,"times":0,"percent":0},{"step":"Step3","users":0,"times":0,"percent":0}]);
				setTimeout(done, 1000)
			});
		});
	});
	describe('Creating user 1', function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+'&begin_session=1')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 1000)
			});
		});
	});
	describe('Creating user 2', function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+DEVICE_ID+'a&app_key='+APP_KEY+'&begin_session=1')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 1000)
			});
		});
	});
	describe('Step2 event', function(){
		it('should success', function(done){
			var params = [{
				"key": "Step2",
				"count": 1
			}];
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&events="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify funnel users', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=funnel&funnel="+FUNNEL_ID+"&period=month")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("total_users", 0);
				ob.should.have.property("success_users", 0);
				ob.should.have.property("success_rate", 0);
				ob.should.have.property("steps", [{"step":"Step1","users":0,"times":0,"percent":0},{"step":"Step2","users":0,"times":0,"percent":0},{"step":"Step3","users":0,"times":0,"percent":0}]);
				setTimeout(done, 1000)
			});
		});
	});
	describe('Step1 event', function(){
		it('should success', function(done){
			var params = [{
				"key": "Step1",
				"count": 1
			}];
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&events="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify funnel users', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=funnel&funnel="+FUNNEL_ID+"&period=month")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("total_users", 1);
				ob.should.have.property("success_users", 0);
				ob.should.have.property("success_rate", 0);
				ob.should.have.property("steps", [{"step":"Step1","users":1,"times":1,"percent":100},{"step":"Step2","users":0,"times":0,"percent":0},{"step":"Step3","users":0,"times":0,"percent":0}]);
				setTimeout(done, 1000)
			});
		});
	});
	describe('Step3 event', function(){
		it('should success', function(done){
			var params = [{
				"key": "Step3",
				"count": 1
			}];
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&events="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify funnel users', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=funnel&funnel="+FUNNEL_ID+"&period=month")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("total_users", 1);
				ob.should.have.property("success_users", 0);
				ob.should.have.property("success_rate", 0);
				ob.should.have.property("steps", [{"step":"Step1","users":1,"times":1,"percent":100},{"step":"Step2","users":0,"times":0,"percent":0},{"step":"Step3","users":0,"times":0,"percent":0}]);
				setTimeout(done, 1000)
			});
		});
	});
	describe('Step2 event', function(){
		it('should success', function(done){
			var params = [{
				"key": "Step2",
				"count": 1
			}];
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&events="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify funnel users', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=funnel&funnel="+FUNNEL_ID+"&period=month")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("total_users", 1);
				ob.should.have.property("success_users", 0);
				ob.should.have.property("success_rate", 0);
				ob.should.have.property("steps", [{"step":"Step1","users":1,"times":1,"percent":100},{"step":"Step2","users":1,"times":1,"percent":100},{"step":"Step3","users":0,"times":0,"percent":0}]);
				setTimeout(done, 5000)
			});
		});
	});
	describe('Step1 event for other user', function(){
		it('should success', function(done){
			var params = [{
				"key": "Step1",
				"count": 1
			}];
			request
			.get('/i?device_id='+DEVICE_ID+'a&app_key='+APP_KEY+"&events="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify funnel users', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=funnel&funnel="+FUNNEL_ID+"&period=month")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("total_users", 2);
				ob.should.have.property("success_users", 0);
				ob.should.have.property("success_rate", 0);
				ob.should.have.property("steps", [{"step":"Step1","users":2,"times":2,"percent":100},{"step":"Step2","users":1,"times":1,"percent":50},{"step":"Step3","users":0,"times":0,"percent":0}]);
				setTimeout(done, 1000)
			});
		});
	});
	describe('Step3 event', function(){
		it('should success', function(done){
			var params = [{
				"key": "Step3",
				"count": 1
			}];
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&events="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify funnel users', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=funnel&funnel="+FUNNEL_ID+"&period=month")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property("total_users", 2);
				ob.should.have.property("success_users", 1);
				ob.should.have.property("success_rate", 50);
				ob.should.have.property("steps", [{"step":"Step1","users":2,"times":2,"percent":100},{"step":"Step2","users":1,"times":1,"percent":50},{"step":"Step3","users":1,"times":1,"percent":50}]);
				setTimeout(done, 1000)
			});
		});
	});
	describe('Reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID, period:"reset"};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Empty get_funnels', function(){
		it('should success', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+"&app_id="+APP_ID+"&method=get_funnels")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.eql([]);
				setTimeout(done, 100)
			});
		});
	});
    describe('Login out', function(){
		it('should redirect to login', function(done){
			request
			.get('/logout')
			.expect('location', '/login')
			.expect(302);
            testUtils.logout();
            setTimeout(done, 5000);
		})
	});
});