var plugin = {},
	common = require('../../../api/utils/common.js'),
	live = require('./parts/data/live.js'),
    plugins = require('../../pluginManager.js');

(function (plugin) {
	plugins.register("/worker", function(ob){
		common.dbLiveMap = {
			'online_users': 'ou',
			'new_users': 'nu',
			'last_active': 'la',
			'last_update': 'lu',
			'is_new': 'n'
		};
	});
	//write api call
	plugins.register("/i", function(ob){
		var params = ob.params;
		if (!params.qstring.begin_session) {
            live.extendUserSession(params);
			return true;
		}
	});
	
	//manipulating crashes
	plugins.register("/o", function(ob){
		var params = ob.params;
		var validateUserForDataReadAPI = ob.validateUserForDataReadAPI;
		switch (params.qstring.method) {
            case 'live':
                validateUserForDataReadAPI(params, live.getOnlineUsers);
				return true;
                break;
            case 'live_graph':
                validateUserForDataReadAPI(params, live.getOnlineUsersGraph);
				return true;
                break;
        }
	});
	
	plugins.register("/session/end", function(ob){
		var params = ob.params;
		var dbAppUser = ob.dbAppUser;
		live.endUserSession((dbAppUser[common.dbUserMap['session_count']] == 1), params);
	});
	
	plugins.register("/session/extend", function(ob){
		var params = ob.params;
		live.extendUserSession(params);
	});
	
	plugins.register("/session/begin", function(ob){
		var params = ob.params;
		var isNewUser = ob.isNewUser;
		live.beginUserSession(isNewUser, params);
	});
	
	plugins.register("/i/apps/create", function(ob){
		var params = ob.params;
		var appId = ob.appId;
        common.db.command({"convertToCapped": 'live_data' + appId, size: 100000, max: 60}, function(err,data){
            if(err){
                common.db.createCollection('live_data' + appId, {capped: true, size: 100000, max: 60}, function(err,data){});
            }
        });
		common.db.collection('max_online_counts').insert({'_id': appId, nu: 0, ou: 0}, function(err, res) {});
		common.db.collection('max_online_counts').ensureIndex({"nu": 1}, function() {});
		common.db.collection('max_online_counts').ensureIndex({"ou": 1}, function() {});
		common.db.collection('online_users').ensureIndex({la: 1}, {expireAfterSeconds: 180}, function () {});
	});
	
	plugins.register("/i/apps/delete", function(ob){
		var appId = ob.appId;
		common.db.collection('live_data' + appId).drop(function(){});
        common.db.collection('max_online_counts').remove({'_id': common.db.ObjectID(appId)}, function(err, res) {});
        common.db.collection('online_users').remove({'_id': {$regex: "^" + appId + "_.*"}}, function(err, res) {});
	});
	
	plugins.register("/i/apps/reset", function(ob){
		var appId = ob.appId;
		common.db.collection('live_data' + appId).drop(function(){
            common.db.createCollection('live_data' + appId, {capped: true, size: 100000, max: 60}, function(err,data){});
        });
		common.db.collection('max_online_counts').update({'_id': common.db.ObjectID(appId)}, {$set: {nu: 0, ou: 0}},function(){});
		common.db.collection('online_users').remove({'_id': {$regex: "^" + appId + "_.*"}}, function(err, res) {});
	});
    plugins.register("/i/apps/clear_all", function(ob){
        var appId = ob.appId;
		common.db.collection('live_data' + appId).drop(function(){
            common.db.createCollection('live_data' + appId, {capped: true, size: 100000, max: 60}, function(err,data){});
        });
		common.db.collection('max_online_counts').update({'_id': common.db.ObjectID(appId)}, {$set: {nu: 0, ou: 0}},function(){});
		common.db.collection('online_users').remove({'_id': {$regex: "^" + appId + "_.*"}}, function(err, res) {});
    })
}(plugin));

module.exports = plugin;