var liveApi = {},
    common = require('../../../../../api/utils/common.js');

(function (liveApi) {

    liveApi.beginUserSession = function (isNewUser, params) {
        if (!isOnlineLive(params)) {
            return true;
        }

        var onlineUsers = {},
            deviceHash = common.crypto.createHash('md5').update(params.qstring.device_id + "").digest('hex');

        onlineUsers[common.dbLiveMap.last_active] = params.time.nowUTC.toDate();
        onlineUsers[common.dbLiveMap.is_new] = (isNewUser) ? 1 : 0;

        common.db.collection('online_users').update({ "_id": params.app_id + "_" + deviceHash }, { '$set': onlineUsers }, { 'upsert': true }, function(err, res) {});
        common.db.collection('live_data' + params.app_id).update({_id:getLiveDataId()}, {$inc:{ou: 1, nu: (isNewUser) ? 1 : 0}}, {upsert:true}, function(err, result) {});

        setTimeout(function() {
            common.db.collection('online_users').aggregate([
                    {
                        $match: { _id: {$regex: "^" + params.app_id + "_.*"}}
                    },
                    {
                        $group: { _id: '$n', total: {$sum: 1}}
                    }
                ],
                function (error, result) {
                    var onlineNewCount = 0,
                        onlineCount = 0;

                    if (result && result.length != 0) {
                        for (var i = 0; i < result.length; i++) {
                            if (result[i]._id == 1) {
                                onlineNewCount += result[i].total;
                            }

                            onlineCount += result[i].total;
                        }
                    }

                    if (onlineCount > 0) {
                        common.db.collection('max_online_counts').update({"_id": params.app_id, "ou": {$lt: onlineCount}}, {'$set': {"ou": onlineCount}}, {'upsert': false},function(){});
                    }

                    if (onlineNewCount > 0) {
                        common.db.collection('max_online_counts').update({"_id": params.app_id, "nu": {$lt: onlineNewCount}}, {'$set': {"nu": onlineNewCount}}, {'upsert': false},function(){});
                    }
                }
            );
        }, 1000);
    };

    liveApi.extendUserSession = function (params) {
        if (!isOnlineLive(params)) {
            return true;
        }

        var deviceHash = common.crypto.createHash('md5').update(params.qstring.device_id + "").digest('hex');
        common.db.collection('online_users').update({"_id": params.app_id + "_" + deviceHash}, {'$set': {'la': params.time.nowUTC.toDate()}}, {'upsert': false},function(){});
    };

    liveApi.endUserSession = function (isNewUser, params) {
        if (!isOnlineLive(params)) {
            return true;
        }

        var deviceHash = common.crypto.createHash('md5').update(params.qstring.device_id + "").digest('hex');
        common.db.collection('online_users').remove({'_id': params.app_id + "_" + deviceHash},function(){});
    };

    liveApi.getOnlineUsers = function (params) {
        if (!isOnlineLive(params)) {
            common.returnMessage(params, 503, 'Online users API is paused for now. Please try again later...');
            return true;
        }

        common.db.collection('online_users').aggregate([
                {
                    $match: { _id: {$regex: "^" + params.app_id + "_.*"}}
                },
                {
                    $group: { _id: '$n', total: {$sum: 1}}
                }
            ],
            function (error, result) {
                var onlineNewCount = 0,
                    onlineCount = 0;

                if (result && result.length != 0) {
                    for (var i = 0; i < result.length; i++) {
                        if (result[i]._id == 1) {
                            onlineNewCount += result[i].total;
                        }

                        onlineCount += result[i].total;
                    }
                }

                common.db.collection('max_online_counts').findOne({"_id": params.app_id}, function (err, maxCounts) {
                    var maxOnlineCount = 0,
                        maxNewCount = 0;

                    if (maxCounts) {
                        maxOnlineCount = maxCounts[common.dbLiveMap.online_users] || 0;
                        maxNewCount = maxCounts[common.dbLiveMap.new_users] || 0;
                    }

                    if (onlineCount > maxOnlineCount) {
                        maxOnlineCount = onlineCount;
                    }

                    if (onlineNewCount > maxNewCount) {
                        maxNewCount = onlineNewCount;
                    }

                    common.returnOutput(params, {'o': onlineCount, 'n': onlineNewCount, 'mo': maxOnlineCount, 'mn': maxNewCount});
                });
            }
        );
    };

    liveApi.getOnlineUsersGraph = function (params) {
        if (!isOnlineLive(params)) {
            common.returnMessage(params, 503, 'Online users API is paused for now. Please try again later...');
            return true;
        }

        var liveDataArr = [],
            liveDataObj = {};

        common.db.collection('live_data' + params.app_id).find({}).toArray(function (err, liveData) {
            liveData = liveData || [];
            var currTime = common.moment();

            for (var i = 0; i < 60; i++) {
                liveDataObj[getLiveDataId(currTime.subtract("s", i ? 10 : 0).unix())] = {'o': 0, 'n': 0};
            }

            for (var j = 0; j < liveData.length; j++) {
                if (!liveData[j]) {
                    continue;
                }

                if (liveDataObj[liveData[j]._id]) {
                    liveDataObj[liveData[j]._id] = {
                        'o': liveData[j][common.dbLiveMap.online_users] || 0,
                        'n': liveData[j][common.dbLiveMap.new_users] || 0
                    };
                }
            }

            var liveDataObjKeys = Object.keys(liveDataObj);
            liveDataObjKeys = liveDataObjKeys.sort(function(a,b){return parseInt(a) - parseInt(b)});

            for (var k = 0; k < liveDataObjKeys.length; k++) {
                liveDataArr[k] = liveDataObj[liveDataObjKeys[k]];
            }

            common.returnOutput(params, liveDataArr);
        });
    };

    function getLiveDataId(currTimestamp) {
        var currTime = currTimestamp ? common.moment(currTimestamp * 1000) : common.moment();
        return currTime.format("YYYYMMDDHHmm") + common.zeroFill((Math.floor(currTime.format("s") / 10) * 10), 2);
    }

    function getLiveDataTime(liveDataId) {
        return common.moment(liveDataId, "YYYYMMDDHHmmss");
    }

    function isOnlineLive(params) {
        return true;
    }

}(liveApi));

module.exports = liveApi;