window.LiveView = {
    initialize: function() {
        // add placeholder to the top in order to prevent jumps
        $("#content-top").html("<div style='opacity: 0.5; height:122px;' id='realtime-container'><div class='data'><div class='gauge'></div></div></div>");

		var self = this;
		if(this.template)
			return $.when(countlyLive.initialize()).then(function () {self.renderCommon();});
		else{
			return $.when($.get(countlyGlobal["path"]+'/live/templates/realtimebar.html', function(src){
				self.template = Handlebars.compile(src);
			}), countlyLive.initialize()).then(function () {self.renderCommon();});
		}
    },
	realtimeGraph:{},
    renderCommon:function () {
        if(Backbone.history.fragment == "/"){
            var self = this;
            $("#content-top").html(this.template());
            app.localize();
            var onlineUserCounts = countlyLive.getOnlineUsers(),
                opts = {
                    lines:12,
                    angle:0.5,
                    lineWidth:0.08,
                    pointer:{
                        length:0.7,
                        strokeWidth:0.05,
                        color:'#333333'
                    },
                    colorStart:'#6FADCF',
                    colorStop:'#8FC0DA',
                    strokeColor:'#E0E0E0',
                    generateGradient:true
                };
    
            opts.colorStart = "#52A3EF";
            opts.colorStop = "#52A3EF";
    
            self.onlineUsersGauge = new Donut($('#online-users-gauge')[0]).setOptions(opts);
            self.onlineUsersGauge.setTextField($('#online-users-text')[0]);
            self.onlineUsersGauge.maxValue = onlineUserCounts.mo;
            self.onlineUsersGauge.set(1);
            self.onlineUsersGauge.set(onlineUserCounts.o);
    
            opts.colorStart = "#FF8700";
            opts.colorStop = "#FF8700";
    
            self.newUsersGauge = new Donut($('#new-users-gauge')[0]).setOptions(opts);
            self.newUsersGauge.setTextField($('#new-users-text')[0]);
            self.newUsersGauge.maxValue = onlineUserCounts.mn;
            self.newUsersGauge.set(1);
            self.newUsersGauge.set(onlineUserCounts.n);
    
            $("#max-online-users-text").text(onlineUserCounts.mo);
            $("#max-new-users-text").text(onlineUserCounts.mn);
    
            self.realtimeGraph.counter = Math.floor((new Date()).getTime() / 1000) - (60 * (countlyCommon.DASHBOARD_REFRESH_MS / 1000));
            self.realtimeGraph.series = [
                {
                    data:[],
                    color:'#52A3EF'
                },
                {
                    data:[],
                    color:'#FF8700'
                }
            ];
    
            liveGraphData = countlyLive.getOnlineUsersGraphData();
    
            for (var i = 0; i < liveGraphData.length; i++) {
                self.realtimeGraph.series[0].data.push({x:self.realtimeGraph.counter, y:liveGraphData[i].o});
                self.realtimeGraph.series[1].data.push({x:self.realtimeGraph.counter, y:liveGraphData[i].n});
                self.realtimeGraph.counter = self.realtimeGraph.counter + (countlyCommon.DASHBOARD_REFRESH_MS / 1000);
            }
    
            self.realtimeGraph.graph = new Rickshaw.Graph({
                element:document.querySelector("#realtime-chart"),
                renderer:'bar',
                series:self.realtimeGraph.series,
                interpolation:'linear'
            });
    
            self.realtimeGraph.graph.render();
    
            var Hover = Rickshaw.Class.create(Rickshaw.Graph.HoverDetail, {
                render: function(args) {
                    $("#realtime-graph-counts").html("");
    
                    args.detail.sort(function(a, b) { return a.order - b.order }).forEach( function(d) {
                        var swatch = document.createElement('div');
                        swatch.className = 'swatch';
                        swatch.style.backgroundColor = d.series.color;
    
                        var label = document.createElement('div');
                        label.className = 'label';
                        label.innerHTML = parseInt(d.formattedYValue).toFixed(0);
    
                        $("#realtime-graph-counts").append(swatch);
                        $("#realtime-graph-counts").append(label);
    
                        this.show();
    
                    }, this );
                },
                hide: function() {
                    $("#realtime-graph-counts").html("");
                }
            });
    
            var hoverDetail = new Hover({
                graph: self.realtimeGraph.graph
            });
    
            var xAxis = new Rickshaw.Graph.Axis.Time({
                graph:self.realtimeGraph.graph,
                timeUnit:{
                    name:'1 minute',
                    seconds:60,
                    formatter:function (d) {
                        return moment(d).format("HH:mm");
                    }
                }
            });
    
            xAxis.render();
    
            var yAxis = new Rickshaw.Graph.Axis.Y({
                graph:self.realtimeGraph.graph,
                orientation:'right',
                tickFormat:Rickshaw.Fixtures.Number.formatKMBT,
                pixelsPerTick:24
            });
    
            yAxis.render();
    
            var resizeGraph = function () {
                if ($("#content").outerWidth() < 900) {
                    $(".gauge").css({
                        "display":"none"
                    });
    
                    $("#realtime-container").find(".description").css({
                        "margin-left":"15px"
                    });
                } else if ($("#content").outerWidth() < 1200) {
                    $(".gauge").css({
                        "display":"block",
                        "margin-left":"-15px",
                        "margin-right":"-15px"
                    });
    
                    $("#realtime-container").find(".description").css({
                        "margin-left":"0"
                    });
                } else {
                    $(".gauge").css({
                        "display":"block",
                        "margin-left":"0",
                        "margin-right":"0"
                    });
    
                    $("#realtime-container").find(".description").css({
                        "margin-left":"0"
                    });
                }
    
                var svgWidth = $("#realtime-container").width() - ($(".data:eq(0)").outerWidth() + $(".data:eq(1)").outerWidth()) - 20;
    
                self.realtimeGraph.graph.configure({ width: (svgWidth > 0)? svgWidth : 0});
                self.realtimeGraph.graph.update();
            };
    
            resizeGraph();
    
            $(window).on('resize', function () {
                resizeGraph();
            });
        }
    },
    refresh:function () {
        if(Backbone.history.fragment == "/"){
            var self = this;
            $.when(countlyLive.refresh()).then(function () {
                var onlineUserCounts = countlyLive.getOnlineUsers();
    
                if (self.onlineUsersGauge) {
                    self.onlineUsersGauge.maxValue = ((onlineUserCounts.o > onlineUserCounts.mo) ? onlineUserCounts.o : onlineUserCounts.mo) || 3000;
                    self.onlineUsersGauge.set(onlineUserCounts.o);
                }
    
                if (self.newUsersGauge) {
                    self.newUsersGauge.maxValue = ((onlineUserCounts.n > onlineUserCounts.mn) ? onlineUserCounts.n : onlineUserCounts.mn) || 2000;
                    self.newUsersGauge.set(onlineUserCounts.n);
                }
    
                $("#max-online-users-text").text(onlineUserCounts.mo);
                $("#max-new-users-text").text(onlineUserCounts.mn);
    
                self.realtimeGraph.counter = Math.floor((new Date()).getTime() / 1000) - (60 * (countlyCommon.DASHBOARD_REFRESH_MS / 1000));
                self.realtimeGraph.series[0].data = [];
                self.realtimeGraph.series[1].data = [];
    
                var liveGraphData = countlyLive.getOnlineUsersGraphData();
    
                for (var i = 0; i < liveGraphData.length; i++) {
                    self.realtimeGraph.series[0].data.push({x:self.realtimeGraph.counter, y:liveGraphData[i].o});
                    self.realtimeGraph.series[1].data.push({x:self.realtimeGraph.counter, y:liveGraphData[i].n});
                    self.realtimeGraph.counter = self.realtimeGraph.counter + (countlyCommon.DASHBOARD_REFRESH_MS / 1000);
                }
    
                self.realtimeGraph.graph.update();
            });
        }
    }
};

app.addPageScript("/", function(){
	LiveView.initialize();
});

app.addRefreshScript("/", function(){
	LiveView.refresh();
});