var request = require('supertest');
var should = require('should');
var testUtils = require("../../test/testUtils");
request = request(testUtils.url);

var APP_KEY = "";
var API_KEY_ADMIN = "";
var APP_ID = "";
var DEVICE_ID = "1234567890";

describe('Testing Live data', function(){
	describe('Empty live', function(){
		it('should have no live data', function(done){
			API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=live')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				//{"o":0,"n":0,"mo":0,"mn":0}
				ob.should.have.property("o",0);
				ob.should.have.property("n",0);
				ob.should.have.property("mo",0);
				ob.should.have.property("mn",0);
				setTimeout(done, 100)
			});
		});
	});
	describe('Empty live_graph', function(){
		it('should have no live data', function(done){
			//API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			//APP_ID = testUtils.get("APP_ID");
			//APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=live_graph')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				//[{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0}]
				ob.should.have.length(60);
				for(var i = 0; i < ob.length; i++){
					ob[i].should.have.property("o",0);
					ob[i].should.have.property("n",0);
				}
				setTimeout(done, 100)
			});
		});
	});
	describe('Add user', function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&begin_session=1")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify live', function(){
		it('should have no live data', function(done){
			//API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			//APP_ID = testUtils.get("APP_ID");
			//APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=live')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				//{"o":1,"n":1,"mo":1,"mn":1}
				ob.should.have.property("o",1);
				ob.should.have.property("n",1);
				ob.should.have.property("mo",1);
				ob.should.have.property("mn",1);
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify live_graph', function(){
		it('should have no live data', function(done){
			//API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			//APP_ID = testUtils.get("APP_ID");
			//APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=live_graph')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				//[{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":1,"n":1}]
				ob.should.have.length(60);
				ob.should.containEql({"o":1,"n":1});
				setTimeout(done, 100)
			});
		});
	});
	describe('write bulk live', function(){
		it('should success', function(done){
			this.timeout(20000);
			var params = [
				{"device_id":DEVICE_ID+"1", "app_key":APP_KEY, "begin_session":1},
				{"device_id":DEVICE_ID+"2", "app_key":APP_KEY, "begin_session":1},
				{"device_id":DEVICE_ID+"3", "app_key":APP_KEY, "begin_session":1}
			];
			request
			.get('/i/bulk?requests='+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 1500)
			});
		});
	});
	describe('Verify live', function(){
		it('should have no live data', function(done){
			//API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			//APP_ID = testUtils.get("APP_ID");
			//APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=live')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				//{"o":4,"n":4,"mo":4,"mn":4}
				ob.should.have.property("o",4);
				ob.should.have.property("n",4);
				ob.should.have.property("mo",4);
				ob.should.have.property("mn",4);
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify live_graph', function(){
		it('should have no live data', function(done){
			//API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			//APP_ID = testUtils.get("APP_ID");
			//APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=live_graph')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				//[{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":4,"n":4}]
				ob.should.have.length(60);
				ob.should.containEql({"o":4,"n":4});
				setTimeout(done, 100)
			});
		});
	});
	describe('reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify empty live', function(){
		it('should have no live data', function(done){
			//API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			//APP_ID = testUtils.get("APP_ID");
			//APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=live')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				//{"o":0,"n":0,"mo":0,"mn":0}
				ob.should.have.property("o",0);
				ob.should.have.property("n",0);
				ob.should.have.property("mo",0);
				ob.should.have.property("mn",0);
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify empty live_graph', function(){
		it('should have no live data', function(done){
			//API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			//APP_ID = testUtils.get("APP_ID");
			//APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=live_graph')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				//[{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0},{"o":0,"n":0}]
				ob.should.have.length(60);
				for(var i = 0; i < ob.length; i++){
					ob[i].should.have.property("o",0);
					ob[i].should.have.property("n",0);
				}
				setTimeout(done, 100)
			});
		});
	});
});