var plugin = {},
	common = require('../../../api/utils/common.js'),
    plugins = require('../../pluginManager.js');

(function (plugin) {
    var pluginTree = {
        "#/analytics/languages" : {"/o":{langs:true}},
        "#/analytics/retention" : {"/o":{retention:true}},
        //"#/analytics/events" : {"/o":{events: true, get_events: true}},
        "#/drill" : {"/i/drill":true, "/o":{drill_bookmarks:true, segmentation:true }},
        "#/funnels" : {"/o":{funnel:true, get_funnels:true}},
        "#/crashes" : {"/i/crashes":true/*, "/o":{crashes:true}*/},
        "#/users" : {"/o":{user_details:true}},
        "#/manage/apps" : {"/i/apps":true, "/o/apps":true},
        "#/manage/users" : {"/i/users":true, "/o/users":true},
        "#/manage/plugins" : {"/i/plugins":true, "/o/plugins":true},
        "#/manage/configurations" : {"/i/configs":true, "/o/configs":true},
        "#/manage/logger" : {"/o":{logs:true}},
        "#/manage/systemlogs" : {"/o":{systemlogs:true, systemlogs_meta:true}},
        "#/manage/errorlogs" : {"/o/errorlogs":true, "/i/errorlogs":true},
        "#/manage/reports" : {"/o/reports":true, "/i/reports":true},
        "#/manage/db" : {"/o/db":true},
        "#/manage/blocks" : {"/i/blocks":true, "/o/blocks":true},
        "#/all" : {"all_apps":true},
        "#/analytics/users" : {},
        "#/analytics/sessions" : {},
        "#/analytics/countries" : {},
        "#/analytics/devices" : {"/o":{devices:true}},
        "#/analytics/resolutions" : {},
        "#/analytics/versions" : {},
        //"#/analytics/carriers" : {"/o":{carriers:true}},
        "#/analytics/platforms" : {},
        "#/analytics/density" : {"/o":{density:true}},
        //"#/analytics/sources" : {"/o":{sources:true},"/o/sources":true},
        "#/analytics/views" : {"/o":{views:true, get_view_segments:true},"/o/actions":true},
        "#/analytics/loyalty" : {},
        "#/analytics/frequency" : {},
        "#/analytics/durations" : {},
        "#/analytics/view-frequency" : {},
        "#/analytics/star-rating" : {"/o":{star:true}},
        "#/analytics/slipping-away" : {"/o/slipping":true},
        "#/messaging" : {"/i/pushes":true, "/o/pushes":true},
        "#/messaging/geolocations" : {"/i/geolocations":true},
        "#/attribution" : {"/i/campaign":true, /*"/o/campaign":true,*/ "/o":{campaigndata:true}},
        //"#/analytics/browser" : {"/o":{browser:true}},
        "#/iot-all" : {"/o":{"all_apps":true}},
        "#/analytics/iot-devices" : {},
        "#/analytics/iot-countries" : {},
        "#/analytics/iot-device-types" : {"/o":{devices:true}},
        "#/analytics/iot-versions" : {},
        "#/analytics/iot-platforms" : {}
    };
    
    plugins.register("/validation/user", function(ob){
        if(ob && ob.params && ob.params.member && ob.params.member.restrict && ob.params.member.restrict.length){
            var restrict = ob.params.member.restrict;
            for(var i = 0; i < restrict.length; i++){
                if(pluginTree[restrict[i]]){
                    if(ob.params.apiPath == "/o"){
                        if(pluginTree[restrict[i]][ob.params.apiPath] && pluginTree[restrict[i]][ob.params.apiPath][ob.params.qstring.method])
                            return true;
                    }
                    else if(pluginTree[restrict[i]][ob.params.apiPath]){
                        return true;
                    }
                        
                }
            }
        }
        return false;
    });
	plugins.register("/i/restrict", function(ob){
		var params = ob.params;
        var validateUserForWriteAPI = ob.validateUserForWriteAPI;
		validateUserForWriteAPI(function(){
			if (!params.member.global_admin) {
				common.returnMessage(params, 401, 'User is not a global administrator');
				return false;
			}
            else if(!params.qstring.user_id){
                common.returnMessage(params, 400, 'Provide user_id');
				return false;
            }
            var data = null;
            if(params.qstring.restrict){
                try{
                    data = JSON.parse(params.qstring.restrict);
                }
                catch(err){
                    console.log("Error parsing restrict", params.qstring.restrict);
                    data = null;
                }
            }
            if(data){
                common.db.collection('members').findOne({'_id': common.db.ObjectID(params.qstring.user_id)}, function(err, memberBefore) {
                    common.db.collection('members').update({'_id': common.db.ObjectID(params.qstring.user_id)}, {'$set': {restrict:data}}, {safe: true}, function(err, isOk) {
                        plugins.dispatch("/systemlogs", {params:params, action:"restrict_sections", data:{before:memberBefore || {}, update:{restrict:data}, user_id: params.qstring.user_id}});
                        common.returnMessage(params, 200, 'Success');
                    });
                })
            }
            else{
                common.returnMessage(params, 400, 'Error updating restrict');
            }
        }, params);
        return true;
    });
    
    plugins.register("/o/restrict", function(ob){
		var params = ob.params;
        var validateUserForMgmtReadAPI = ob.validateUserForMgmtReadAPI;
		validateUserForMgmtReadAPI(function(){
			if (!params.member.global_admin) {
				common.returnMessage(params, 401, 'User is not a global administrator');
				return false;
			}
            common.db.collection('members').find({restrict:{$exists:true}},{restrict:1}).toArray(function (err, members) {
                var res = {};
                if(!err && members){
                    for(var i = 0; i < members.length; i++){
                        res[members[i]._id] = members[i].restrict;
                    }
                }
                common.returnOutput(params, res);
            });
        }, params);
        return true;
    });
}(plugin));

module.exports = plugin;