var retention = {},
    common = require('../../../../../api/utils/common.js'),
    plugins = require('../../../../pluginManager.js'),
    momentiso = require('moment-isocalendar');

(function (retention) {

    retention.process = function (user, isNewUser, params, callback) {
        var curDate = common.getDate(params.time.timestamp, params.appTimezone),
            currMoment = common.moment(common.getDate(params.time.timestamp, params.appTimezone)),
            currIsoMoment = common.moment(common.getDate(params.time.timestamp, params.appTimezone)).isoday(1),
            dailyBucket = currMoment.format("YYYYMMDD"),
            weeklyBucket = currIsoMoment.format("YYYYMMDD"),
            monthlyBucket = currMoment.format("YYYYMM");

        if (!isNewUser && user && user[common.dbUserMap.first_seen]) {
            var userFirstDate = common.getDate(user[common.dbUserMap.first_seen], params.appTimezone),
                userFirstMoment = common.moment(common.getDate(user[common.dbUserMap.first_seen], params.appTimezone)),
                userFirstIsoMoment = common.moment(common.getDate(user[common.dbUserMap.first_seen], params.appTimezone)).isoday(1),
                usrObj = {};
           
            //assign default values
            user.cdfs = user.cdfs || 0;
            user.cwfs = user.cwfs || 0;
            user.cmfs = user.cmfs || 0;
            user.cadfs = user.cadfs || 0;
            user.cawfs = user.cawfs || 0;
            user.camfs = user.camfs || 0;
            
            //checking relative daily retention
            if(user.cdfs < plugins.getConfig("retention_segments").span && parseInt(common.getDiff(currMoment, userFirstMoment, 'days')) == user.cdfs + 1){
                usrObj["cdfs"] = user.cdfs + 1;
            }
            
            //checking relative weekly retention
            if(user.cwfs < plugins.getConfig("retention_segments").span && parseInt(common.getDiff(currMoment, userFirstMoment, 'weeks')) == user.cwfs + 1){
                usrObj["cwfs"] = user.cwfs + 1;
            }
            
            //checking relative monthly retention
            if(user.cmfs < plugins.getConfig("retention_segments").span && parseInt(Math.floor(common.getDiff(currMoment, userFirstMoment, 'days')/30)) == user.cmfs + 1){
                usrObj["cmfs"] = user.cmfs + 1;
            }
            
            //checking absolute daily retention
            if(user.cadfs < plugins.getConfig("retention_segments").span){
                //reseting dates to full days
                curDate.setHours(0,0,0,0);
                userFirstDate.setHours(0,0,0,0);
                if(parseInt(common.getDiff(common.moment(curDate), common.moment(userFirstDate), 'days')) == user.cadfs + 1){
                    usrObj["cadfs"] = user.cadfs + 1;
                }
            }
            
            //check absolute weekly retention
            if(user.cawfs < plugins.getConfig("retention_segments").span){
                var firstSeenYear = userFirstIsoMoment.isocalendar()[0],
                    currYear = currIsoMoment.isocalendar()[0],
                    retWeeks;
            
                if (currYear == firstSeenYear) {
                    retWeeks = currIsoMoment.isoweek() - userFirstIsoMoment.isoweek();
                } else {
                    var weeksToAdd = 0;
                    for (var i = parseInt(firstSeenYear); i < parseInt(currYear); i++) {
                        weeksToAdd += common.getISOWeeksInYear(i);
                    }
            
                    retWeeks = (weeksToAdd + currIsoMoment.isoweek()) - userFirstIsoMoment.isoweek();
                }
                if(retWeeks == user.cawfs + 1){
                    usrObj["cawfs"] = user.cawfs + 1;
                }
            }
            
            //checking absolute monthly retention
            if(user.camfs < plugins.getConfig("retention_segments").span){
                var yearDiff = (currMoment.format("YYYY") - userFirstMoment.format("YYYY")),
                    monthDiff = (currMoment.format("M") - userFirstMoment.format("M")),
                    retMonths = (yearDiff * 12) + monthDiff;
                if(retMonths == user.camfs + 1){
                    usrObj["camfs"] = user.camfs + 1;
                }
            }
            
            if(Object.keys(usrObj).length){
                common.updateAppUser(params, {$set: usrObj}, function(){
                    if(callback)
                        callback(usrObj); 
                });
            }
            else{
                if(callback)
                    callback();
            }
        } else {
            //assign retention buckets
            var update = {'fsd': dailyBucket, 'fsw':weeklyBucket, 'fsm':monthlyBucket};
            common.updateAppUser(params, {'$set': update}, function(){
                if(callback)
                    callback(update); 
            });
        }
    };
    
    retention.get = function (params) {
        var currMoment = common.moment(common.getDate(params.time.timestamp, params.appTimezone)),
            start = '',
            format = "YYYYMMDD",
            span = 60*60*24,
            bucket = "fsd",
            rl = 'd';

        switch (params.qstring.period) {
            case 'daily':
                start = currMoment.clone().subtract('days', plugins.getConfig("retention_segments").span);
                rl = 'd';
                bucket = "fsd";
                break;
            case 'weekly':
                start = currMoment.clone().subtract('weeks', plugins.getConfig("retention_segments").span).isoday(1);
                rl = 'w';
                span = 60*60*24*7;
                bucket = "fsw";
                break;
            case 'monthly':
                start = currMoment.clone().subtract('months', plugins.getConfig("retention_segments").span);
                format = "YYYYMM";
                rl = 'm';
                span = 60*60*24*20;
                bucket = "fsm";
                break;
            case 'adaily':
                start = currMoment.clone().subtract('days', plugins.getConfig("retention_segments").span);
                rl = 'ad';
                bucket = "fsd";
                break;
            case 'aweekly':
                start = currMoment.clone().subtract('weeks', plugins.getConfig("retention_segments").span).isoday(1);
                rl = 'aw';
                span = 60*60*24*7;
                bucket = "fsw";
                break;
            case 'amonthly':
                start = currMoment.clone().subtract('months', plugins.getConfig("retention_segments").span);
                format = "YYYYMM";
                rl = 'am';
                span = 60*60*24*20;
                bucket = "fsm";
                break;
            default:
                start = currMoment.clone().subtract('days', plugins.getConfig("retention_segments").span);
                break;
        }
        var buckets = {};
        var docs = [];
        //generate empty buckets
        var startTs = start.unix(), 
            id;
        while(startTs <= currMoment.unix()){
            if(rl == 'd' || rl == 'ad')
                id = common.moment(startTs*1000).format("YYYYMMDD");
            else if(rl == 'w' || rl == 'aw')
                id = common.moment(startTs*1000).isoday(1).format("YYYYMMDD");
            else if(rl == 'm' || rl == 'am')
                id = common.moment(startTs*1000).format("YYYYMM");
            
            if(typeof buckets[id] === "undefined"){
                buckets[id] = {_id:id, tu:0};
                docs.push(id);
            }
            startTs += span;
        }
        var query = {};
        try{
            query = JSON.parse(params.qstring.query);
        }
        catch(ex){
            query = {};
        }
        query[bucket] = {$in: docs};     
        common.db.collection("app_users"+params.qstring.app_id).aggregate([
            {
                $match: query
            },
            {
                $group: { _id: {d: "$"+bucket, r: "$c"+rl+"fs"}, tu: {$sum: 1}}
            }
        ],{allowDiskUse:true},function (err, users) {
            for(var i = 0; i < users.length; i++){
                if(users[i]._id.d){
                    if(buckets[users[i]._id.d]){
                        buckets[users[i]._id.d].tu += users[i].tu;
                        if(users[i]._id.r){
                            for(var t = 1; t <= users[i]._id.r; t++){
                                if(typeof buckets[users[i]._id.d][t] === "undefined")
                                    buckets[users[i]._id.d][t] = 0;
                                buckets[users[i]._id.d][t] += users[i].tu;
                            }
                        }
                    }
                }
            }
            var res = [];
            for(var i in buckets){
                res.push(buckets[i]);
            }
            common.returnOutput(params, res);
        });
    };

}(retention));

module.exports = retention;