(function (countlyRetention, $, undefined) {

    //Private Properties
    var _periodObj = {},
        _retentionData = [],
        _activeBucket = "daily",
        _timestamp,
        _buckets = {
            'daily':{
                input:"YYYYMMDD",
                display:function (m) {
                    return countlyCommon.formatDate(m,"MMM D");
                },
                tick:function (m) {
                    return countlyCommon.formatDate(m,"MMM D");
                },
                columnTick:function (m) {
                    return countlyCommon.formatDate(m,"MMM D");
                },
                period:"days"
            },
            'weekly':{
                input:"YYYYMMDD",
                display:function (m) {
                    return "W" + m.isoweek() + ", " + countlyCommon.formatDate(m,"MMM D");
                },
                tick:function (m) {
                    return "W" + m.isoweek();
                },
                columnTick:function (m) {
                    return "W" + m.isoweek();
                },
                period:"weeks"
            },
            'monthly':{
                input:"YYYYMM",
                display:function (m) {
                    return countlyCommon.formatDate(m,"MMMM");
                },
                tick:function (m) {
                    return countlyCommon.formatDate(m,"MMM");
                },
                columnTick:function (m) {
                    return countlyCommon.formatDate(m,"MMM");
                },
                period:"months"
            },
            'adaily':{
                input:"YYYYMMDD",
                display:function (m) {
                    return countlyCommon.formatDate(m,"MMM D");
                },
                tick:function (m) {
                    return countlyCommon.formatDate(m,"MMM D");
                },
                columnTick:function (m) {
                    return countlyCommon.formatDate(m,"MMM D");
                },
                period:"days"
            },
            'aweekly':{
                input:"YYYYMMDD",
                display:function (m) {
                    return "W" + m.isoweek() + ", " + countlyCommon.formatDate(m,"MMM D");
                },
                tick:function (m) {
                    return "W" + m.isoweek();
                },
                columnTick:function (m) {
                    return "Week " + m.isoweek();
                },
                period:"weeks"
            },
            'amonthly':{
                input:"YYYYMM",
                display:function (m) {
                    return countlyCommon.formatDate(m,"MMMM");
                },
                tick:function (m) {
                    return countlyCommon.formatDate(m,"MMM");
                },
                columnTick:function (m) {
                    return countlyCommon.formatDate(m,"MMM");
                },
                period:"months"
            }
        };

    //Public Methods
    countlyRetention.initialize = function (query) {
        if (typeof countlySegmentation != "undefined") {
            countlySegmentation.reset();
        }
        if(query)
            query = JSON.stringify(query);
        if (!countlyCommon.DEBUG) {
            return $.when(
                $.ajax({
                    type:"GET",
                    url:countlyCommon.API_PARTS.data.r,
                    data:{
                        "api_key":countlyGlobal.member.api_key,
                        "app_id":countlyCommon.ACTIVE_APP_ID,
                        "method":"retention",
                        "period":_activeBucket,
                        "timestamp":_timestamp || undefined,
                        "query": query || undefined
                    },
                    dataType:"jsonp",
                    success:function (json) {
                        if (_retentionData.length) {
                            _retentionData = _.sortBy(json, function (data) {
                                return data._id;
                            });
                        } else {
                            _retentionData = json;
                        }
                    }
                })
            ).then(function () {
                    return true;
                });
        } else {
            return true;
        }
    };
    
    countlyRetention.refresh = function (query) {
            if(query)
                query = JSON.stringify(query);
            return $.ajax({
                    type:"GET",
                    url:countlyCommon.API_PARTS.data.r,
                    data:{
                        "api_key":countlyGlobal.member.api_key,
                        "app_id":countlyCommon.ACTIVE_APP_ID,
                        "method":"retention",
                        "period":_activeBucket,
                        "timestamp":_timestamp || undefined,
                        "query": query || undefined,
                        "display_loader": false
                    },
                    dataType:"jsonp",
                    success:function (json) {
                        if (_retentionData.length) {
                            _retentionData = _.sortBy(json, function (data) {
                                return data._id;
                            });
                        } else {
                            _retentionData = json;
                        }
                    }
                });
    };

    countlyRetention.setActiveBucket = function (bucket) {
        if (bucket == "daily" || bucket == "weekly" || bucket == "monthly" || bucket == "adaily" || bucket == "aweekly" || bucket == "amonthly") {
            _activeBucket = bucket;
        }
    };
    
    countlyRetention.getActiveBucket = function () { return _activeBucket;};

    countlyRetention.setTimestamp = function (timestamp) {
        _timestamp = timestamp;
    };

    countlyRetention.getActivePeriod = function () {
        return _buckets[_activeBucket].period;
    };

    countlyRetention.getRetentionData = function () {

        var retData = {
                max:0,
                'column-ticks':[],
                period:_buckets[_activeBucket].period,
                rows:[],
                graph1:{
                    dp:[],
                    tip:[],
                    dpMax:0,
                    ticks:[]
                },
                graph2:{
                    dp:[],
                    dpMax:0,
                    ticks:[]
                }
            },
            currMoment = moment(),
            tmpCounter = 1;

        for (var i = 0; i < _retentionData.length; i++) {
            var periodRetention = [],
                tmpMoment = moment(_retentionData[i]._id, _buckets[_activeBucket].input),
                tmpMomentClone = tmpMoment.clone(),
                tickMoment = tmpMoment.clone(),
                tmpMaxRetention = Math.floor(currMoment.diff(tmpMoment, _buckets[_activeBucket].period, true)),
                dateString = _buckets[_activeBucket].display(tmpMoment);
                
            var limit = countlyRetentionConfig["retention_span"] || 10;
            if (tmpMaxRetention > limit) {
                tmpMaxRetention = limit;
            }

            if (i == 0) {
                retData['max'] = tmpMaxRetention;

                for (var k = 1; k <= tmpMaxRetention; k++) {
                    retData["column-ticks"].push({
                        count:k
                    });
                }
            }

            if (tmpMaxRetention != 0) {
                retData.graph1.dp[i] = {};
                retData.graph1.dp[i].data = [];

                retData.graph2.dp[i] = {};
                retData.graph2.dp[i].data = [];
                retData.graph2.dp[i].label = _buckets[_activeBucket].tick(tmpMomentClone);
            }

            for (var j = 1; j <= tmpMaxRetention; j++) {
                var percent = 0;

                if(_retentionData[i].tu > 0)
                    percent = (((_retentionData[i][j + ""] || 0) / _retentionData[i].tu) * 100).toFixed(2);

                periodRetention.push({
                    date:_buckets[_activeBucket].display(tmpMoment.add(_buckets[_activeBucket].period, 1)),
                    users:_retentionData[i][j + ""] || 0,
                    percent:parseFloat(percent),
                    retention:j,
                    percentFloat:(percent / 100).toFixed(2),
                    class: (percent > 50)? "light": "dark"
                });

                if (tmpMaxRetention != 0) {
                    retData.graph1.dp[i].data.push([tmpCounter, percent]);
                    retData.graph2.dp[i].data.push([j, percent]);

                    retData.graph1.tip[tmpCounter] = _retentionData[i].tu - (_retentionData[i][j + ""] || 0) + " of " + _retentionData[i].tu;

                    retData.graph2.ticks.push([j, "+" + j]);

                    tmpCounter++;
                }
            }

            if (tmpMaxRetention != 0) {
                retData.graph1.ticks.push([(tmpCounter + tmpCounter - tmpMaxRetention - 1) / 2, _buckets[_activeBucket].tick(tmpMomentClone)]);
            }

            var hoverRetDate = [];

            for (var k = 1; k <= tmpMaxRetention; k++) {
                hoverRetDate.push(_buckets[_activeBucket].columnTick(tickMoment.add(_buckets[_activeBucket].period, 1)));
            }

            retData.rows.push({
                date:dateString,
                bucket: _retentionData[i]._id,
                users:_retentionData[i].tu,
                retention:periodRetention,
                retColumns:JSON.stringify(hoverRetDate)
            });
        }

        retData.graph1.dpMax = tmpCounter;
        retData.graph2.dpMax = retData['max'] + 1;

        return retData;
    };

}(window.countlyRetention = window.countlyRetention || {}, jQuery));