var request = require('supertest');
var should = require('should');
var testUtils = require("../../test/testUtils");
request = request(testUtils.url);

var APP_KEY = "";
var API_KEY_ADMIN = "";
var APP_ID = "";
var DEVICE_ID = "1234567890";
var daysBack = [32, 31, 30, 29, 28, 27, 21, 14, 7, 1];
var dailyTs = parseInt(new Date().getTime()/1000 - 24 * 60 * 60 * 25);
var days = [];
for(var i = 0; i < daysBack.length; i++){
	days.push(parseInt(new Date().getTime()/1000 - 24 * 60 * 60 * daysBack[i]));
}
var correct = {
	"31":{"daily":["1"]},
	"30":{"daily":["1","2"]},
	"29":{"daily":["1","2","3"]},
	"28":{"daily":["1","2","3","4"]},
	"27":{"daily":["1","2","3","4","5"]},
	"21":{"daily":["1","2","3","4","5"],"weekly":["1"]},
	"14":{"daily":["1","2","3","4","5"],"weekly":["1","2"]},
	"7":{"daily":["1","2","3","4","5"],"weekly":["1","2","3"]},
	"1":{"daily":["1","2","3","4","5"],"weekly":["1","2","3","4"],"monthly":["1"]}
};

function testDay(day, ts, device, u, correct){
	describe('Start session days '+day+" ago", function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+device+'&app_key='+APP_KEY+'&begin_session=1&timestamp='+ts)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 1000)
			});
		});
	});
	describe('Session duration days '+day+" ago", function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+device+'&app_key='+APP_KEY+'&session_duration=30&timestamp='+ts)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 1000)
			});
		});
	});
	describe('End session days '+day+" ago", function(){
		it('should success', function(done){
			this.timeout(20000);
			request
			.get('/i?device_id='+device+'&app_key='+APP_KEY+'&end_session=1&timestamp='+ts)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 15000)
			});
		});
	});
	describe('Verify retention relative daily days '+day+" ago", function(){
		it('should have no retention', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=retention&timestamp='+dailyTs)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
                var ob = JSON.parse(res.text);
                var hasRetention = false;
				for(var i = 0; i < ob.length; i++){
                    if(ob[i].tu != 0){
                        hasRetention = true;
                        var period = "daily";
                        if(correct && correct[period]){
                            for(var j = 0; j < correct[period].length; j++){
                                ob[i].should.have.property(correct[period][j],u);
                            }
                        }
                    }
                }
                hasRetention.should.be.true();
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify retention absolute daily days '+day+" ago", function(){
		it('should have no retention', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=retention&period=adaily&timestamp='+dailyTs)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
                var ob = JSON.parse(res.text);
                var hasRetention = false;
				for(var i = 0; i < ob.length; i++){
                    if(ob[i].tu != 0){
                        hasRetention = true;
                        var period = "daily";
                        if(correct && correct[period]){
                            for(var j = 0; j < correct[period].length; j++){
                                ob[i].should.have.property(correct[period][j],u);
                            }
                        }
                    }
                }
                hasRetention.should.be.true();
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify retention relative weekly days '+day+" ago", function(){
		it('should have no retention', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=retention&period=weekly')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				var hasRetention = false;
				for(var i = 0; i < ob.length; i++){
                    if(ob[i].tu != 0){
                        hasRetention = true;
                        var period = "weekly";
                        if(correct && correct[period]){
                            for(var j = 0; j < correct[period].length; j++){
                                ob[i].should.have.property(correct[period][j],u);
                            }
                        }
                    }
                }
                hasRetention.should.be.true();
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify retention absolute weekly days '+day+" ago", function(){
		it('should have no retention', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=retention&period=aweekly')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				var hasRetention = false;
				for(var i = 0; i < ob.length; i++){
                    if(ob[i].tu != 0){
                        hasRetention = true;
                        var period = "weekly";
                        if(correct && correct[period]){
                            for(var j = 0; j < correct[period].length; j++){
                                ob[i].should.have.property(correct[period][j],u);
                            }
                        }
                    }
                }
                hasRetention.should.be.true();
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify retention relative monthly days '+day+" ago", function(){
		it('should have no retention', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=retention&period=monthly')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				var hasRetention = false;
				for(var i = 0; i < ob.length; i++){
                    if(ob[i].tu != 0){
                        hasRetention = true;
                        var period = "monthly";
                        if(correct && correct[period]){
                            for(var j = 0; j < correct[period].length; j++){
                                ob[i].should.have.property(correct[period][j],u);
                            }
                        }
                    }
                }
                hasRetention.should.be.true();
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify retention absolute monthly days '+day+" ago", function(){
		it('should have no retention', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=retention&period=amonthly')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				var hasRetention = false;
				for(var i = 0; i < ob.length; i++){
                    if(ob[i].tu != 0){
                        hasRetention = true;
                        var period = "monthly";
                        if(correct && correct[period]){
                            for(var j = 0; j < correct[period].length; j++){
                                ob[i].should.have.property(correct[period][j],u);
                            }
                        }
                    }
                }
                hasRetention.should.be.true();
				setTimeout(done, 100)
			});
		});
	});
}

describe('Testing Retention', function(){
	describe('Empty retention', function(){
		it('should have no retention', function(done){
			API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=retention')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				for(var i = 0; i < ob.length; i++){
                    ob[i].should.have.property('tu', 0);
                }
				setTimeout(done, 100)
			});
		});
	});
	for(var i = 0; i < days.length; i++){
		testDay(daysBack[i], days[i], DEVICE_ID, 1, correct[daysBack[i]]);
	}
	describe('Reset app', function(){
		it('should reset data', function(done){
            this.timeout(10000);
			var params = {app_id:APP_ID};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify empty retention', function(){
		it('should have no retention', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=retention')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				for(var i = 0; i < ob.length; i++){
                    ob[i].should.have.property('tu', 0);
                }
				setTimeout(done, 100)
			});
		});
	});
});