var pluginManager = require('../pluginManager.js'),
    async = require('async'),
    moment = require('moment'),
    crypto = require('crypto'),
    retention = require('./api/parts/data/retention.js'),
    common = require('../../api/utils/common.js'),
    drillDb = pluginManager.dbConnection("countly_drill");

console.log("Upgrading retention plugin");
console.log("This may take a while, depending on amount of users and sessions they had");
function done(){
	console.log("Retention plugin upgrade finished");
	common.db.close();
    drillDb.close();
}
pluginManager.loadConfigs(common.db, function(){
    common.db.collection('apps').find({}).toArray(function (err, apps) {
        if (!apps || err) {
            done();
            return;
        }
        function upgrade(app, callback){
            console.log("Processing app", app.name);
            common.db.collection('app_users' + app._id).find({ _id: { $ne: "uid-sequence" }, fsd:{$exists:false} }, {uid:1, fs:1}).toArray(function(err, users){
                if (!users || err) {
                    callback();
                    return;
                }
                function processUser(user, cb){
                    console.log("Processing user", app.name, user.uid);
                    if(user && user.fs){
                        var collectionName = "drill_events" + crypto.createHash('sha1').update("[CLY]_session" + app._id).digest('hex');
                        drillDb.collection(collectionName).find({uid:user.uid}, {uid:1, ts:1}).sort({ts:1}).toArray(function(err, sessions){
                            if (!sessions || err) {
                                setTimeout( function() {
                                    cb();
                                },0);
                                return;
                            }
                            var breakOut = false;
                            var first = true;
                            async.forEachSeries(sessions, function(item, c){
                                //if(breakOut)
                                    //return c();
                                console.log("Processing session", app.name, user.uid, item.ts);
                                var params = {};
                                params.qstring = {}
                                params.qstring.timestamp = item.ts;
                                params.app_id = app['_id'];
                                params.app_cc = app['country'];
                                params.app_name = app['name'];
                                params.appTimezone = app['timezone'];
                                params.app = app;
                                params.app_user_id = user._id;
                                params.app_user = user;
                                params.time = common.initTimeObj(params.appTimezone, params.qstring.timestamp);
                                retention.process(user, first, params, function(data){
                                    if(first){
                                        first = false;
                                    }
                                    if(data){
                                        for(var i in data){
                                            if(i == "cdfs" || i == "cwfs" || i == "cmfs" || i == "cadfs" || i == "cawfs" || i == "camfs")
                                                user[i] = data[i];
                                        }
                                        setTimeout( function() {
                                            c();
                                        },0);
                                    }
                                    else{
                                        breakOut = true;
                                        setTimeout( function() {
                                            c();
                                        },0);
                                    }
                                });
                            }, function(){
                                setTimeout( function() {
                                    cb();
                                },0);
                            });
                        });
                    }
                    else{
                        setTimeout( function() {
                            cb();
                        },0);
                    }
                }
                async.forEachSeries(users, processUser, function(){
                    console.log("Processed app", app.name);
                    setTimeout( function() {
                        callback();
                    },0);
                });
            });
        }
        async.forEachSeries(apps, upgrade, done);
    });
});
