(function (countlyRevenue, $, undefined) {

    //Private Properties
    var _periodObj = {},
        _revenueDb = {},
        _activeAppId = 0,
        _initialized = false,
        _period = null;

    //Public Methods
    countlyRevenue.initialize = function () {
        if (_initialized && _period == countlyCommon.getPeriodForAjax() && _activeAppId == countlyCommon.ACTIVE_APP_ID) {
            return countlyRevenue.refresh();
        }

        _period = countlyCommon.getPeriodForAjax();

        if (!countlyCommon.DEBUG) {
            _activeAppId = countlyCommon.ACTIVE_APP_ID;
            _initialized = true;

            return $.when(
                countlyUser.initialize()
            ).then(function(){
                return $.ajax({
                    type:"GET",
                    url:countlyCommon.API_PARTS.data.r,
                    data:{
                        "api_key":countlyGlobal.member.api_key,
                        "app_id":countlyCommon.ACTIVE_APP_ID,
                        "method" : "events",
                        "event": countlyGlobal["apps"][countlyCommon.ACTIVE_APP_ID]["iap_event"] || "",
                        "period":_period
                    },
                    dataType:"jsonp",
                    success:function (json) {
                        _revenueDb = json;
                    }
                });
            });

        } else {
            _revenueDb = {"2012":{}};
            return true;
        }
    };

    countlyRevenue.refresh = function () {
        if (!countlyCommon.DEBUG) {

            if (_activeAppId != countlyCommon.ACTIVE_APP_ID) {
                _activeAppId = countlyCommon.ACTIVE_APP_ID;
                return countlyRevenue.initialize();
            }

            return $.when(
                countlyUser.refresh()
            ).then(function(){
                return $.ajax({
                    type:"GET",
                    url:countlyCommon.API_PARTS.data.r,
                    data:{
                        "api_key":countlyGlobal.member.api_key,
                        "app_id":countlyCommon.ACTIVE_APP_ID,
                        "method" : "events",
                        "event": countlyGlobal["apps"][countlyCommon.ACTIVE_APP_ID]["iap_event"] || "",
                        "action" : "refresh"
                    },
                    dataType:"jsonp",
                    success:function (json) {
                        _revenueDb = $.extend(true, _revenueDb, json);
                    }
                });
            });

        } else {
            _revenueDb = {"2012":{}};
            return true;
        }
    };

    countlyRevenue.hasIAPEvent = function () {
        return true;
    };

    countlyRevenue.reset = function () {
        _initialized = false;
        _revenueDb = {};
    };

    countlyRevenue.getRevenueData = function () {
        _periodObj = countlyCommon.periodObj;

        var currRevenue = 0,
            prevRevenue = 0,
            tmp_x,
            tmp_y,
            sessionData = countlySession.getSessionData();

        if (_periodObj.isSpecialPeriod) {
            for (var i = 0; i < (_periodObj.currentPeriodArr.length); i++) {
                tmp_x = countlyCommon.getDescendantProp(_revenueDb, _periodObj.currentPeriodArr[i]);
                tmp_y = countlyCommon.getDescendantProp(_revenueDb, _periodObj.previousPeriodArr[i]);
                tmp_x = countlyEvent.clearEventsObject(tmp_x);
                tmp_y = countlyEvent.clearEventsObject(tmp_y);

                currRevenue += tmp_x["s"];
                prevRevenue += tmp_y["s"];
            }
        } else {
            tmp_x = countlyCommon.getDescendantProp(_revenueDb, _periodObj.activePeriod);
            tmp_y = countlyCommon.getDescendantProp(_revenueDb, _periodObj.previousPeriod);
            tmp_x = countlyEvent.clearEventsObject(tmp_x);
            tmp_y = countlyEvent.clearEventsObject(tmp_y);

            currRevenue = tmp_x["s"];
            prevRevenue = tmp_y["s"];
        }

        var currUsers = sessionData.usage["total-users"]["total"],
            prevUsers = sessionData.usage["total-users"]["prev-total"],
            currAvgRev = (currUsers == 0)? 0 : (currRevenue / currUsers).toFixed(2),
            prevAvgRev = (prevUsers == 0)? 0 : (prevRevenue / prevUsers).toFixed(2),
            currPaying = sessionData.usage["paying-users"]["total"],
            prevPaying = sessionData.usage["paying-users"]["prev-total"],
            currAvgPaying = (currPaying == 0)? 0 : (currRevenue / currPaying).toFixed(2),
            prevAvgPaying = (prevPaying == 0)? 0 : (prevRevenue / prevPaying).toFixed(2),
            currPayingTotal = (currUsers <= 0)? 0 : ((currPaying / currUsers) * 100).toFixed(2),
            prevPayingTotal = (prevUsers <= 0)? 0 : ((prevPaying / prevUsers) * 100).toFixed(2);

        return {
            "revenue": {
                "total": currRevenue.toFixed(2),
                "change": countlyCommon.getPercentChange(prevRevenue, currRevenue)
            },
            "avg-revenue-per-user": {
                "total": currAvgRev,
                "change": countlyCommon.getPercentChange(prevAvgRev, currAvgRev)
            },
            "avg-revenue-per-paying": {
                "total": currAvgPaying,
                "change": countlyCommon.getPercentChange(prevAvgPaying, currAvgPaying)
            },
            "paying": {
                "total": currPaying,
                "change": countlyCommon.getPercentChange(prevPaying, currPaying)
            },
            "paying-total": {
                "total": currPayingTotal + "%",
                "change": countlyCommon.getPercentChange(prevPayingTotal, currPayingTotal)
            }
        };
    };

    countlyRevenue.getRevenueDP = function () {
        var chartData = [
                { data:[], label:jQuery.i18n.map["revenue.revenue"], color:'#DDDDDD', mode:"ghost"},
                { data:[], label:jQuery.i18n.map["revenue.revenue"], color: countlyCommon.GRAPH_COLORS[1] }
            ],
            dataProps = [
                {
                    name:"ps",
                    func:function (dataObj) {
                        return dataObj["s"].toFixed(2);
                    },
                    period:"previous"
                },
                {
                    name:"s",
                    func:function (dataObj) {
                        return dataObj["s"].toFixed(2);
                    }
                }
            ];

        var retData = countlyCommon.extractChartData(_revenueDb, countlyEvent.clearEventsObject, chartData, dataProps);
        retData.chartData = _.map(retData.chartData, function(obj){  delete obj["ps"]; return obj; });

        return retData;
    };

    countlyRevenue.getAvgRevenueDP = function () {
        var tmpDb = $.extend(true, _revenueDb, countlySession.getSessionDb());

        var chartData = [
                { data:[], label:countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.avg-revenue"] + " " + jQuery.i18n.map["common.per-user"]), color:'#DDDDDD', mode:"ghost"},
                { data:[], label:countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.avg-revenue"] + " " + jQuery.i18n.map["common.per-user"]), color:countlyCommon.GRAPH_COLORS[1] }
            ],
            dataProps = [
                {
                    name:"prev_avg",
                    func:function (dataObj) {
                        return (dataObj["u"] == 0)? 0 : (dataObj["s"] / dataObj["u"]).toFixed(2);
                    },
                    period:"previous"
                },
                {
                    name:"curr_avg",
                    func:function (dataObj) {
                        return (dataObj["u"] == 0)? 0 : (dataObj["s"] / dataObj["u"]).toFixed(2);
                    }
                }
            ];

        var retData = countlyCommon.extractChartData(tmpDb, countlyRevenue.clearRevenueObject, chartData, dataProps);
        retData.chartData = _.map(retData.chartData, function(obj){ delete obj["prev_avg"]; return obj; });

        return retData;
    };

    countlyRevenue.getAvgPayingRevenueDP = function () {
        var tmpDb = $.extend(true, _revenueDb, countlySession.getSessionDb());

        var chartData = [
                { data:[], label:countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.avg-revenue"] + " " + jQuery.i18n.map["common.per-paying-user"]), color:'#DDDDDD', mode:"ghost"},
                { data:[], label:countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.avg-revenue"] + " " + jQuery.i18n.map["common.per-paying-user"]), color:countlyCommon.GRAPH_COLORS[1] }
            ],
            dataProps = [
                {
                    name:"prev_avg",
                    func:function (dataObj) {
                        return (dataObj["p"] == 0)? 0 : (dataObj["s"] / dataObj["p"]).toFixed(2);
                    },
                    period:"previous"
                },
                {
                    name:"curr_avg",
                    func:function (dataObj) {
                        return (dataObj["p"] == 0)? 0 : (dataObj["s"] / dataObj["p"]).toFixed(2);
                    }
                }
            ];

        var retData = countlyCommon.extractChartData(tmpDb, countlyRevenue.clearRevenueObject, chartData, dataProps);
        retData.chartData = _.map(retData.chartData, function(obj){ delete obj["prev_avg"]; return obj; });

        return retData;
    };

    countlyRevenue.getPayingDP = function () {
        var sessionDb = $.extend(true, {}, countlySession.getSessionDb()),
            chartData = [
                { data:[], label:countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.paying"] + " " + jQuery.i18n.map["common.users"]), color:'#DDDDDD', mode:"ghost"},
                { data:[], label:countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.paying"] + " " + jQuery.i18n.map["common.users"]), color:countlyCommon.GRAPH_COLORS[1] }
            ],
            dataProps = [
                {
                    name:"prev_paying",
                    func:function (dataObj) {
                        return dataObj["p"].toFixed(2);
                    },
                    period:"previous"
                },
                { name:"p" }
            ];

        var retData = countlyCommon.extractChartData(sessionDb, countlyRevenue.clearRevenueObject, chartData, dataProps);
        retData.chartData = _.map(retData.chartData, function(obj){  delete obj["prev_paying"]; return obj; });

        return retData;
    };

    countlyRevenue.getPayingTotalDP = function () {
        var sessionDb = $.extend(true, {}, countlySession.getSessionDb()),
            chartData = [
                { data:[], label:"% " + countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.paying-total"] + " " + jQuery.i18n.map["common.users"]), color:'#DDDDDD', mode:"ghost"},
                { data:[], label:"% " + countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.paying-total"] + " " + jQuery.i18n.map["common.users"]), color:countlyCommon.GRAPH_COLORS[1] }
            ],
            dataProps = [
                {
                    name:"prev_paying_non_paying",
                    func:function (dataObj) {
                        return (dataObj["u"] <= 0)? 0 : ((dataObj["p"] / dataObj["u"]) * 100).toFixed(2);
                    },
                    period:"previous"
                },
                {
                    name:"curr_paying_non_paying",
                    func:function (dataObj) {
                        return (dataObj["u"] <= 0)? 0 : ((dataObj["p"] / dataObj["u"]) * 100).toFixed(2);
                    }
                }
            ];

        var retData = countlyCommon.extractChartData(sessionDb, countlyRevenue.clearRevenueObject, chartData, dataProps);
        retData.chartData = _.map(retData.chartData, function(obj){  delete obj["prev_paying_non_paying"]; return obj; });

        return retData;
    };

    countlyRevenue.clearRevenueObject = function (obj) {
        if (obj) {
            if (!obj["t"]) obj["t"] = 0;
            if (!obj["n"]) obj["n"] = 0;
            if (!obj["u"]) obj["u"] = 0;
            if (!obj["d"]) obj["d"] = 0;
            if (!obj["e"]) obj["e"] = 0;
            if (!obj["p"]) obj["p"] = 0;
            if (!obj["s"]) obj["s"] = 0;
            if (!obj["c"]) obj["c"] = 0;
        }
        else {
            obj = {"t":0, "n":0, "u":0, "d":0, "e":0, "p":0, "s":0, "c":0};
        }

        return obj;
    };

}(window.countlyRevenue = window.countlyRevenue || {}, jQuery));