window.RevenueView = countlyView.extend({
    activeTab: store.get("countly_rev_tab") || "revenue",
    beforeRender: function() {
		if(this.template)
			return $.when(countlyRevenue.initialize()).then(function () {});
		else{
			var self = this;
			return $.when($.get(countlyGlobal["path"]+'/revenue/templates/revenue.html', function(src){
				self.template = Handlebars.compile(src);
			}), countlyRevenue.initialize()).then(function () {});
		}
    },
    initialize:function () {},
    pageScript:function () {
        $("#" + this.activeTab).addClass("active");

        var self = this;

        $(".rev-container").find(".item").on("click", function() {
            $(".rev-container").find(".item").removeClass("active");
            $(this).addClass("active");
            store.set("countly_rev_tab", $(this).attr("id"));
            self.activeTab = $(this).attr("id");
            self.refresh(true);
        });

        function fixMenu() {
            if ($("#content-container").width() < 960) {
                $("#rev-left").find(".item .number").hide();
            } else {
                $("#rev-left").find(".item .number").show();
            }
        }

        $(window).on('resize', function () {
            fixMenu();
        });

        fixMenu();
        app.localize();
    },
    getActiveData: function() {
        var activeDp = null,
            tableHeader = "",
            tableDataProp = "";

        switch (this.activeTab) {
            case "revenue":
                activeDp = countlyRevenue.getRevenueDP();
                tableHeader = jQuery.i18n.map["revenue.revenue"];
                tableDataProp = "s";
                break;
            case "avg-revenue-per-user":
                activeDp = countlyRevenue.getAvgRevenueDP();
                tableHeader = countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.avg-revenue"] + " " + jQuery.i18n.map["common.per-user"]);
                tableDataProp = "curr_avg";
                break;
            case "avg-revenue-per-paying":
                activeDp = countlyRevenue.getAvgPayingRevenueDP();
                tableHeader = countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.avg-revenue"] + " " + jQuery.i18n.map["common.per-paying-user"]);
                tableDataProp = "curr_avg";
                break;
            case "paying":
                activeDp = countlyRevenue.getPayingDP();
                tableHeader = countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.paying"] + " " + jQuery.i18n.map["common.users"]);
                tableDataProp = "p";
                break;
            case "paying-total":
                activeDp = countlyRevenue.getPayingTotalDP();
                tableHeader = countlyCommon.toFirstUpper(jQuery.i18n.map["revenue.paying-total"] + " " + jQuery.i18n.map["common.users"]);
                tableDataProp = "curr_paying_non_paying";
                break;
            default:
                activeDp = countlyRevenue.getRevenueDP();
                break;
        }

        return {
            activeDp: activeDp,
            tableHeader: tableHeader,
            tableDataProp: tableDataProp
        };
    },
    drawTable:function() {
        if (this.dtable && this.dtable.fnDestroy) {
            this.dtable.fnDestroy(true);
            delete this.dtable;
        }

        $(".revenue-table").remove();
        $("#content").append('<div class="revenue-table"><table class="d-table" cellpadding="0" cellspacing="0"></table></div>');

        var activeData = this.getActiveData();

        this.dtable = $('.d-table').dataTable($.extend({}, $.fn.dataTable.defaults, {
            "aaData": activeData.activeDp.chartData,
            "aoColumns": [
                {"mData":"date", "sType":"customDate", "sTitle":jQuery.i18n.map["common.date"]},
                {"mData":activeData.tableDataProp, sType:"formatted-num", "mRender":function(d) { return countlyCommon.formatNumber(d); }, "sTitle":activeData.tableHeader}
            ]
        }));

        $(".d-table").stickyTableHeaders();
    },
    renderCommon:function (isRefresh) {
        this.templateData = {
            "page-title":jQuery.i18n.map["revenue.iap"].toUpperCase(),
            "logo-class":"revenue"
        };

        var revenueData = countlyRevenue.getRevenueData(),
            activeData = this.getActiveData();

        this.templateData["revenue"] = [
            {
                "title":jQuery.i18n.map["revenue.revenue"].toUpperCase(),
                "subtitle":"",
                "data":revenueData['revenue'],
                "id":"revenue"
            },
            {
                "title":jQuery.i18n.map["revenue.avg-revenue"].toUpperCase(),
                "subtitle":jQuery.i18n.map["common.per-user"].toLowerCase(),
                "data":revenueData['avg-revenue-per-user'],
                "id":"avg-revenue-per-user"
            },
            {
                "title":jQuery.i18n.map["revenue.avg-revenue"].toUpperCase(),
                "subtitle":jQuery.i18n.map["common.per-paying-user"].toLowerCase(),
                "data":revenueData['avg-revenue-per-paying'],
                "id":"avg-revenue-per-paying"
            },
            {
                "title":jQuery.i18n.map["revenue.paying"].toUpperCase(),
                "subtitle":jQuery.i18n.map["common.users"].toLowerCase(),
                "data":revenueData['paying'],
                "id":"paying"
            },
            {
                "title":jQuery.i18n.map["revenue.paying-total"].toUpperCase(),
                "subtitle":jQuery.i18n.map["common.users"].toLowerCase(),
                "data":revenueData['paying-total'],
                "id":"paying-total"
            }
        ];

        if (!isRefresh) {
            $(this.el).html(this.template(this.templateData));

            this.drawTable();
            this.pageScript();
        }

        countlyCommon.drawTimeGraph(activeData.activeDp.chartDP, "#dashboard-graph");
    },
    refresh:function (changedItem) {
        var self = this;
        $.when(this.beforeRender()).then(function () {
            if (app.activeView != self) {
                return false;
            }
            self.renderCommon(true);
            var newPage = $("<div>" + self.template(self.templateData) + "</div>");
            $(self.el).find("#rev-left").replaceWith(newPage.find("#rev-left"));

            if (changedItem) {
                self.drawTable();
            } else {
                var activeData = self.getActiveData();
                CountlyHelpers.refreshTable(self.dtable, activeData.activeDp.chartData);
            }

            self.pageScript();
        });
    },
    destroy:function() {
        if (this.dtable && this.dtable.fnDestroy) {
            this.dtable.fnDestroy(true);
            delete this.dtable;
        }
    }
});

window.RevenueDashboardView = {
    initialize: function() {
        $(".dashboard-summary").before("<div style='opacity: 0.5; height:65px;' id='dashboard-rev'></div>");

		var self = this;
		if(this.template)
			return $.when(countlyRevenue.initialize()).then(function () {self.renderCommon();});
		else{
			return $.when($.get(countlyGlobal["path"]+'/revenue/templates/revenue-dashboard.html', function(src){
				self.template = Handlebars.compile(src);
			}), countlyRevenue.initialize()).then(function () {
                self.renderCommon();
            });
		}
    },
    renderCommon:function (isRefresh) {
		if (Backbone.history.fragment == "/" && countlyRevenue.hasIAPEvent()) {
			var revenueData = countlyRevenue.getRevenueData();
			this.templateData = {};
            this.templateData["revenue"] = [
                {
                    "title":jQuery.i18n.map["revenue.revenue"].toUpperCase(),
                    "subtitle":"",
                    "data":revenueData['revenue']
                },
                {
                    "title":jQuery.i18n.map["revenue.avg-revenue"].toUpperCase(),
                    "subtitle":jQuery.i18n.map["common.per-user"].toLowerCase(),
                    "data":revenueData['avg-revenue-per-user']
                },
                {
                    "title":jQuery.i18n.map["revenue.paying-total"].toUpperCase(),
                    "subtitle":jQuery.i18n.map["common.users"].toLowerCase(),
                    "data":revenueData['paying-total']
                }
            ];

            if(!isRefresh) {
                $("#dashboard-rev").remove();
                $(".dashboard-summary").before(this.template(this.templateData));
            }
        }
    },
    refresh:function () {
        var self = this;
        if (Backbone.history.fragment == "/" && countlyRevenue.hasIAPEvent()) {
            $.when(countlyRevenue.refresh()).then(function () {
                self.renderCommon(true);
                var newPage = $("<div>" + self.template(self.templateData) + "</div>");
                $(".rev-container").replaceWith(newPage.find(".rev-container"));
            });
        }
    }
};

//register views
app.revenueView = new RevenueView();

app.route("/revenue", "revenue", function () {
	this.renderWhenReady(this.revenueView);
});

app.addPageScript("/", function(){
    if(countlyGlobal["apps"][countlyCommon.ACTIVE_APP_ID].type == "mobile" || countlyGlobal["apps"][countlyCommon.ACTIVE_APP_ID].type == "web"){
        RevenueDashboardView.initialize();
    }
});

app.addRefreshScript("/", function(){
    if(countlyGlobal["apps"][countlyCommon.ACTIVE_APP_ID].type == "mobile" || countlyGlobal["apps"][countlyCommon.ACTIVE_APP_ID].type == "web"){
        RevenueDashboardView.refresh();
    }
});

app.addPageScript("/manage/apps", function(){
	var addApp = '<tr class="help-zone-vs appmng-revenue" data-help-localize="manage-apps.app-iap-event">'+
		'<td>'+
			'<span data-localize="management-applications.iap-event"></span>'+
			'<span><a href="http://resources.count.ly/v1.0/docs/revenue-analytics" target="_blank"><i class="fa fa-info revenue-help" style="margin-left:5px; color:#CCC;" data-help="revenue.tooltip"></i></a></span>'+
		'</td>'+
		'<td>'+
			'<input type="text" value="" placeholder="Enter IAP event key..." data-localize="placeholder.iap-event-key" id="app-add-iap-event">&nbsp;&nbsp;&nbsp;<span data-localize="placeholder.iap-help"></span>'+
		'</td>'+
	'</tr>';
	
	$("#add-new-app table .table-add").before(addApp);

	var editApp = '<tr class="help-zone-vs appmng-revenue" data-help-localize="manage-apps.app-iap-event">'+
		'<td>'+
			'<span data-localize="management-applications.iap-event"></span>'+
			'<span><a href="http://resources.count.ly/v1.0/docs/revenue-analytics" target="_blank"><i class="fa fa-info revenue-help" style="margin-left:5px; color:#CCC;" data-help="revenue.tooltip"></i></a></span>'+
		'</td>'+
		'<td id="app-edit-iap">'+
			'<div class="read"></div>'+
			'<div class="edit">'+
				'<input type="text" value="" placeholder="Enter IAP event key..." data-localize="placeholder.iap-event-key" id="app-edit-iap-event">&nbsp;&nbsp;&nbsp;<span data-localize="placeholder.iap-help"></span>'+
			'</div>'+
		'</td>'+
	'</tr>';
	
	$(".app-details table .table-edit").before(editApp);
    
    $('.revenue-help').tipsy({gravity:$.fn.tipsy.autoNS, title:function () {
                    return ($(this).data("help")) ? jQuery.i18n.map[$(this).data("help")] : "";
                }, fade:true, offset:5, cssClass:'yellow', opacity:1, html:true});
    
	var appId = countlyCommon.ACTIVE_APP_ID;
    if(!countlyGlobal["apps"][appId] || countlyGlobal["apps"][appId].type == "mobile" || countlyGlobal["apps"][appId].type == "web"){
        $(".appmng-revenue").show();
    } 
    else{
        $(".appmng-revenue").hide();
    }
    
	if(countlyGlobal['apps'][appId]){
		$("#app-edit-iap").find(".read").text(countlyGlobal['apps'][appId]["iap_event"] || "");
		$("#app-edit-iap").find(".edit input").val(countlyGlobal['apps'][appId]["iap_event"] || "");
	}
	
	$("#save-app-add").click(function () {
		var iapEvent = $("#app-add-iap-event").val();
		if(iapEvent && iapEvent.length > 0){
			$.ajax({
				type:"POST",
				url:"/events/iap",
				data:{
					"app_id":countlyCommon.ACTIVE_APP_ID,
					"event_key":iapEvent,
					_csrf:countlyGlobal['csrf_token']
				},
				success:function (result) {
					if (result) {
						if(countlyGlobal["admin_apps"][appId])
							countlyGlobal["admin_apps"][appId]["iap_event"] = iapEvent;
						if(countlyGlobal['apps'][appId])
							countlyGlobal['apps'][appId]["iap_event"] = iapEvent;
						$("#app-edit-iap").find(".read").text(iapEvent);
						$("#app-edit-iap").find(".edit input").val(iapEvent);
					}
				}
			});
		}
	});
	
	$("#save-app-edit").click(function () {
		var iapEvent = $("#app-edit-iap-event").val();
		$.ajax({
			type:"POST",
			url:countlyGlobal["path"]+"/events/iap",
			data:{
				"app_id":appId,
				"event_key":iapEvent,
				_csrf:countlyGlobal['csrf_token']
			},
			success:function (result) {
				if (result) {
					countlyGlobal["admin_apps"][appId]["iap_event"] = iapEvent;
					countlyGlobal['apps'][appId]["iap_event"] = iapEvent;
					$("#app-edit-iap").find(".read").text(iapEvent);
					$("#app-edit-iap").find(".edit input").val(iapEvent);
				}
			}
		});
	});
	app.addAppManagementSwitchCallback(function(appId, type){
        if(countlyGlobal['apps'][appId]){
            $("#app-edit-iap").find(".read").text(countlyGlobal['apps'][appId]["iap_event"] || "");
            $("#app-edit-iap").find(".edit input").val(countlyGlobal['apps'][appId]["iap_event"] || "");
        }
    });
});

app.addAppManagementSwitchCallback(function(appId, type){
    if(type == "mobile" || type == "web"){
        $(".appmng-revenue").show();
    } 
    else{
        $(".appmng-revenue").hide();
    }
});

app.addPageScript("/analytics/events", function(){
    if(countlyGlobal["apps"][countlyCommon.ACTIVE_APP_ID].type == "mobile" || countlyGlobal["apps"][countlyCommon.ACTIVE_APP_ID].type == "web"){
        $("#edit-events-button").on("click", function () {
            var first = true;
            $('.slimScrollDiv #edit-event-table-container table').find('tr').each(function(){
                if(first){
                    first = false;
                    $(this).find('th').eq(2).after('<th>IAP?</th>');
                }
                else
                    $(this).find('td').eq(2).after('<td class="iap-td" style="text-align:center;"><i class="fa fa-money iap-event" data-event-key="' + $(this).find('td').eq(2).find("i").data("event-key") + '"></i></td>');
            });
            
            var currIap = countlyGlobal["admin_apps"][countlyCommon.ACTIVE_APP_ID]["iap_event"];
            if (currIap) {
                $(".iap-event[data-event-key='" + currIap + "']").parent().addClass("active");
            }
            
            $("#edit-event-table-container table td.iap-td").on("click", function() {		
                if ($(this).hasClass("active")) {
                    $(this).removeClass("active");
                } else {
                    $(".iap-td").removeClass("active");
                    $(this).addClass("active");
                }
            });
            
            $(".dialog #events-save").on("click", function () {
                var iapEvent = $(".iap-td.active").find(".iap-event").data("event-key") || "";
                $.ajax({
                    type:"POST",
                    url:"/events/iap",
                    data:{
                        "app_id":countlyCommon.ACTIVE_APP_ID,
                        "event_key":iapEvent,
                        _csrf:countlyGlobal['csrf_token']
                    },
                    success:function (result) {
                        if (result) {
                            countlyGlobal["admin_apps"][countlyCommon.ACTIVE_APP_ID]["iap_event"] = iapEvent;
                            countlyGlobal['apps'][countlyCommon.ACTIVE_APP_ID]["iap_event"] = iapEvent;
                        }
                    }
                });
            });
        });
    }
});

app.addPageScript("#", function(){
	if (countlyRevenue.hasIAPEvent()) {
        $(".sidebar-menu #revenue-menu").removeClass("hide");
    } else {
        $(".sidebar-menu #revenue-menu").addClass("hide");
    }
});

$( document ).ready(function() {
	var menu = '<a href="#/revenue" class="item hide" id="revenue-menu">'+
        '<div class="logo ion-card"></div>'+
        '<div class="text" data-localize="revenue.iap"></div>'+
    '</a>';
	if($('.sidebar-menu:not(#iot-type) #management-menu').length)
		$('.sidebar-menu:not(#iot-type) #management-menu').before(menu);
	else
		$('.sidebar-menu:not(#iot-type)').append(menu);
});