var request = require('supertest');
var should = require('should');
var testUtils = require("../../test/testUtils");
request = request.agent(testUtils.url);

var APP_KEY = "";
var API_KEY_ADMIN = "";
var APP_ID = "";
var DEVICE_ID = "1234567890";

describe('Testing Revenue', function(){
    describe('verify no paying users', function(){
		it('should have 0 user', function(done){
            API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=users')
			.expect(200)
			.end(function(err, res){
				res.text.should.eql("{}");
                setTimeout(done, 1000);
			});
		});
	});
	describe('Setting iap event', function(){
		before(function( done ){
			testUtils.login( request );
			testUtils.waitLogin( done );
		});
		it('should have save iap', function(done){
			request
			.post("/events/iap")
			.send({app_id:APP_ID, event_key:"Purchase", _csrf:testUtils.getCSRF()})
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				setTimeout(done, 100)
			});
		});
	});
	describe('creating iap event', function(){
		it('should success', function(done){
			var params = [{
				"key": "Purchase",
				"count": 1,
				"sum": 1.5
			}];
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&events="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
	describe('verify paying users', function(){
		it('should have 1 user', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=users')
			.expect(200)
			.end(function(err, res){
				testUtils.validateSessionData(err, res, done, {meta:{}, e:1, p:1});
			});
		});
	});
	describe('reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
    describe('Login out', function(){
		it('should redirect to login', function(done){
			request
			.get('/logout')
			.expect('location', '/login')
			.expect(302);
            testUtils.logout();
            setTimeout(done, 5000);
		})
	});
});