var request = require('supertest');
var should = require('should');
var crypto= require('crypto');
var path = require("path");
var testUtils = require("../../../test/testUtils");
request = request(testUtils.url);

var APP_KEY = "";
var API_KEY_ADMIN = "";
var APP_ID = "";
var DEVICE_ID = "1234567890";

describe('Testing Users profiles', function(){
	//{"iTotalRecords":0,"iTotalDisplayRecords":0,"aaData":[]}
	describe('Empty users', function(){
		it('should have no users', function(done){
			API_KEY_ADMIN = testUtils.get("API_KEY_ADMIN");
			APP_ID = testUtils.get("APP_ID");
			APP_KEY = testUtils.get("APP_KEY");
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("iTotalRecords", 0);
				ob.should.have.property("iTotalDisplayRecords", 0);
				ob.should.have.property("aaData", []);
				setTimeout(done, 100)
			});
		});
	});
	describe('New user', function(){
		it('should success', function(done){
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&begin_session=1")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 500)
			});
		});
	});
	//{"iTotalRecords":1,"iTotalDisplayRecords":1,"aaData":[{"_id":"0c4c6a8a57a6110ad4547f785686079f4cd298d6","sc":1,"ls":1421426421,"cc":"Unknown","cty":"Unknown","picture":"/users/images/default_user.png"}]}
	describe('Verify user', function(){
		it('should have user', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("iTotalRecords", 1);
				ob.should.have.property("iTotalDisplayRecords", 1);
				ob.should.have.property("aaData");
				ob.aaData[0].should.have.property("_id", crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'));
				ob.aaData[0].should.have.property("sc", 1);
				ob.aaData[0].should.have.property("cc", "Unknown");
				ob.aaData[0].should.have.property("cty", "Unknown");
				ob.aaData[0].should.have.property("picture", "/users/images/users/default_user.png");
				ob.aaData[0].should.have.property("ls");
				setTimeout(done, 100);
			});
		});
	});
	describe('Set user data', function(){
		it('should success', function(done){
			var params = {"name": "Test1", "username": "test", "email": "test@test.com", "organization": "Countly", "phone": "12345678", "picture": "https://pbs.twimg.com/profile_images/1442562237/012_n_400x400.jpg", "gender": "M", "byear": "1980", "custom":{"key1":"value1", "key2":"value2"}};
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&user_details="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify all user data', function(){
		it('should have user data', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("iTotalRecords", 1);
				ob.should.have.property("iTotalDisplayRecords", 1);
				ob.should.have.property("aaData");
				ob.aaData.should.have.length(1);
				ob.aaData[0].should.have.property("_id", crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'));
				ob.aaData[0].should.have.property("sc", 1);
				ob.aaData[0].should.have.property("cc", "Unknown");
				ob.aaData[0].should.have.property("cty", "Unknown");
				ob.aaData[0].should.have.property("name", "Test1");
				ob.aaData[0].should.have.property("email", "test@test.com");
				ob.aaData[0].should.have.property("hasInfo", true);
				ob.aaData[0].should.have.property("picture", "https://pbs.twimg.com/profile_images/1442562237/012_n_400x400.jpg");
				ob.aaData[0].should.have.property("ls");
				setTimeout(done, 100);
			});
		});
	});
	describe('Verify specific user data', function(){
		it('should have user data', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details&uid='+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("_id", crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'));
				ob.should.have.property("sc", 1);
				ob.should.have.property("did", DEVICE_ID);
				ob.should.have.property("cc", "Unknown");
				ob.should.have.property("cty", "Unknown");
				ob.should.have.property("name", "Test1");
				ob.should.have.property("username", "test");
				ob.should.have.property("email", "test@test.com");
				ob.should.have.property("organization", "Countly");
				ob.should.have.property("phone", "12345678");
				ob.should.have.property("hasInfo", true);
				ob.should.have.property("gender", "M");
				ob.should.have.property("byear", "1980");
				ob.should.have.property("custom",{"key1":"value1", "key2":"value2"});
				ob.should.have.property("picture", "https://pbs.twimg.com/profile_images/1442562237/012_n_400x400.jpg");
				ob.should.have.property("ls");
				ob.should.have.property("fs");
                ob.should.have.property("sessions").and.not.empty;
                ob.sessions.should.have.length(1);
				ob.sessions[0].should.have.property("_id");
				ob.sessions[0].should.have.property("nts");
				ob.sessions[0].should.have.property("sd", 0);
				ob.sessions[0].should.have.property("ts");
				ob.should.have.property("events",[]);
				ob.should.have.property("funnels",[]);
				setTimeout(done, 100);
			});
		});
	});
	describe('Change user data', function(){
		it('should success', function(done){
			var params = {"name": "Test2", "username": "test2", "email": "test2@test.com", "organization": "Testly", "phone": "87654321", "picture": "", "gender": "F", "byear": "1990", "custom":{"key1":"value", "key2":"value", "key3":"value", "forinc":1, "formul":2, "formax":2, "formin":1}};
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&user_details="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify specific user data', function(){
		it('should have changed user data', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details&uid='+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("_id", crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'));
				ob.should.have.property("sc", 1);
				ob.should.have.property("did", DEVICE_ID);
				ob.should.have.property("cc", "Unknown");
				ob.should.have.property("cty", "Unknown");
				ob.should.have.property("name", "Test2");
				ob.should.have.property("username", "test2");
				ob.should.have.property("email", "test2@test.com");
				ob.should.have.property("organization", "Testly");
				ob.should.have.property("phone", "87654321");
				ob.should.have.property("hasInfo", true);
				ob.should.have.property("gender", "F");
				ob.should.have.property("byear", "1990");
				ob.should.have.property("custom",{"key1":"value", "key2":"value", "key3":"value", "forinc":1, "formul":2, "formax":2, "formin":1});
				ob.should.have.property("picture", "https://www.gravatar.com/avatar/f2c97b1f2d2898cd2d6466ce95d4ba33?d=identicon&amp;s=150");
				ob.should.have.property("ls");
				ob.should.have.property("fs");
				ob.should.have.property("sessions").and.not.empty;
                ob.sessions.should.have.length(1);
				ob.sessions[0].should.have.property("_id");
				ob.sessions[0].should.have.property("nts");
				ob.sessions[0].should.have.property("sd", 0);
				ob.sessions[0].should.have.property("ts");
				ob.should.have.property("events",[]);
				ob.should.have.property("funnels",[]);
				setTimeout(done, 100);
			});
		});
	});
    describe('Use modifiers on user data', function(){
		it('should success', function(done){
			var params = {"custom":{"key1":{"$push":"value1"}, "forinc":{"$inc":1}, "formul":{"$mul":2}, "formax":{"$max":1}, "formin":{"$min":0}, "set":{"$addToSet":["val1","val2"]}}};
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&user_details="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
	describe('Verify specific user data', function(){
		it('should have changed user data', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details&uid='+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("_id", crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'));
				ob.should.have.property("sc", 1);
				ob.should.have.property("did", DEVICE_ID);
				ob.should.have.property("cc", "Unknown");
				ob.should.have.property("cty", "Unknown");
				ob.should.have.property("name", "Test2");
				ob.should.have.property("username", "test2");
				ob.should.have.property("email", "test2@test.com");
				ob.should.have.property("organization", "Testly");
				ob.should.have.property("phone", "87654321");
				ob.should.have.property("hasInfo", true);
				ob.should.have.property("gender", "F");
				ob.should.have.property("byear", "1990");
				ob.should.have.property("custom",{"key1":["value","value1"], "key2":"value", "key3":"value", "forinc":2, "formul":4, "formax":2, "formin":0, "set":["val1","val2"]});
				ob.should.have.property("picture", "https://www.gravatar.com/avatar/f2c97b1f2d2898cd2d6466ce95d4ba33?d=identicon&amp;s=150");
				ob.should.have.property("ls");
				ob.should.have.property("fs");
				ob.should.have.property("sessions").and.not.empty;
                ob.sessions.should.have.length(1);
				ob.sessions[0].should.have.property("_id");
				ob.sessions[0].should.have.property("nts");
				ob.sessions[0].should.have.property("sd", 0);
				ob.sessions[0].should.have.property("ts");
				ob.should.have.property("events",[]);
				ob.should.have.property("funnels",[]);
				setTimeout(done, 100);
			});
		});
	});
    describe('Use more modifiers on user data', function(){
		it('should success', function(done){
			var params = {"custom":{"forinc":{"$inc":1}, "formul":{"$mul":2}, "formax":{"$max":3}, "formin":{"$min":1}, "set":{"$pull":"val1"}}};
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&user_details="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 100)
			});
		});
	});
    describe('Verify specific user data', function(){
		it('should have changed user data', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details&uid='+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("_id", crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'));
				ob.should.have.property("sc", 1);
				ob.should.have.property("did", DEVICE_ID);
				ob.should.have.property("cc", "Unknown");
				ob.should.have.property("cty", "Unknown");
				ob.should.have.property("name", "Test2");
				ob.should.have.property("username", "test2");
				ob.should.have.property("email", "test2@test.com");
				ob.should.have.property("organization", "Testly");
				ob.should.have.property("phone", "87654321");
				ob.should.have.property("hasInfo", true);
				ob.should.have.property("gender", "F");
				ob.should.have.property("byear", "1990");
				ob.should.have.property("custom",{"key1":["value","value1"], "key2":"value", "key3":"value", "forinc":3, "formul":8, "formax":3, "formin":0, "set":["val2"]});
				ob.should.have.property("picture", "https://www.gravatar.com/avatar/f2c97b1f2d2898cd2d6466ce95d4ba33?d=identicon&amp;s=150");
				ob.should.have.property("ls");
				ob.should.have.property("fs");
				ob.should.have.property("sessions").and.not.empty;
                ob.sessions.should.have.length(1);
				ob.sessions[0].should.have.property("_id");
				ob.sessions[0].should.have.property("nts");
				ob.sessions[0].should.have.property("sd", 0);
				ob.sessions[0].should.have.property("ts");
				ob.should.have.property("events",[]);
				ob.should.have.property("funnels",[]);
				setTimeout(done, 100);
			});
		});
	});
	describe('Upload picture', function(){
		it('should success', function(done){
			var dir = path.resolve(__dirname, 'test.png');
			request
			.post('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&user_details")
			.attach('avatar', dir)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify image', function(){
		it('should be accessible', function(done){
			request
			.get("/userimages/"+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex')+".png")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				setTimeout(done, 100);
			});
		});
	});
	describe('Verify upload', function(){
		it('should have uploaded file', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details&uid='+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("picture", "/userimages/"+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex')+".png");
				setTimeout(done, 5000);
			});
		});
	});
	describe('Add session and event', function(){
		it('should success', function(done){
			this.timeout(20000);
			var params = [{
				"key": "test",
				"count": 1
			}];
			request
			.get('/i?device_id='+DEVICE_ID+'&app_key='+APP_KEY+"&end_session=1&events="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 15000)
			});
		});
	});
	describe('Verify session and event', function(){
		it('should have session and event', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details&uid='+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex'))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				// { _id: '54bcd0bf05b308b628d7f747', nts: '', sd: 0, ts: 1421660346000 }
				ob.should.have.property("sessions").and.not.empty;
				ob.sessions[0].should.have.property("_id");
				ob.sessions[0].should.have.property("nts");
				ob.sessions[0].should.have.property("sd", 0);
				ob.sessions[0].should.have.property("ts");
				//[{_id: '54b969926e52798c484b3024',c: 1,key: 'test',s: 0,sg: {},ts: 1421437330000}]
				ob.should.have.property("events").and.not.empty;
				ob.events[0].should.have.property("_id");
				ob.events[0].should.have.property("c", 1);
				ob.events[0].should.have.property("key", 'test');
				ob.events[0].should.have.property("s", 0);
				ob.events[0].should.have.property("sg", {});
				ob.events[0].should.have.property("ts");
				setTimeout(done, 100);
			});
		});
	});
	describe('Upload picture for new user', function(){
		it('should success', function(done){
			var dir = path.resolve(__dirname, 'test.png');
			request
			.post('/i?device_id='+DEVICE_ID+'a&app_key='+APP_KEY+"&user_details")
			.attach('avatar', dir)
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result','Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify image', function(){
		it('should be accessible', function(done){
			request
			.get("/userimages/"+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "a").digest('hex')+".png")
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				setTimeout(done, 100);
			});
		});
	});
	describe('Verify upload', function(){
		it('should have uploaded file', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details&uid='+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "a").digest('hex'))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("picture", "/userimages/"+crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "a").digest('hex')+".png");
				setTimeout(done, 100);
			});
		});
	});
	describe('Verify two users', function(){
		it('should have user', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("iTotalRecords", 2);
				ob.should.have.property("iTotalDisplayRecords", 2);
				ob.should.have.property("aaData");
				ob.aaData.should.have.length(2);
				setTimeout(done, 100);
			});
		});
	});
	describe('write bulk users', function(){
		it('should success', function(done){
			this.timeout(20000);
			var params = [
				{"device_id":DEVICE_ID+"1", "app_key":APP_KEY, "user_details":{"name": "Test1", "username": "test1", "email": "test1@test.com"}},
				{"device_id":DEVICE_ID+"2", "app_key":APP_KEY, "user_details":{"name": "Test2", "username": "test2", "email": "test2@test.com"}},
				{"device_id":DEVICE_ID+"3", "app_key":APP_KEY, "user_details":{"name": "Test3", "username": "test3", "email": "test3@test.com"}}
			];
			request
			.get('/i/bulk?requests='+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 1500)
			});
		});
	});
	describe('Verify bulk users', function(){
		it('should have user', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("iTotalRecords", 5);
				ob.should.have.property("iTotalDisplayRecords", 5);
				ob.should.have.property("aaData");
				ob.aaData.should.have.length(5);
                
                var id1 = crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "").digest('hex');
                var id2 = crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "a").digest('hex');
                var id3 = crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "1").digest('hex');
                var id4 = crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "2").digest('hex');
                var id5 = crypto.createHash('sha1').update(APP_KEY + DEVICE_ID + "3").digest('hex');
                for(var i = 0; i < ob.aaData.length; i++){
                    if(ob.aaData[i]._id == id1){
                        ob.aaData[i].should.have.property("_id", id1);
                        ob.aaData[i].should.have.property("sc", 1);
                        ob.aaData[i].should.have.property("cc", "Unknown");
                        ob.aaData[i].should.have.property("cty", "Unknown");
                        ob.aaData[i].should.have.property("name", "Test2");
                        ob.aaData[i].should.have.property("email", "test2@test.com");
                        ob.aaData[i].should.have.property("hasInfo", true);
                        ob.aaData[i].should.have.property("picture", "/userimages/"+id1+".png");
                        ob.aaData[i].should.have.property("ls");
                    }
                    else if(ob.aaData[i]._id == id2){
                        ob.aaData[i].should.have.property("_id", id2);
                        ob.aaData[i].should.have.property("picture", "/userimages/"+id2+".png");
                    }
                    else if(ob.aaData[i]._id == id3){
                        ob.aaData[i].should.have.property("_id", id3);
                        ob.aaData[i].should.have.property("name", "Test1");
                        ob.aaData[i].should.have.property("email", "test1@test.com");
                        ob.aaData[i].should.have.property("hasInfo", true);
                        ob.aaData[i].should.have.property("picture", "https://www.gravatar.com/avatar/94fba03762323f286d7c3ca9e001c541?d=identicon&amp;s=150");
                    }
                    else if(ob.aaData[i]._id == id4){
                        ob.aaData[i].should.have.property("_id", id4);
                        ob.aaData[i].should.have.property("name", "Test2");
                        ob.aaData[i].should.have.property("email", "test2@test.com");
                        ob.aaData[i].should.have.property("hasInfo", true);
                        ob.aaData[i].should.have.property("picture", "https://www.gravatar.com/avatar/f2c97b1f2d2898cd2d6466ce95d4ba33?d=identicon&amp;s=150");
                    }
                    else if(ob.aaData[i]._id == id5){
                        ob.aaData[i].should.have.property("_id", id5);
                        ob.aaData[i].should.have.property("name", "Test3");
                        ob.aaData[i].should.have.property("email", "test3@test.com");
                        ob.aaData[i].should.have.property("hasInfo", true);
                        ob.aaData[i].should.have.property("picture", "https://www.gravatar.com/avatar/59c02e9fd89cfb6fa54b67e846d0631b?d=identicon&amp;s=150");
                    }
                }
				
				setTimeout(done, 100);
			});
		});
	});
	describe('reset app', function(){
		it('should reset data', function(done){
			var params = {app_id:APP_ID};
			request
			.get('/i/apps/reset?api_key='+API_KEY_ADMIN+"&args="+JSON.stringify(params))
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.have.property('result', 'Success');
				setTimeout(done, 5000)
			});
		});
	});
	describe('Verify empty users', function(){
		it('should have no users', function(done){
			request
			.get('/o?api_key='+API_KEY_ADMIN+'&app_id='+APP_ID+'&method=user_details')
			.expect(200)
			.end(function(err, res){
				if (err) return done(err);
				var ob = JSON.parse(res.text);
				ob.should.not.be.empty;
				ob.should.have.property("iTotalRecords", 0);
				ob.should.have.property("iTotalDisplayRecords", 0);
				ob.should.have.property("aaData", []);
				setTimeout(done, 100)
			});
		});
	});
});